@@ -2,7 +2,7 @@
  *
  * c h a r . c                          -- Characters management
  *
- * Copyright © 1993-2020 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,16 +23,11 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??????
- * Last file update:  2-Jun-2020 10:47 (eg)
  */
 
 #include <ctype.h>
 #include "stklos.h"
 
-
-SCM STk_char_foldcase(SCM c);
-
-
 struct charelem {
   char *name;
   unsigned char value;
@@ -95,12 +90,10 @@ static struct charelem chartable [] = {
 };
 
 struct utf8_conversion_char {
-  uint32_t key;
-  uint32_t val;
+  utf8_char key;
+  utf8_char val;
 };
 
-typedef uint32_t utf8_char;
-
 #include "utf8-tables.inc"
 
 static int search_conversion_table(unsigned int ch, struct utf8_conversion_char table[],
@@ -154,6 +147,11 @@ static int search_ordered_list(utf8_char ch,  utf8_char table[], int len) {
   return -1;
 }
 
+// Public version of search_ordered_list
+int STk_valid_utf8_char_codep(utf8_char ch, utf8_char table[], int len)
+{
+  return search_ordered_list(ch, table, len);
+}
 
 /*===========================================================================*\
  *
@@ -163,10 +161,10 @@ static int search_ordered_list(utf8_char ch,  utf8_char table[], int len) {
 
 static void error_bad_char(SCM c)
 {
-  STk_error("bad char", c);
+  STk_error("bad char ~S", c);
 }
 
-static Inline int charcomp(SCM c1, SCM c2)
+static inline int charcomp(SCM c1, SCM c2)
 {
   return (CHARACTER_VAL(c1) - CHARACTER_VAL(c2));
 }
@@ -181,18 +179,14 @@ static int charcompi(SCM c1, SCM c2)
            tolower((unsigned char) CHARACTER_VAL(c2)));
 }
 
-/* Comparison of characters. No test on types */
-int STk_charcomp(SCM c1, SCM c2)  { return charcomp(c1,  c2);  }
-int STk_charcompi(SCM c1, SCM c2) { return charcompi(c1,  c2); }
-
 
 int STk_string2char(char *s)
 /* converts a char name to a char */
 {
   register struct charelem *p;
-  uint32_t val;
+  utf8_char val;
 
-  /* Try to see if it is a multi-byte character */
+  /* Try to see if it is a multibyte character */
   if (* (STk_utf8_grab_char(s, &val)) == '\0') return val;
 
   if (*s == 'x') {
@@ -254,12 +248,12 @@ doc>
  *
  * These procedures impose a total ordering on the set of characters.
  * It is guaranteed that under this ordering:
- * ,(itemize
- * (item [The upper case characters are in order.])
- * (item [The lower case characters are in order.])
- * (item [The digits are in order.])
- * (item [Either all the digits precede all the upper case letters, or vice versa.])
- * (item [Either all the digits precede all the lower case letters, or vice versa.])
+ *
+ * - The upper case characters are in order.
+ * - The lower case characters are in order.
+ * - The digits are in order.
+ * - Either all the digits precede all the upper case letters, or vice versa.
+ * - Either all the digits precede all the lower case letters, or vice versa.
  * )
 doc>
  */
@@ -388,7 +382,7 @@ DEFINE_PRIMITIVE("char-lower-case?", char_islower, subr1, (SCM c)) {
  * (digit-value #\3)        => 3
  * (digit-value #\x0664)    => 4
  * (digit-value #\x0AE6)    => 0
- * (digit-value #\x0EA6)    #f
+ * (digit-value #\x0EA6)    => #f
  * @end lisp
 doc>
  */
@@ -422,7 +416,7 @@ DEFINE_PRIMITIVE("digit-value", digit_value, subr1, (SCM c))
  * ordering and some subset of the integers under the |<=|
  * ordering. That is, if
  * @lisp
- *    (char<=? a b) => #t  ,(bold "and")  (<= x y) => #t
+ *    (char<=? a b) => #t  and  (<= x y) => #t
  * @end lisp
  * and x and y are in the domain of |integer->char|, then
  * @lisp
@@ -433,8 +427,8 @@ DEFINE_PRIMITIVE("digit-value", digit_value, subr1, (SCM c))
  *            (integer->char y))     =>  #t
  * @end lisp
  * |integer->char| accepts an exact number between 0 and #xD7FFF or between
- * #xE000 and #x10FFFF, if UTF8 encoding is used. Otherwise it accepts a
- * number between0 and #xFF.
+ * #xE000 and #x10FFFF, if UTF8 encoding is used. Otherwise, it accepts a
+ * number between 0 and #xFF.
 doc>
  */
 DEFINE_PRIMITIVE("char->integer", char2integer, subr1, (SCM c))
@@ -446,7 +440,7 @@ DEFINE_PRIMITIVE("char->integer", char2integer, subr1, (SCM c))
 
 DEFINE_PRIMITIVE("integer->char", integer2char, subr1, (SCM i))
 {
-  int c = STk_integer_value(i);
+  long c = STk_integer_value(i);
 
   if (STk_use_utf8) {
     /* Unicode defines characters in the range [0, #xd7FF] U [#xE000, #x10FFFF] */
@@ -475,18 +469,18 @@ DEFINE_PRIMITIVE("integer->char", integer2char, subr1, (SCM i))
  * lower case.
 doc>
  */
-uint32_t STk_to_upper(uint32_t c) {
+utf8_char STk_to_upper(utf8_char c) {
   if (STk_use_utf8) {
     int res = search_conversion_table(c, lower_table, lower_table_length);
-    return (res <= 0) ? c : (uint32_t) res;   // -1: not a lowercase, 0 lower without upper
+    return (res <= 0) ? c : (utf8_char) res;   // -1: not a lowercase, 0 lower without upper
   } else
     return toupper(c);
 }
 
-uint32_t STk_to_lower(uint32_t c) {
+utf8_char STk_to_lower(utf8_char c) {
   if (STk_use_utf8) {
     int res = search_conversion_table(c, upper_table, upper_table_length);
-    return (res <=0) ? c : (uint32_t) res;    // -1: not a lowercase, 0 lower without lower
+    return (res <=0) ? c : (utf8_char) res;    // -1: not a lowercase, 0 lower without lower
   } else
     return tolower(c);
 }
@@ -494,14 +488,14 @@ uint32_t STk_to_lower(uint32_t c) {
 DEFINE_PRIMITIVE("char-upcase", char_upcase, subr1, (SCM c))
 {
   if (!CHARACTERP(c)) error_bad_char(c);
-  return MAKE_CHARACTER(STk_to_upper((uint32_t) CHARACTER_VAL(c)));
+  return MAKE_CHARACTER(STk_to_upper((utf8_char) CHARACTER_VAL(c)));
 }
 
 
 DEFINE_PRIMITIVE("char-downcase", char_downcase, subr1, (SCM c))
 {
   if (!CHARACTERP(c)) error_bad_char(c);
-  return MAKE_CHARACTER(STk_to_lower((uint32_t) CHARACTER_VAL(c)));
+  return MAKE_CHARACTER(STk_to_lower((utf8_char) CHARACTER_VAL(c)));
 }
 /*
 <doc EXT char-foldcase
@@ -514,10 +508,10 @@ DEFINE_PRIMITIVE("char-downcase", char_downcase, subr1, (SCM c))
  * does not exist.
 doc>
  */
-uint32_t STk_to_fold(uint32_t c) {
+utf8_char STk_to_fold(utf8_char c) {
   if (STk_use_utf8) {
     int res = search_conversion_table(c, fold_table, fold_table_length);
-    return (res <=0) ? STk_to_lower(c) : (uint32_t) res; 
+    return (res <=0) ? STk_to_lower(c) : (utf8_char) res;
   } else
     return tolower(c);
 }
@@ -525,7 +519,57 @@ uint32_t STk_to_fold(uint32_t c) {
 DEFINE_PRIMITIVE("char-foldcase", char_foldcase, subr1, (SCM c))
 {
   if (!CHARACTERP(c))  error_bad_char(c);
-  return MAKE_CHARACTER(STk_to_fold((uint32_t) CHARACTER_VAL(c)));
+  return MAKE_CHARACTER(STk_to_fold((utf8_char) CHARACTER_VAL(c)));
+}
+
+/* ----------------------------------------------------------------------
+ * UTF8 support for char-sets 
+ * ---------------------------------------------------------------------- */
+
+static inline SCM make_char_list1(struct utf8_conversion_char *tab, int len)
+{
+  SCM lst = STk_nil;
+
+  for (int i = len-1; i >=0; i--) {
+    lst = STk_cons(MAKE_CHARACTER(tab[i].key), lst);
+  }
+  return lst;
+}
+
+static inline SCM make_char_list2(utf8_char *tab, int len)
+{
+  SCM lst = STk_nil;
+
+  for (int i = len-1; i >=0; i--) {
+    lst = STk_cons(MAKE_CHARACTER(tab[i]), lst);
+  }
+  return lst;
+}
+
+
+DEFINE_PRIMITIVE("%uppers-list", uppers_list, subr0, (void))
+{
+  return make_char_list1(upper_table, upper_table_length);
+}
+
+DEFINE_PRIMITIVE("%lowers-list", lowers_list, subr0, (void))
+{
+  return make_char_list1(lower_table, lower_table_length);
+}
+
+DEFINE_PRIMITIVE("%digits-list", digits_list, subr0, (void))
+{
+  return make_char_list1(digits_table, digits_table_length);
+}
+
+DEFINE_PRIMITIVE("%letters-list", letters_list, subr0, (void))
+{
+  return make_char_list2(letters_table, letters_table_length);
+}
+
+DEFINE_PRIMITIVE("%spaces-list", spaces_list, subr0, (void))
+{
+  return make_char_list2(spaces_table, spaces_table_length);
 }
 
 
@@ -561,5 +605,11 @@ int STk_init_char(void)
   ADD_PRIMITIVE(char_downcase);
   ADD_PRIMITIVE(char_foldcase);
 
+  /* Support for char-sets */
+  ADD_PRIMITIVE(uppers_list);
+  ADD_PRIMITIVE(lowers_list);
+  ADD_PRIMITIVE(digits_list);
+  ADD_PRIMITIVE(letters_list);
+  ADD_PRIMITIVE(digits_list);
   return TRUE;
 }
