@@ -1,7 +1,7 @@
 /*
  * f p o r t . c                                -- File ports
  *
- * Copyright © 2000-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2000-2024 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,19 +14,18 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should hcave received a copy of the GNU General Public License
+ * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  * USA.
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  8-Jan-2000 14:48 (eg)
- * Last file update: 28-Aug-2021 11:45 (eg)
  *
- * This implementation is built by reverse engineering on an old SUNOS 4.1.1
+ * This implementation is built by reverse engineering an old SUNOS 4.1.1
  * stdio.h. It has been simplified to fit the needs for STklos. In particular
- * non buffered file are not implemented. Anyway this is faster than an
- * implementation using the C buffered IO (at least on glibc)
+ * non-buffered files are not implemented. Anyway this is faster than an
+ * implementation using the C buffered IO (at least on glibc).
  *
  */
 #include <ctype.h>
@@ -40,17 +39,17 @@
 
 int STk_interactive = 0;                  /* We are in interactive mode */
 SCM STk_stdin, STk_stdout, STk_stderr;    /* The unredirected ports */
-MUT_DECL(all_fports_mutex)
+MUT_DECL(all_fports_mutex);
 
 /*
- * Implementation of our own popen/pclose. We use her file descriptor, instad
+ * Implementation of our own popen/pclose. We use its file descriptor instead
  * of FILE *.
  *
- * The fd_popen function returns also the pid of the shell process launched
+ * The fd_popen function also returns the pid of the shell process launched
  * for the redirection. This pid is stored in the internal file port
- * representation to wait on it during the fd_pclose.
+ * representation to wait on it during fd_pclose.
  */
-static int fd_popen(char *cmd, char *mode, int *pid)
+static int fd_popen(const char *cmd, const char *mode, int *pid)
 {
   int p[2];
 
@@ -74,7 +73,7 @@ static int fd_popen(char *cmd, char *mode, int *pid)
       execlp("sh", "sh", "-c", cmd, NULL);
       STk_panic("*** shell error in fd_popen");
       return -1; /* for the compiler */
-    default:                                 /* Parent process*/
+    default:                                 /* Parent process */
       if (*mode == 'r') {
         close(p[1]);
         return p[0];
@@ -96,8 +95,8 @@ static int fd_pclose(int fd, int pid) {
 
 
 /*
- * Since we manage ourselves our ports, we need to keep a reference on
- * all the open ports such that all the port are flushed when we exit
+ * Since we manage our ports ourselves, we need to keep a reference to
+ * all the open ports such that all the ports are flushed when we exit
  * from the program. However, to avoid the that the GC "sees" these
  * references, we need to use a list of objects containing "masqueraded"
  * pointers.
@@ -115,7 +114,7 @@ static struct port_list {
 static void nop_release_port(SCM _UNUSED(port))
 {
   /* This function is used as a release procedure when closing all files
-   * at program exit. We use an empty procedure to avoid interferences
+   * at program exit. We use an empty procedure to avoid interference
    * in all_file_ports management
    */
 }
@@ -166,7 +165,7 @@ void STk_close_all_ports(void)
   }
   MUT_UNLOCK(all_fports_mutex);
 
-  /* Finally close error and output port (must be done last) */
+  /* Finally, close error and output ports (must be done last) */
   STk_close(eport);
   STk_close(oport);
 }
@@ -229,13 +228,13 @@ static int flush_buffer(struct fstream *f)
 /*=============================================================================*/
 
 
-static Inline int Feof(void *stream)
+static inline int Feof(void *stream)
 {
   return PORT_STREAM_FLAGS(stream) & STK_IOEOF;
 }
 
 
-static Inline int Freadyp(void *stream)
+static inline int Freadyp(void *stream)
 {
   if (PORT_CNT(stream) > 0)
     return TRUE;
@@ -251,7 +250,7 @@ static Inline int Freadyp(void *stream)
   }
 }
 
-static Inline int Fgetc(void *stream)
+static inline int Fgetc(void *stream)
 {
   if (PORT_IDLE(stream) != STk_nil) {
     /* There is at least an idle handler */
@@ -273,7 +272,7 @@ static Inline int Fgetc(void *stream)
 }
 
 
-static Inline int Fread(void *stream, void *buff, int count)
+static inline int Fread(void *stream, void *buff, int count)
 {
   int copied, avail;
 
@@ -323,7 +322,7 @@ static off_t Fseek(void *stream, off_t offset, int whence)
 }
 
 
-static Inline int Fputc(int c, void *stream)
+static inline int Fputc(int c, void *stream)
 {
   int ret = c;
 
@@ -347,7 +346,7 @@ static Inline int Fputc(int c, void *stream)
 }
 
 
-static Inline int Fwrite(void *stream, void *buff, int count)
+static inline int Fwrite(void *stream, const void *buff, int count)
 {
   /* Flush (eventually) chars which are already in the buffer before writing  */
   if (PORT_CNT(stream)) {
@@ -363,12 +362,12 @@ static Inline int Fwrite(void *stream, void *buff, int count)
 }
 
 
-static Inline int Fnputs(void *stream, char *s, int len)
+static inline int Fnputs(void *stream, const char *s, int len)
 {
   int res, flush = (PORT_STREAM_FLAGS(stream) & STK_IOLBF);
 
   if (len > TTY_BUFSIZE) {
-    /* This is a long string don't use the buffer and write it directly on stream */
+    /* This is a long string so don't use the buffer; write it directly to the stream */
     res = Fwrite(stream, s, len);
   } else {
     int free, count = len;
@@ -403,18 +402,18 @@ static Inline int Fnputs(void *stream, char *s, int len)
 }
 
 
-static Inline int Fputs(char *s, void *stream)
+static inline int Fputs(const char *s, void *stream)
 {
   return Fnputs(stream, s, strlen(s));
 }
 
-static Inline int Fputstring(SCM s, void *stream)
+static inline int Fputstring(SCM s, void *stream)
 {
   return Fnputs(stream, STRING_CHARS(s), STRING_SIZE(s));
 }
 
 
-static Inline int  Fflush(void *stream)
+static inline int  Fflush(void *stream)
 {
   return flush_buffer(stream);          /* Problem if file opened for reading */
 }
@@ -441,7 +440,7 @@ static void fport_finalizer(struct port_obj *port, void _UNUSED(*client_data))
 
 
 static struct port_obj *
-make_fport(char *fname, int fd, int flags)
+make_fport(const char *fname, int fd, int flags)
 {
   struct fstream  *fs = STk_must_malloc(sizeof(struct fstream));
   int n, mode;
@@ -460,8 +459,8 @@ make_fport(char *fname, int fd, int flags)
   /* keep the indication that file is opened in read in the stream part */
   if (flags & (PORT_READ | PORT_RW)) mode |= STK_IOREAD;
 
-  /* Set the case sensitive bit */
-  if (STk_read_case_sensitive) flags |= PORT_CASE_SENSITIVE;
+  /* Set the case-sensitive bit */
+  flags |= PORT_CASE_SENSITIVE;
 
   /* Initialize the stream part */
   PORT_BASE(fs)          = STk_must_malloc_atomic(n);
@@ -475,7 +474,7 @@ make_fport(char *fname, int fd, int flags)
   PORT_WEVENT(fs)        = STk_false;
   PORT_IDLE(fs)          = STk_nil;
 
-  /* Initialize now the port itsef */
+  /* Now initialize the port itself */
   NEWCELL(res, port);
 
   PORT_STREAM(res)      = fs;
@@ -515,9 +514,9 @@ make_fport(char *fname, int fd, int flags)
 #ifdef WIN32_0000
 static char *convert_for_win32(char *mode)
 {
-  /* Things are complicated on Win32 (as always). So we onvert all files
-   * in binaries files. Note that this function is not called when we work
-   * on ports since they only accept version without "b" on Cygwin
+  /* Things are complicated on Win32 (as always) so we convert all files
+   * to binary files. Note that this function is not called when we work
+   * on ports since they only accept versions without "b" on Cygwin.
    */
   switch (*mode) {
     case 'r': if (mode[1] == '\0') return "rb";
@@ -534,7 +533,7 @@ static char *convert_for_win32(char *mode)
 }
 #endif
 
-static int convert_mode(char* mode) {
+static int convert_mode(const char *mode) {
   char first = *mode;
 
   if (mode[1] == 'b')
@@ -561,9 +560,9 @@ static int convert_mode(char* mode) {
 }
 
 
-static SCM open_file_port(SCM filename, char *mode, int flags, int error)
+static SCM open_file_port(SCM filename, const char *mode, int flags, int error)
 {
-  char *full_name, *name;
+  const char *full_name, *name;
   SCM z;
   int fd;
 
@@ -603,7 +602,7 @@ static SCM open_file_port(SCM filename, char *mode, int flags, int error)
 
 
 
-SCM STk_fd2scheme_port(int fd, char *mode, char *identification)
+SCM STk_fd2scheme_port(int fd, const char *mode, char *identification)
 {
   int flags;
 
@@ -662,10 +661,10 @@ void STk_set_line_buffered_mode(SCM port)
  * Takes a string naming an existing file and returns an input port capable
  * of delivering characters from the file. If the file cannot be opened,
  * an error is signalled.
- * @l
- * ,(bold "Note:") if |filename| starts with the string ,(code  (q "| ")),
- * this procedure returns a pipe port. Consequently, it is not possible to
- * open a file whose name starts with those two characters.
+ *
+ * NOTE: if |filename| starts with the string `"| "`, this procedure returns a pipe port.
+ * Consequently, it is not possible to open a file whose name starts with those two
+ * characters.
 doc>
  */
 DEFINE_PRIMITIVE("open-input-file", open_input_file, subr1, (SCM filename))
@@ -688,10 +687,10 @@ DEFINE_PRIMITIVE("open-binary-input-file", open_binary_ifile, subr1, (SCM filena
  * port capable of writing characters to a new file by that name. If the file
  * cannot be opened, an error is signalled. If a file with the given name
  * already exists, it is rewritten.
- * @l
- * ,(bold "Note:") if |filename| starts with the string ,(code  (q "| ")),
- * this procedure returns a pipe port. Consequently, it is not possible to
- * open a file whose name starts with those two characters.
+ *
+ * NOTE: if |filename| starts with the string `"| "`, this procedure returns a pipe port.
+ * Consequently, it is not possible to open a file whose name starts with those two
+ * characters.
 doc>
  */
 DEFINE_PRIMITIVE("open-output-file", open_output_file, subr1, (SCM filename))
@@ -713,7 +712,7 @@ DEFINE_PRIMITIVE("open-binary-output-file", open_binary_ofile, subr1, (SCM filen
  * (output-file-port? obj)
  *
  * Returns |#t| if |obj| is a file input port or a file output port respectively,
- * otherwise returns #f.
+ * otherwise returns |#f|.
 doc>
  */
 DEFINE_PRIMITIVE("input-file-port?", input_fportp, subr1, (SCM port))
@@ -733,30 +732,30 @@ DEFINE_PRIMITIVE("output-file-port?", output_fportp, subr1, (SCM port))
  *
  * Opens the file whose name is |filename| with the specified string
  * |mode| which can be:
- * ,(itemize
- * (item [|"r"| to open file for reading. The stream is positioned at
- * the beginning of the file.])
  *
- * (item [|"r+"| to open file for reading and writing.  The stream is
- * positioned at the beginning of the file.])
+ * - |"r"| to open the file for reading. The stream is positioned at
+ *   the beginning of the file.
  *
- * (item [|"w"| to truncate file to zero length or create file for writing.
- * The stream is positioned at the beginning of the file.])
+ * - |"r+"| to open the file for reading and writing. The stream is
+ *   positioned at the beginning of the file.
  *
- * (item [|"w+"| to open  file for reading and writing. The file is created
- * if it does not exist, otherwise it is truncated. The stream is positioned
- * at the beginning of the file.])
+ * - |"w"| to truncate the file to zero length or create the file for writing.
+ *   The stream is positioned at the beginning of the file.
  *
- * (item [|"a"| to open for writing.  The file is created if  it  does
- * not exist. The stream is positioned at the end of the file.])
+ * - |"w+"| to open the file for reading and writing. The file is created
+ *   if it does not exist, otherwise it is truncated. The stream is positioned
+ *   at the beginning of the file.
+ *
+ * - |"a"| to open the file for writing. The file is created if it does
+ *   not exist. The stream is positioned at the end of the file.
+ *
+ * - |"a+"| to open the file for reading and writing. The file is created
+ *   if it does not exist. The stream is positioned at the end of the file.
  *
- * (item [|"a+"| to open file for reading and writing. The file is created
- * if it does not exist. The stream is positioned at the end of the file.])
- * )
  * If the file can be opened, |open-file| returns the textual port associated
- * with the given file, otherwise it returns |#f|. Here again, the ``magic''
- * string "@pipe " permits to open a pipe port (in this case mode can only be
- * |"r"| or |"w"|).
+ * with the given file, otherwise it returns |#f|. Here again, the *_magic_*
+ * string "@pipe " permits to open a pipe port. (In this case mode can only be
+ * |"r"| or |"w"|.)
 doc>
  */
 
@@ -846,13 +845,10 @@ static int find_file_nature(SCM f)
 }
 
 
-SCM STk_load_source_file(SCM f)
+SCM STk_load_source_file(SCM f, SCM env)
 {
   SCM sexpr;
-  SCM eval_symb, eval, ref;
-
-  /* //FIXME: eval devrait être connu sans faire de lookup(i.e. exporté par la VM)*/
-  eval_symb = STk_intern("eval");
+  SCM eval, ref, eval_symb = STk_intern("eval");
 
   for ( ; ; ) {
     /* We need to lookup for eval symbol for each expr, since it can
@@ -862,28 +858,37 @@ SCM STk_load_source_file(SCM f)
     sexpr = STk_read_constant(f, PORT_CASE_SENSITIVEP(f));
     if (sexpr == STk_eof) break;
     eval  = STk_lookup(eval_symb, STk_current_module(), &ref, TRUE);
-    STk_C_apply(eval, 1, sexpr);
+
+    /* NOTE: we cannot call *always* eval with 2 parameters since, at boot
+     * time, things about modules are not completely initialized.
+     */
+     if (env) STk_C_apply(eval, 2, sexpr, env);
+     else     STk_C_apply(eval, 1, sexpr);
   }
   STk_close_port(f);
   return STk_true;
 }
 
 
-static SCM load_file(SCM filename)
+static SCM load_file(SCM filename, SCM env)
 {
   SCM f;
-  char *fname = STRING_CHARS(filename);
+  char *fname;
+
+  if (!STRINGP(filename))   STk_error_bad_file_name(filename);
+  if (env && !MODULEP(env)) STk_error("bad environment ~S", env);
 
   /* Verify that file is not a directory */
+  fname = STRING_CHARS(filename);
   if (STk_dirp(fname)) return STk_false;
 
   /* It's Ok, try now to load file */
   f = STk_open_file(fname, "r");
   if (f != STk_false) {
     switch (find_file_nature(f)) {
-      case FILE_IS_SOURCE: return STk_load_source_file(f);
-      case FILE_IS_BCODE:  return STk_load_bcode_file(f);
-      case FILE_IS_OBJECT: return STk_load_object_file(f, fname);
+      case FILE_IS_SOURCE: return STk_load_source_file(f, env);
+      case FILE_IS_BCODE:  return STk_load_bcode_file(f, env);
+      case FILE_IS_OBJECT: return STk_load_object_file(f, fname, env);
     }
   }
   return STk_false;
@@ -891,24 +896,38 @@ static SCM load_file(SCM filename)
 
 
 /*
-<doc load
+<doc R57RS load
  * (load filename)
+ * (load filename env)
  *
+ * (((load-path)))
+ * (((load-suffixes)))
  * |Filename| should be a string naming an existing file containing Scheme
  * expressions. |Load| has been extended in STklos to allow loading of
- * file containing Scheme compiled code as well as object files
- * (,(emph "aka") shared objects). The loading of object files is not available on
- * all architectures. The value returned by |load| is ,(emph "void").
- * @l
+ * files containing Scheme compiled code as well as object files (_aka_
+ * shared objects). The loading of object files is not available on
+ * all architectures. The value returned by |load| is *_void_*.
+ *
+ * The optional parameter |env| may be an environment specifier (as defined by
+ * {{rseven}}) or a module. When |env| is specified, the loaded code is evaluated
+ * (or run) in the environment given by |env|. If omitted,
+ * |(interaction-environment)| is assumed. When |env| is specified as a module,
+ * it is mutable.
+ *
  * If the file whose name is |filename| cannot be located, |load| will try
- * to find it in one of the directories given by ,(code (ref :mark "load-path"))
- * with the suffixes given by ,(code (ref :mark "load-suffixes")).
+ * to find it in one of the directories given by `"load-path"`
+ * with the suffixes given by `"load-suffixes"`.
+ *
+ * NOTE: {{rfive}} defines only the one parameter version of |load|.
 doc>
  */
-DEFINE_PRIMITIVE("load", scheme_load, subr1, (SCM filename))
+
+DEFINE_PRIMITIVE("load", scheme_load, subr12, (SCM filename, SCM env))
 {
-  if (!STRINGP(filename)) STk_error_bad_file_name(filename);
-  if (load_file(filename) == STk_false)
+  // NOTE: This function is overloaded by the load function in
+  // "lib/load.stk". We keep it here to have a fallback function, for a
+  // possible alternative boot image.
+  if (load_file(filename, env) == STk_false)
     STk_error_file_name("cannot load ~S", filename);
   return STk_void;
 }
@@ -917,23 +936,27 @@ DEFINE_PRIMITIVE("load", scheme_load, subr1, (SCM filename))
 /*
 <doc EXT try-load
  * (try-load filename)
+ * (try-load filename env)
  *
- * |try-load| tries to load the file named |filename|. As |load|,
+ * |try-load| tries to load the file named |filename|. As with |load|,
  * |try-load| tries to find the file given the current load path
  * and a set of suffixes if |filename| cannot be loaded. If |try-load|
  * is able to find a readable file, it is loaded, and |try-load| returns
- * |#t|. Otherwise,  |try-load| retuns |#f|.
+ * |#t|. Otherwise, |try-load| retuns |#f|.
+ *
+ * See the description of |env| environment in the <<load>> primitive.
 doc>
  */
-DEFINE_PRIMITIVE("try-load", scheme_try_load, subr1, (SCM filename))
+DEFINE_PRIMITIVE("try-load", scheme_try_load, subr12, (SCM filename, SCM env))
 {
-  if (!STRINGP(filename)) STk_error_bad_file_name(filename);
-  return load_file(filename);
+  // NOTE: This primitive will be overloaded (but sill used bay the
+  // overloading function) in the file "lib/load.stk"
+  return load_file(filename, env);
 }
 
 
 
-DEFINE_PRIMITIVE("%file-informations", file_informations, subr1, (SCM filename))
+DEFINE_PRIMITIVE("%file-information", file_information, subr1, (SCM filename))
 {
   SCM f, res = STk_nil;
   char *fname;
@@ -947,7 +970,7 @@ DEFINE_PRIMITIVE("%file-informations", file_informations, subr1, (SCM filename))
     f = STk_open_file(fname, "r");
     if (f != STk_false) {
       int type  = find_file_nature(f);
-      char* str = "";
+      char *str = "";
 
       switch (type) {
         case FILE_IS_SOURCE: str = "source"; break;
@@ -999,7 +1022,7 @@ int STk_init_fport(void)
   ADD_PRIMITIVE(port_idle);
 
   //  ADD_PRIMITIVE(dbg);
-  ADD_PRIMITIVE(file_informations);
+  ADD_PRIMITIVE(file_information);
   return TRUE;
 }
 
