@@ -1,17 +1,16 @@
 /*
  * gnu-getopt.c         -- Adaptation of the GNU getopt function for STklos
  *
- * Copyright © 2001-2020 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2001-2023 Erick Gallesio <eg@stklos.net>
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  1-Nov-2001 23:40 (eg)
- * Last file update:  2-Jun-2020 12:55 (eg)
  *
  * The following code is constituted of the GNU getopt, getopt_long
  * and getopt_long_only implementation. Code is slightly modified (all
  * modifications are tagged with the string "STklos"). The
  * modifications permit to use the Sheme IO function instead of the
- * standard C fprintf
+ * standard C fprintf (plus ANSIfication of some K&R function definitions)
  */
 
 
@@ -91,7 +90,7 @@ static void getopt_fprintf(FILE* f, const char *format, ...)
 
 void STk_start_getopt_from_scheme(void)
 {
-  we_are_in_scheme = TRUE; /* Don't use anymore fprintf for reportin errors */
+  we_are_in_scheme = TRUE; /* Don't use fprintf anymore for reporting errors */
 }
 
 
@@ -150,7 +149,7 @@ void STk_start_getopt_from_scheme(void)
    to intersperse the options with the other arguments.
 
    As `getopt' works, it permutes the elements of ARGV so that,
-   when it is done, all the options precede everything else.  Thus
+   when it is done, all the options precede everything else.  Thus,
    all application programs are extended to handle flexible argument order.
 
    Setting the environment variable POSIXLY_CORRECT disables permutation.
@@ -360,9 +359,9 @@ text_set_element (__libc_subinit, store_args_and_env);
 static void exchange (char **);
 #endif
 
+//STklos: make de definition C-ANSI
 static void
-exchange (argv)
-     char **argv;
+exchange (char **argv)
 {
   int bottom = first_nonopt;
   int middle = last_nonopt;
@@ -445,11 +444,12 @@ exchange (argv)
 #if defined __STDC__ && __STDC__
 static const char *_getopt_initialize (int, char *const *, const char *);
 #endif
+//STklos: make de definition C-ANSI
 static const char *
-_getopt_initialize (argc, argv, optstring)
-     int argc;
-     char *const *argv;
-     const char *optstring;
+_getopt_initialize (
+                    int argc,
+                    char *const *argv,
+                    const char *optstring)
 {
   (void) (argc);
   (void) (argv);
@@ -535,7 +535,7 @@ _getopt_initialize (argc, argv, optstring)
    OPTSTRING is a string containing the legitimate option characters.
    If an option character is seen that is not listed in OPTSTRING,
    return '?' after printing an error message.  If you set `opterr' to
-   zero, the error message is suppressed but we still return '?'.
+   zero, the error message is suppressed, but we still return '?'.
 
    If a char in OPTSTRING is followed by a colon, that means it wants an arg,
    so the following text in the same ARGV-element, or the text of the following
@@ -570,14 +570,15 @@ _getopt_initialize (argc, argv, optstring)
    If LONG_ONLY is nonzero, '-' as well as '--' can introduce
    long-named options.  */
 
+//STklos: make de definition C-ANSI
 int
-_getopt_internal (argc, argv, optstring, longopts, longind, long_only)
-     int argc;
-     char *const *argv;
-     const char *optstring;
-     const struct option *longopts;
-     int *longind;
-     int long_only;
+_getopt_internal (
+                  int argc,
+                  char *const *argv,
+                  const char *optstring,
+                  const struct option *longopts,
+                  int *longind,
+                  int long_only)
 {
   optarg = NULL;
 
@@ -684,7 +685,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 
      If long_only and the ARGV-element has the form "-f", where f is
      a valid short option, don't consider it an abbreviated form of
-     a long option that starts with f.  Otherwise there would be no
+     a long option that starts with f.  Otherwise, there would be no
      way to give the -f short option.
 
      On the other hand, if there's a long option "fubar" and
@@ -1026,11 +1027,12 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
   }
 }
 
+//STklos: make de definition C-ANSI
 int
-getopt (argc, argv, optstring)
-     int argc;
-     char *const *argv;
-     const char *optstring;
+getopt (
+        int argc,
+        char *const *argv,
+        const char *optstring)
 {
   return _getopt_internal (argc, argv, optstring,
                            (const struct option *) 0,
@@ -1049,13 +1051,14 @@ getopt (argc, argv, optstring)
  */
 
 
+//STklos: make de definition C-ANSI
 int
-getopt_long (argc, argv, options, long_options, opt_index)
-     int argc;
-     char *const *argv;
-     const char *options;
-     const struct option *long_options;
-     int *opt_index;
+getopt_long (
+             int argc,
+             char *const *argv,
+             const char *options,
+             const struct option *long_options,
+             int *opt_index)
 {
   return _getopt_internal (argc, argv, options, long_options, opt_index, 0);
 }
@@ -1065,13 +1068,14 @@ getopt_long (argc, argv, options, long_options, opt_index)
    but does match a short option, it is parsed as a short option
    instead.  */
 
+//STklos: make de definition C-ANSI
 int
-getopt_long_only (argc, argv, options, long_options, opt_index)
-     int argc;
-     char *const *argv;
-     const char *options;
-     const struct option *long_options;
-     int *opt_index;
+getopt_long_only (
+                  int argc,
+                  char *const *argv,
+                  const char *options,
+                  const struct option *long_options,
+                  int *opt_index)
 {
   return _getopt_internal (argc, argv, options, long_options, opt_index, 1);
 }
