@@ -2,7 +2,7 @@
  *
  * h a s h  . c                 -- Hash Tables (mostly SRFI-69)
  *
- * Copyright © 1994-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1994-2024 Erick Gallesio <eg@stklos.net>
  *
  +=============================================================================
  ! This code is a rewriting of the file tclHash.c of the Tcl
@@ -36,10 +36,10 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 17-Jan-1994 17:49
- * Last file update: 10-Apr-2021 18:42 (eg)
  */
 
 #include "stklos.h"
+#include "vm.h"
 #include "hash.h"
 
 
@@ -64,7 +64,7 @@
  *
 \*===========================================================================*/
 
-static unsigned long hash_string(register char *string)
+static unsigned long hash_string(register const char *string)
 {
   register unsigned long result = 0;
   register int c;
@@ -79,7 +79,7 @@ static unsigned long hash_string(register char *string)
    *    and multiplying by 9 is just about as good.
    * 2. Times-9 is (shift-left-3) plus (old).  This means that each
    *    character's bits hang around in the low-order bits of the
-   *    hash value for ever, plus they spread fairly rapidly up to
+   *    hash value forever, plus they spread fairly rapidly up to
    *    the high-order bits to fill out the hash value.  This seems
    *    works well both for decimal and non-decimal strings.
    */
@@ -93,7 +93,7 @@ static unsigned long hash_string(register char *string)
 
 static unsigned long hash_scheme_string(SCM str)
 {                                            /* The same one for Scheme strings */
-  char *string;
+  const char *string;
   unsigned long result = 0;
   int i, l;
 
@@ -138,10 +138,15 @@ static unsigned long sxhash(SCM obj)
                         for (i=VECTOR_SIZE(obj)-1; i >= 0; i--)
                           h = HASH_WORD(h, sxhash(VECTOR_DATA(obj)[i]));
                         return h;
+    case tc_uvector:    h = UVECTOR_TYPE(obj);     /* start with some "entropy" */
+                        for (long i=UVECTOR_SIZE(obj)-1; i >= 0; i--)
+                          h = HASH_WORD(h, sxhash(STk_uvector_get(obj, i)));
+                        return h;
+
     default:            /* A complex type (STklos object, user defined type,
                          * hashtable...). In this case we return the type of the
-                         * object. This is very  inneficient but it should be
-                         * rare to use a  structured object as a key.
+                         * object. This is very inefficient, but it should be
+                         * rare to use a structured object as a key.
                          */
                          return (unsigned long) BOXED_TYPE(obj);
   }
@@ -152,11 +157,11 @@ static unsigned long sxhash(SCM obj)
  *                              H a s h    S t a t s
  *
 \*===========================================================================*/
-#define _MAX_COUNT 5
+#define MAX_COUNT 5
 
 static void hash_stats(struct hash_table_obj *h, SCM port)
 {
-  int i, j, more, len, count[_MAX_COUNT] = {0};
+  int i, j, more, len, count[MAX_COUNT] = {0};
 
   /* If we're using unicode, we can draw a nicer histogram bar... */
   char *bar_char;
@@ -175,7 +180,7 @@ static void hash_stats(struct hash_table_obj *h, SCM port)
         STk_fprintf(port,bar_char);
     STk_putc('\n', port);
 
-    if (len < _MAX_COUNT)
+    if (len < MAX_COUNT)
       count[len] += 1;
     else
       more += 1;
@@ -186,12 +191,12 @@ static void hash_stats(struct hash_table_obj *h, SCM port)
               (double) HASH_NENTRIES(h)/ HASH_NBUCKETS(h));
   STk_fprintf(port, "Repartition\n");
 
-  for (i = 0; i < _MAX_COUNT; i++) {
+  for (i = 0; i < MAX_COUNT; i++) {
     if (count[i])
       STk_fprintf(port, "  %d buckets with %d entries\n", count[i], i);
   }
   if (more)
-    STk_fprintf(port, "  %d buckets with more than %d entries\n", more, _MAX_COUNT);
+    STk_fprintf(port, "  %d buckets with more than %d entries\n", more, MAX_COUNT);
 }
 
 
@@ -225,7 +230,7 @@ static void enlarge_table(register struct hash_table_obj *h)
     HASH_BUCKETS(h)[i] = STk_nil;
   }
 
-  /*  Rehash all of the existing entries into the new bucket array. */
+  /*  Rehash all the existing entries into the new bucket array. */
   for (i = 0; i < old_size; i++) {
     for (tmp = old_buckets[i]; !NULLP(tmp); tmp = CDR(tmp)) {
       switch (BOXED_INFO(h)) {
@@ -253,6 +258,11 @@ static void enlarge_table(register struct hash_table_obj *h)
           }
         }
         break;
+
+      /* C hash tables (as symbol hash) but with direct char* (not interning) */
+      case HASH_C_FLAG:
+        index = hash_string(CAR(CAR(tmp))) & HASH_MASK(h);
+        break;
       }
       /* Place the old value at new index */
       HASH_BUCKETS(h)[index] = STk_cons(CAR(tmp), HASH_BUCKETS(h)[index]);
@@ -299,7 +309,7 @@ void STk_hashtable_init(struct hash_table_obj *h, int flag)
  *
 \*===========================================================================*/
 
-static Inline SCM hash_get_symbol(struct hash_table_obj *h, char *s, int *index)
+static inline SCM hash_get_symbol(struct hash_table_obj *h, const char *s, int *index)
 {
   register SCM l;
 
@@ -312,7 +322,7 @@ static Inline SCM hash_get_symbol(struct hash_table_obj *h, char *s, int *index)
 }
 
 
-SCM STk_hash_intern_symbol(struct hash_table_obj *h, char *s, SCM (*create) (char*s))
+SCM STk_hash_intern_symbol(struct hash_table_obj *h, const char *s, SCM (*create) (const char *s))
 {
   SCM z;
   int index;
@@ -333,19 +343,20 @@ SCM STk_hash_intern_symbol(struct hash_table_obj *h, char *s, SCM (*create) (cha
 
 /*===========================================================================*\
  *
- *             v a r i a b l e s    h a s h t a b l e   f u n t i o n s
+ *             v a r i a b l e s    h a s h t a b l e   f u n c t i o n s
  *
  *
  * Here variable are the variables defined in a module. Keys are symbols.
- * The value associated to a bucket is an A-list (symbol . value) of all the
- * symbols with the same hash value
+ * The value associated to a bucket is an list of symbols with the same hash
+ * value. Each component of this lis is a tc_global_obj (a couple symbol +
+ * index of this variable in the global store.
  *
 \*===========================================================================*/
 
-SCM STk_hash_get_variable(struct hash_table_obj *h, SCM v, int *index)
+static inline SCM hash_get_variable(struct hash_table_obj *h, SCM v, int *index)
 {
   register SCM l;
-  char *s = SYMBOL_PNAME(v);
+  const char *s = SYMBOL_PNAME(v);
 
   *index = hash_string(s) & HASH_MASK(h);
 
@@ -355,46 +366,23 @@ SCM STk_hash_get_variable(struct hash_table_obj *h, SCM v, int *index)
   return (SCM) NULL;
 }
 
-
-void STk_hash_set_variable(struct hash_table_obj *h, SCM v, SCM value)
+SCM STk_hash_get_variable(struct hash_table_obj *h, SCM v)
 {
-  SCM z;
-  int index;
-
-  z = STk_hash_get_variable(h, v, &index);
-
-  if (z) {
-    /* Variable already exists. Change its value*/
-    *BOX_VALUES(CDR(z)) = value;
-  } else {
-    SCM z;
-
-    /* Create a new box for this value */
-    z = STk_make_box(value);
-
-    /* Enter the new variable in table */
-    HASH_BUCKETS(h)[index] = STk_cons(STk_cons(v, z),
-                                      HASH_BUCKETS(h)[index]);
-    HASH_NENTRIES(h) += 1;
-    /* If the table has exceeded a decent size, rebuild it */
-    if (HASH_NENTRIES(h) >= HASH_NEWSIZE(h)) enlarge_table(h);
-  }
+  int idx;
+  return hash_get_variable(h, v, &idx);
 }
 
-void STk_hash_set_alias(struct hash_table_obj *h, SCM v, SCM value)
+void STk_hash_define_variable(struct hash_table_obj *h, SCM v, SCM value)
 {
-  SCM z;
+  SCM z, new;
   int index;
 
-  z = STk_hash_get_variable(h, v, &index);
+  z   = hash_get_variable(h, v, &index);
+  new = STk_global_store_define(z, v, value);
 
-  if (z) {
-    /* Variable already exists. Change its value*/
-    CDR(z) = value;
-  } else {
-    /* Enter the new variable in table */
-    HASH_BUCKETS(h)[index] = STk_cons(STk_cons(v, value),
-                                      HASH_BUCKETS(h)[index]);
+  if (!z) {
+    /* v was not define before, enter it in the module hash table */
+    HASH_BUCKETS(h)[index] = STk_cons(new, HASH_BUCKETS(h)[index]);
     HASH_NENTRIES(h) += 1;
     /* If the table has exceeded a decent size, rebuild it */
     if (HASH_NENTRIES(h) >= HASH_NEWSIZE(h)) enlarge_table(h);
@@ -402,7 +390,21 @@ void STk_hash_set_alias(struct hash_table_obj *h, SCM v, SCM value)
 }
 
 
+void STk_hash_set_alias(struct hash_table_obj *h, SCM v, SCM old)
+{
+  SCM z, new;
+  int index;
 
+  z   = hash_get_variable(h, v, &index);
+  new = STk_global_store_alias(z, v, old);
+
+  if (!z) {
+    HASH_BUCKETS(h)[index] = STk_cons(new, HASH_BUCKETS(h)[index]);
+    HASH_NENTRIES(h) += 1;
+    /* If the table has exceeded a decent size, rebuild it */
+    if (HASH_NENTRIES(h) >= HASH_NEWSIZE(h)) enlarge_table(h);
+  }
+}
 
 /*===========================================================================*\
  *
@@ -438,6 +440,10 @@ static void error_bad_hash_table(SCM obj)
   STk_error("bad hash table ~S", obj);
 }
 
+static void error_hash_immutable(SCM obj)
+{
+  STk_error("hash table ~S is not mutable", obj);
+}
 
 static void error_bad_procedure(SCM obj)
 {
@@ -488,6 +494,38 @@ SCM STk_make_basic_hash_table(void) {
   return z;
 }
 
+
+/*
+<doc EXT hash-immutable!
+ * (hash-immutable! obj)
+ *
+ * If |obj| is a hash table, makes it immutable. Otherwise, raises
+ * an error.
+doc>
+*/
+DEFINE_PRIMITIVE("hash-table-immutable!", hash_table_lock, subr1, (SCM ht))
+{
+  if(!HASHP(ht)) error_bad_hash_table(ht);
+  BOXED_INFO(ht) |= HASH_CONST;
+  return STk_void;
+}
+
+/*
+<doc EXT hash-mutable?
+ * (hash-mutable? obj)
+ *
+ * Returns |#t| if |obj| is an immutable hash table, |#f| if it
+ * is a mutable hash table, and raises an error if |obj| is not a
+ * hash table.
+doc>
+*/
+DEFINE_PRIMITIVE("hash-table-mutable?", hash_table_mutablep, subr1, (SCM ht))
+{
+  if(!HASHP(ht)) error_bad_hash_table(ht);
+  return MAKE_BOOLEAN(!(HASH_CONSTP(ht)));
+}
+
+
 /*
 <doc EXT hash-table?
  * (hash-table? obj)
@@ -550,7 +588,7 @@ DEFINE_PRIMITIVE("hash-table-hash-function", hash_table_hash_func,
  * (hash-table-set! hash key value)
  *
  * Enters an association between |key| and |value| in the|hash| table.
- * The value returned by |hash-table-set!| is ,(emph "void").
+ * The value returned by |hash-table-set!| is *_void_*.
 doc>
 */
 DEFINE_PRIMITIVE("hash-table-set!", hash_set, subr3, (SCM ht, SCM key, SCM val))
@@ -560,6 +598,8 @@ DEFINE_PRIMITIVE("hash-table-set!", hash_set, subr3, (SCM ht, SCM key, SCM val))
 
   if (!HASHP(ht)) error_bad_hash_table(ht);
 
+  if (HASH_CONSTP(ht)) error_hash_immutable(ht);
+
   switch (HASH_TYPE(ht)) {
     case hash_eqp:
       index = RANDOM_INDEX(ht, key);
@@ -625,7 +665,7 @@ DEFINE_PRIMITIVE("hash-table-set!", hash_set, subr3, (SCM ht, SCM key, SCM val))
  * @end lisp
 doc>
 */
-static Inline SCM hash_table_search(SCM ht, SCM key)
+static inline SCM hash_table_search(SCM ht, SCM key)
 {
   int index = 0;
   SCM func, l = STk_nil;
@@ -711,7 +751,7 @@ DEFINE_PRIMITIVE("hash-table-exists?", hash_existp, subr2, (SCM ht, SCM key))
  * (hash-table-delete! hash key)
  *
  * Deletes the entry for |key| in |hash|, if it exists. Result of
- * |hash-table-delete!| is ,(emph "void").
+ * |hash-table-delete!| is *_void_*.
  *
  * @lisp
  * (define h (make-hash-table))
@@ -730,6 +770,8 @@ DEFINE_PRIMITIVE("hash-table-delete!", hash_delete, subr2, (SCM ht, SCM key))
 
   if (!HASHP(ht)) error_bad_hash_table(ht);
 
+  if (HASH_CONSTP(ht)) error_hash_immutable(ht);
+
   l = prev = STk_nil;
 
   switch (HASH_TYPE(ht)) {
@@ -767,6 +809,26 @@ DEFINE_PRIMITIVE("hash-table-delete!", hash_delete, subr2, (SCM ht, SCM key))
   return STk_void;
 }
 
+DEFINE_PRIMITIVE("hash-table-clear!", hash_clear, subr1, (SCM ht))
+{
+  if (!HASHP(ht)) error_bad_hash_table(ht);
+
+  /* We could either clear the fields that are not to be kept from the
+     hashtable, one at a time, or select the fields to be kept,
+     re-initialize and restore what needs to be restored. We chose
+     the second approach. */
+  hash_type type = HASH_TYPE(ht);
+  SCM comparison = HASH_COMPAR(ht);
+  SCM hash_fct   = HASH_HASH(ht);
+
+  STk_hashtable_init ((struct hash_table_obj *)ht, BOXED_INFO(ht));
+
+  HASH_TYPE(ht)   = type;
+  HASH_COMPAR(ht) = comparison;
+  HASH_HASH(ht)   = hash_fct;
+
+  return STk_void;
+}
 
 
 /*
@@ -777,12 +839,12 @@ DEFINE_PRIMITIVE("hash-table-delete!", hash_delete, subr2, (SCM ht, SCM key))
  * |Proc| must be a procedure taking two arguments. |Hash-table-for-each|
  * calls |proc| on each key/value association in |hash|, with the key as
  * the first argument and the value as the second.  The value returned by
- * |hash-table-for-each| is ,(emph "void").
- *  ,(linebreak)
- * ,(bold "Note:") The order of application of |proc| is unspecified.
- *  ,(linebreak)
- * ,(bold "Note:") |hash-table-walk| is another name for |hash-table-for-each|
- * (this is the name used in ,(link-srfi 69)).
+ * |hash-table-for-each| is *_void_*.
+ *
+ * NOTE: The order of application of |proc| is unspecified.
+ *
+ * NOTE: |hash-table-walk| is another name for |hash-table-for-each|
+ * (this is the name used in {{link-srfi 69}}.
  *
  * @lisp
  * (let ((h   (make-hash-table))
@@ -820,8 +882,8 @@ DEFINE_PRIMITIVE("hash-table-for-each", hash_for_each, subr2, (SCM ht, SCM proc)
  * the first argument and the value as the second.  The result of
  * |hash-table-map| is a list of the values returned by |proc|, in an
  * unspecified order.
- * ,(linebreak)
- * ,(bold "Note:") The order of application of |proc| is unspecified.
+ *
+ * NOTE: The order of application of |proc| is unspecified.
  * @lisp
  * (let ((h (make-hash-table)))
  *   (dotimes (i 5)
@@ -855,12 +917,12 @@ DEFINE_PRIMITIVE("hash-table-map", hash_map, subr2, (SCM ht, SCM proc))
  * (hash-table-hash obj)
  *
  * Computes a hash code for an object and returns this hash code as a
- * non negative integer. A property of |hash-table-hash| is that
+ * non-negative integer. A property of |hash-table-hash| is that
  * @lisp
  * (equal? x y) => (equal? (hash-table-hash x) (hash-table-hash y)
  * @end lisp
  *
- * as the the Common Lisp |sxhash| function from which this procedure is
+ * as the Common Lisp |sxhash| function from which this procedure is
  * modeled.
 doc>
 */
@@ -878,9 +940,9 @@ DEFINE_PRIMITIVE("hash-table-hash", hash_hash, subr1, (SCM obj))
  *
  * Prints  overall information about |hash|, such as the number of entries
  * it contains, the number of buckets in its hash array, and the utilization
- * of the buckets. Informations are printed on |port|. If no |port| is given
+ * of the buckets. Information is printed on |port|. If no |port| is given
  * to |hash-table-stats|, information are printed on the current output port
- * (see ,(ref :mark "current-output-port")).
+ * (see _<<curroport,`current-output-port` primitive>>_).
 doc>
 */
 DEFINE_PRIMITIVE("hash-table-stats", hash_stats, subr12, (SCM ht, SCM port))
@@ -893,6 +955,54 @@ DEFINE_PRIMITIVE("hash-table-stats", hash_stats, subr12, (SCM ht, SCM port))
   return STk_void;
 }
 
+/*===========================================================================*\
+ *
+ *                      C hash tables
+ *
+ * For internal use: keys are C strings and values are in fact void*
+ *
+ * NOTE: Use only the functions defined below for these hash tables. Standard
+ * hash table functions cannot deal with this sort of hash tables (except the
+ * function enlarge_table).
+ *
+\*===========================================================================*/
+
+SCM STk_make_C_hash_table(void)
+{
+  SCM z;
+
+  NEWCELL(z, hash_table);
+  STk_hashtable_init((struct hash_table_obj *) z, HASH_C_FLAG);
+  return z;
+}
+
+void *STk_C_hash_get(struct hash_table_obj *ht, const char *key)
+{
+  int index = hash_string(key) & HASH_MASK(ht);
+  for(SCM l = HASH_BUCKETS(ht)[index]; !NULLP(l); l = CDR(l))
+    if (strcmp(CAR(CAR(l)), key) == 0) return (void*) CDR(CAR(l));
+  /* Not found */
+  return NULL;
+}
+
+void STk_C_hash_set(struct hash_table_obj *ht, const char *key, void *val)
+{
+  SCM l;
+  int index = hash_string(key) & HASH_MASK(ht);
+
+  for(l = HASH_BUCKETS(ht)[index]; !NULLP(l); l = CDR(l))
+    if (strcmp(CAR(CAR(l)), key) == 0) break;
+
+  if (NULLP(l)) {
+    HASH_BUCKETS(ht)[index] = STk_cons(STk_cons((SCM) key, val), HASH_BUCKETS(ht)[index]);
+    HASH_NENTRIES(ht) += 1;
+    /* If the table has exceeded a decent size, rebuild it */
+    if (HASH_NENTRIES(ht) >= HASH_NEWSIZE(ht))
+      enlarge_table((struct hash_table_obj *) ht);
+  }
+  else
+    CDR(CAR(l)) = val;
+}
 
 /*===========================================================================*\
  *
@@ -909,6 +1019,8 @@ int STk_init_hash(void)
 
   /* Define primitives */
   ADD_PRIMITIVE(make_hash);
+  ADD_PRIMITIVE(hash_table_lock);
+  ADD_PRIMITIVE(hash_table_mutablep);
   ADD_PRIMITIVE(hashtablep);
   ADD_PRIMITIVE(hash_table_size);
   ADD_PRIMITIVE(hash_table_eqv_func);
@@ -920,6 +1032,7 @@ int STk_init_hash(void)
   ADD_PRIMITIVE(hash_existp);
 
   ADD_PRIMITIVE(hash_delete);
+  ADD_PRIMITIVE(hash_clear);
 
   ADD_PRIMITIVE(hash_for_each);
   ADD_PRIMITIVE(hash_map);
