@@ -20,7 +20,6 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 19-Nov-1993 16:12
- * Last file update: 30-Apr-2021 14:51 (eg)
  */
 
 #include "stklos.h"
@@ -48,7 +47,7 @@ static void error_const_cell(SCM x)
   STk_error("changing the constant ~s is not allowed", x);
 }
 
-static SCM make_uninterned_keyword(char *name)
+static SCM make_uninterned_keyword(const char *name)
 {
   SCM z;
 
@@ -59,7 +58,7 @@ static SCM make_uninterned_keyword(char *name)
 }
 
 
-SCM STk_makekey(char *token)
+SCM STk_makekey(const char *token)
 {
   SCM res;
   MUT_DECL(lck);
@@ -94,7 +93,7 @@ doc>
  */
 DEFINE_PRIMITIVE("make-keyword", make_keyword, subr1, (SCM str))
 {
-  char *s = "";
+  const char *s = "";
 
   if (STRINGP(str))
     s = STRING_CHARS(str);
@@ -238,8 +237,8 @@ DEFINE_PRIMITIVE("key-set!", key_set, subr3, (SCM l, SCM key, SCM val))
  * |List| must be a list of keywords and their respective values.
  * |key-delete| remove the |key| and its associated value of the keyword
  * list. The key can be absent of the list.
- * ,(linebreak)
- * |key-delete!| does the same job than |key-delete| by physically 
+ * 
+ * |key-delete!| does the same job as |key-delete| by physically
  * modifying its |list| argument.
  * @lisp
  * (key-delete '(:one 1 :two 2) :two)    => (:one 1)
