@@ -1,7 +1,7 @@
 /*
  * l i b . c                    -- Scheme library
  *
- * Copyright © 2000-2020 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2000-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  5-Jan-2000 12:17 (eg)
- * Last file update: 30-May-2020 17:22 (eg)
  */
 
 
@@ -40,12 +39,13 @@ STk_init_library(int _UNUSED(*argc), char _UNUSED(***argv), int stack_size)
 
 
   return
+    STk_init_vm()                               &&
     STk_init_env()                              &&
     STk_init_symbol()                           &&
     STk_late_init_env()                         &&
     STk_init_struct()                           &&
     STk_init_cond()                             &&
-    STk_init_vm()                               &&
+    STk_late_init_vm()                         &&
     STk_init_threads(stack_size, start_stack)   &&
     STk_init_port()                             &&
     STk_init_extend()                           &&
@@ -77,6 +77,7 @@ STk_init_library(int _UNUSED(*argc), char _UNUSED(***argv), int stack_size)
     STk_init_cpointer()                         &&
     STk_init_box()                              &&
     STk_init_ffi()                              &&
+    STk_init_syntax()                           &&
 #ifdef STK_DEBUG
     STk_init_utf8()                             &&
 #endif
