@@ -2,7 +2,7 @@
  *
  * l i s t . c                  -- Lists procedures
  *
- * Copyright © 1993-2020 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,6 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??-Oct-1993 21:37
- * Last file update: 23-Jun-2020 09:41 (eg)
  */
 
 #include "stklos.h"
@@ -67,6 +66,10 @@ static void error_not_exact_positive(SCM x)
 {
   STk_error("index ~W is not an exact positive integer", x);
 }
+static void error_bad_comparison_function(SCM x)
+{
+  STk_error("bad comparison function ~S", x);
+}
 
 int STk_int_length(SCM l)
 {
@@ -92,7 +95,6 @@ SCM STk_argv2list(int argc, SCM *argv)
 }
 
 
-
 DEFINE_PRIMITIVE("pair?", pairp, subr1, (SCM x))
 /*
 <doc pair?
@@ -106,7 +108,6 @@ doc>
 }
 
 
-
 DEFINE_PRIMITIVE("cons", cons, subr2, (SCM x, SCM y))
 /*
 <doc cons
@@ -183,7 +184,7 @@ DEFINE_PRIMITIVE("set-car!", setcar, subr2, (SCM cell, SCM value))
  * (set-car! pair obj)
  *
  * Stores |obj| in the car field of |pair|.
- * The value returned by |set-car!| is ,(emph "void").
+ * The value returned by |set-car!| is *_void_*.
  * @lisp
  *    (define (f) (list 'not-a-constant-list))
  *    (define (g) '(constant-list))
@@ -207,7 +208,7 @@ DEFINE_PRIMITIVE("set-cdr!", setcdr, subr2, (SCM cell, SCM value))
  * (set-cdr! pair obj)
  *
  * Stores |obj| in the cdr field of |pair|.
- * The value returned by |set-cdr!| is ,(emph "void").
+ * The value returned by |set-cdr!| is *_void_*.
  *
 doc>
  */
@@ -219,6 +220,50 @@ doc>
   return STk_void;
 }
 
+DEFINE_PRIMITIVE("%cxr", cxr, subr2, (SCM l, SCM name))
+{
+  /* Special function to compute cadr, cddr, ...
+   * (%cxr lst #:dda) returns the caddr of lst
+   * The key given indicates the value of x (reversed)
+   * Using the non reversed value of x is simpler but incurs
+   * a certain time penalty.
+   * In case of error, we can display a clear message (with some work to
+   * rebuild the original function name), but we have time.
+   * NOTE: using strings (instead of keywords) is less efficient because
+   * the char * is at the end of the object. Using symbols is also fast
+   * (even a bit faster, don't know why), but it is harder  to detect that
+   * that we can inline when we have (%cxr lst 'daa), because of the quote. 
+   */
+  if (KEYWORDP(name)) {
+    SCM lst   = l;
+    const char *str = KEYWORD_PNAME(name);
+
+    for (const char *s = str;  *s; s++) {
+      if (CONSP(lst))
+        lst = (*s == 'a') ? CAR(lst): CDR(lst);
+      else {
+        int len   = strlen(str);
+        char *loc = STk_must_malloc_atomic(len+3); // 'c' + X + 'r' + \0
+
+        /* build location */
+        loc[0] = 'c';
+        for (int i = 0; i < len; i++) loc[i+1] = str[len-1-i];
+        loc[len+1] = 'r';
+        loc[len+2] = '\0';
+
+        /* display clear error */
+        name = STk_intern(loc);
+        STk_error_with_location(name, "wrong type of argument ~S for c%cr in (~s '~w)",
+                                lst, *s, name, l);
+      }
+    }
+    return lst;
+  }
+  else {
+    error_wrong_type(name);
+    return STk_void; // to avoid a compiler warning
+  }
+}
 
 DEFINE_PRIMITIVE("null?", nullp, subr1, (SCM x))
 /*
@@ -346,7 +391,7 @@ SCM STk_append2(SCM l1, SCM l2)
   CDR(prev) = l2;
   return res;
 Error:
-  STk_error("argument ~S is not a list", l1);
+  error_bad_list(l1);
   return STk_void; /* never reached */
 }
 
@@ -413,13 +458,13 @@ DEFINE_PRIMITIVE("list-tail", list_tail, subr2, (SCM list, SCM k))
   x = STk_integer_value(k);
   if (x >= 0) {
     for (l=list; x > 0; x--) {
-      if (NULLP(l) || !CONSP(l)) STk_error("list ~S too short", list);
+      if (NULLP(l) || !CONSP(l)) error_too_short(list);
       l = CDR(l);
     }
     return l;
   }
 
-  STk_error("index ~S is not an exact positive integer", k);
+  error_not_exact_positive(k);
   return STk_void; /* never reached */
 }
 
@@ -470,7 +515,7 @@ DEFINE_PRIMITIVE("list-ref", list_ref, subr2, (SCM list, SCM k))
  * (let ((ls (list 'one 'two 'five!)))
  *    (list-set! ls 2 'three)
  *    ls)                              => (one two three)
- * (list-set! ’(0 1 2) 1 "oops")       => error (constant list)
+ * (list-set! '(0 1 2) 1 "oops")       => error (constant list)
  * @end lisp
 doc>
 */
@@ -512,7 +557,7 @@ DEFINE_PRIMITIVE("list-set!", list_set, subr3, (SCM list, SCM k, SCM obj))
  * |(list-tail list k)| for |k| less than the length of list.
  * If |obj| does not occur in |list|, then |#f| (not the empty list) is
  * returned. |Memq| uses |eq?| to compare obj with the elements of list,
- * while |memv| uses |eqv?| and |member| uses |compare|, if given, and 
+ * while |memv| uses |eqv?| and |member| uses |compare|, if given, and
  * |equal?| otherwise.
  *
  * @lisp
@@ -528,7 +573,7 @@ DEFINE_PRIMITIVE("list-set!", list_set, subr3, (SCM list, SCM k, SCM obj))
  *    (memv 101 '(100 101 102))       =>  (101 102)
  * @end lisp
  *
- * ,(bold "Note:") As in R7RS, the |member| function accepts also a
+ * NOTE: As in R7RS, the |member| function accepts also a
  * comparison function.
 doc>
  */
@@ -540,7 +585,7 @@ doc>
 
 
 #define LMEMBER(compare)                                        \
-{                                                               \
+do{                                                             \
   register SCM ptr;                                             \
                                                                 \
   if (!CONSP(list) && !NULLP(list)) error_bad_list(list);       \
@@ -554,20 +599,24 @@ doc>
     if ((ptr=CDR(ptr)) == list) error_circular_list(ptr);       \
   }                                                             \
   return STk_false;                                             \
-}
+}while(0)
 
 
 DEFINE_PRIMITIVE("memq", memq, subr2, (SCM obj, SCM list))
-     LMEMBER(PTR_EQ)
+{
+    LMEMBER(PTR_EQ);
+}
 
 DEFINE_PRIMITIVE("memv", memv, subr2, (SCM obj, SCM list))
-     LMEMBER(PTR_EQV)
+{
+    LMEMBER(PTR_EQV);
+}
 
 DEFINE_PRIMITIVE("member", member, subr23, (SCM obj, SCM list, SCM cmp))
 {
   if (cmp) {
     if (STk_procedurep(cmp) != STk_true)
-      STk_error("bad comparison function ~S", cmp);
+      error_bad_comparison_function(cmp);
 
     LMEMBER(PTR_CMPGEN);
   } else {
@@ -604,18 +653,18 @@ DEFINE_PRIMITIVE("member", member, subr23, (SCM obj, SCM list, SCM cmp))
  *                               =>  (5 7)
  * @end lisp
  *
- * ,(bold "Rationale:") Although they are ordinarily used as predicates,
+ * IMPORTANT: Although they are ordinarily used as predicates,
  * |memq|, |memv|, |member|, |assq|, |assv|, and |assoc| do not have question
  * marks in their names because they return useful values rather than just
  * |#t| or #|f|.
  *
- * ,(bold "Note:") As in R7RS, the |assoc| function accepts also a
+ * NOTE: As in R7RS, the |assoc| function accepts also a
  * comparison function.
 doc>
  */
 
 #define LASSOC(compare)                                         \
-{                                                               \
+do{                                                             \
   register SCM l,tmp;                                           \
                                                                 \
   for(l=alist; CONSP(l); ) {                                    \
@@ -626,19 +675,23 @@ doc>
   if (NULLP(l)) return(STk_false);                              \
   STk_error("improper list ~W", alist);                         \
   return STk_void; /* never reached */                          \
-}
+}while(0)
 
 DEFINE_PRIMITIVE("assq", assq, subr2, (SCM obj, SCM alist))
-     LASSOC(PTR_EQ)
+{
+  LASSOC(PTR_EQ);
+}
 
 DEFINE_PRIMITIVE("assv", assv, subr2, (SCM obj, SCM alist))
-     LASSOC(PTR_EQV)
+{
+  LASSOC(PTR_EQV);
+}
 
 DEFINE_PRIMITIVE("assoc", assoc, subr23, (SCM obj, SCM alist, SCM cmp))
 {
   if (cmp) {
     if (STk_procedurep(cmp) != STk_true)
-      STk_error("bad comparison function ~S", cmp);
+      error_bad_comparison_function(cmp);
 
     LASSOC(PTR_CMPGEN);
  } else {
@@ -747,7 +800,7 @@ DEFINE_PRIMITIVE("last-pair", last_pair, subr1, (SCM l))
  * |Filter| returns all the elements of |list| that satisfy predicate
  * |pred|. The |list| is not disordered: elements that appear in the
  * result list occur in the same order as they occur in the argument
- * list. |Filter!| does the same job than |filter| by physically
+ * list. |Filter!| does the same job as |filter| by physically
  * modifying its |list| argument
  * @lisp
  * (filter even? '(0 7 8 8 43 -4)) => (0 8 8 -4)
@@ -825,7 +878,7 @@ DEFINE_PRIMITIVE("filter!", dfilter, subr2, (SCM pred, SCM list))
  *        (l2 (list 3))
  *        (l3 (list 4 5))
  *        (l4 (append! l1 l2 l3)))
- *   (list l1 l2 l3))  => ((1 2 3 4 5) (3 4 5) (4 5))
+ *   (list l1 l2 l3 l4))  => ((1 2 3 4 5) (3 4 5) (4 5) (1 2 3 4 5))
  * @end lisp
  * An error is signaled if one of the given lists is a constant list.
 doc>
@@ -879,7 +932,7 @@ DEFINE_PRIMITIVE("reverse!", dreverse, subr1, (SCM l))
   SCM tmp, p, prev;
 
   for(p=l, prev=STk_nil; !NULLP(p); prev=p, p=tmp) {
-    if (!CONSP(p)) STk_error("bad list ~W", l);
+    if (!CONSP(p)) error_bad_list(l);
     if (BOXED_INFO(p) & CONS_CONST) error_const_cell(p);
     tmp = CDR(p);
     CDR(p) = prev;
@@ -915,7 +968,7 @@ SCM STk_dremq(SCM obj, SCM list)
 
 /*
  *
- * Fast version of assq; for internal use only (alist must be well formed)
+ * Fast version of assq; for internal use only (alist must be well-formed)
  *
  */
 SCM STk_int_assq(SCM obj, SCM alist)
@@ -1005,6 +1058,7 @@ int STk_init_list(void)
   ADD_PRIMITIVE(cdr);
   ADD_PRIMITIVE(setcar);
   ADD_PRIMITIVE(setcdr);
+  ADD_PRIMITIVE(cxr);
   ADD_PRIMITIVE(nullp);
   ADD_PRIMITIVE(listp);
   ADD_PRIMITIVE(list);
@@ -1021,7 +1075,7 @@ int STk_init_list(void)
   ADD_PRIMITIVE(assv);
   ADD_PRIMITIVE(assoc);
   ADD_PRIMITIVE(list_copy);
- 
+
   ADD_PRIMITIVE(pair_mutable);
   ADD_PRIMITIVE(list_star);
   ADD_PRIMITIVE(last_pair);
