@@ -1,7 +1,7 @@
 /*
  * md5.c            -- MD5 algorithm
  *
- * Copyright © 2007-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@essi.fr>
+ * Copyright © 2007-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@essi.fr]
  *    Creation date: 13-May-2007 22:21 (eg)
- * Last file update:  1-Sep-2021 14:15 (eg)
  */
 
 /*
@@ -45,21 +44,21 @@ struct md5_context
 
 
 
-#define GET_UINT32(n,b,i)                                       \
-{                                                               \
-    (n) = (uint32) ((uint8 *) b)[(i)]                           \
-      | (((uint32) ((uint8 *) b)[(i)+1]) <<  8)                 \
-      | (((uint32) ((uint8 *) b)[(i)+2]) << 16)                 \
-      | (((uint32) ((uint8 *) b)[(i)+3]) << 24);                \
-}
+#define GET_UINT32(n,b,i)                                         \
+do{                                                               \
+    (n) = (uint32) ((uint8 *) (b))[(i)]                           \
+      | (((uint32) ((uint8 *) (b))[(i)+1]) <<  8)                 \
+      | (((uint32) ((uint8 *) (b))[(i)+2]) << 16)                 \
+      | (((uint32) ((uint8 *) (b))[(i)+3]) << 24);                \
+}while(0)
 
-#define PUT_UINT32(n,b,i)                                       \
-{                                                               \
-    (((uint8 *) b)[(i)]  ) = (uint8) (((n)      ) & 0xFF);      \
-    (((uint8 *) b)[(i)+1]) = (uint8) (((n) >>  8) & 0xFF);      \
-    (((uint8 *) b)[(i)+2]) = (uint8) (((n) >> 16) & 0xFF);      \
-    (((uint8 *) b)[(i)+3]) = (uint8) (((n) >> 24) & 0xFF);      \
-}
+#define PUT_UINT32(n,b,i)                                         \
+do{                                                               \
+    (((uint8 *) (b))[(i)]  ) = (uint8) (((n)      ) & 0xFF);      \
+    (((uint8 *) (b))[(i)+1]) = (uint8) (((n) >>  8) & 0xFF);      \
+    (((uint8 *) (b))[(i)+2]) = (uint8) (((n) >> 16) & 0xFF);      \
+    (((uint8 *) (b))[(i)+3]) = (uint8) (((n) >> 24) & 0xFF);      \
+}while(0)
 
 static void md5_starts( struct md5_context *ctx )
 {
@@ -71,7 +70,13 @@ static void md5_starts( struct md5_context *ctx )
     ctx->state[3] = 0x10325476;
 }
 
-static void md5_process( struct md5_context *ctx, uint8 data[64] )
+#if defined(__GNUC__) || defined(__CLANG__)
+#  define STAT64 static 64
+#else
+#  define STAT64 64
+#endif
+
+static void md5_process( struct md5_context *ctx, const uint8 data[STAT64] )
 {
     uint32 A, B, C, D, X[16];
 
@@ -280,7 +285,7 @@ static inline char hexchar(unsigned int v)
 <doc EXT md5sum
  * (md5sum obj)
  *
- * Return a string contening the md5 dum of |obj|. The given parameter can
+ * Return a string contening the md5 sum of |obj|. The given parameter can
  * be a string or an open input port.
 doc>
 */
