@@ -1,7 +1,7 @@
 /*                                                      -*- coding: utf-8 -*-
  * m i s c . c          -- Misc. functions
  *
- * Copyright © 2000-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2000-2024 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,19 +21,24 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  9-Jan-2000 12:50 (eg)
- * Last file update:  6-Sep-2021 17:28 (eg)
  */
 
+#include <limits.h>
 #include "stklos.h"
 #include "gnu-getopt.h"
 #include "git-info.h"
 
 #ifdef STK_DEBUG
-#include <execinfo.h>
+  #ifdef HAVE_BACKTRACE
+     #include <execinfo.h>
+  #endif
 
 int STk_interactive_debug = 0;
 #endif
 
+int STk_count_allocations = 0;         /* Set it to 1 to have GC accouniting */
+
+#define BSIZEOF(t) ((int) (sizeof(t) * CHAR_BIT))
 
 static void error_bad_string(SCM str)
 {
@@ -53,6 +58,33 @@ char *STk_strdup(const char *s)
 }
 
 
+void* STk_count_malloc(size_t size)
+{
+  STk_thread_inc_allocs(STk_current_thread(), size);
+  return GC_MALLOC(size);
+}
+
+
+void* STk_count_malloc_atomic(size_t size)
+{
+  STk_thread_inc_allocs(STk_current_thread(), size);
+  return GC_MALLOC_ATOMIC(size);
+}
+
+/* Getter and Setter for the count-allocation parameter */
+SCM get_count_allocs(void)
+{
+  return MAKE_BOOLEAN(STk_count_allocations);
+}
+
+SCM set_count_allocs(SCM value)
+{
+  STk_count_allocations = (value != STk_false);
+  return STk_void;
+}
+
+
+
 void STk_add_primitive(struct primitive_obj *o)
 {
   SCM symbol;
@@ -70,9 +102,7 @@ void STk_add_primitive_in_module(struct primitive_obj *o, SCM module)
 }
 
 
-
-
-SCM STk_eval_C_string(char *str, SCM module)
+SCM STk_eval_C_string(const char *str, SCM module)
 {
   SCM ref, eval = STk_lookup(STk_intern("eval-from-string"),
                              module,
@@ -82,15 +112,15 @@ SCM STk_eval_C_string(char *str, SCM module)
 }
 
 
-SCM STk_read_from_C_string(char *str)
+SCM STk_read_from_C_string(const char *str)
 {
-  return STk_read(STk_open_C_string(str), STk_read_case_sensitive);
+  return STk_read(STk_open_C_string(str), 1);
 }
 
 
 /*===========================================================================*\
  *
- * Primitives that don't feet anywhere else
+ * Primitives that don't fit anywhere else
  *
 \*===========================================================================*/
 /*
@@ -99,24 +129,83 @@ SCM STk_read_from_C_string(char *str)
  * (implementation-version)
  *
  * Returns a string identifying the current version of the system. A
- * version is constituted of two numbers separated by a point: the version
- * and the release numbers. Note that |implementation-version| corresponds
- * to the ,(srfi 112) name of this function.
+ * version is constituted of two (or three) numbers separated by a point:
+ * the version, the release numbers and, eventually, a patch number. The
+ * patch number is used for developments version only; it is absent for stable
+ * releases.
+ *
+ * Note that |implementation-version| corresponds to the {{link-srfi 112}} name of
+ * this function.
 doc>
  */
 DEFINE_PRIMITIVE("version", version, subr0, (void))
 {
-  return STk_Cstring2string(VERSION);
+  return STk_Cstring2string(FULL_VERSION);
 }
 
-DEFINE_PRIMITIVE("%push-id", push_id, subr0, (void))
+
+/*
+<doc EXT short-version
+ * (short-version)
+ *
+ * Returns a string identifying the current version of the system without
+ * its eventual patch number.
+doc>
+*/
+DEFINE_PRIMITIVE("short-version", short_version, subr0, (void))
 {
-  return STk_Cstring2string("abcdef");
+  return STk_Cstring2string(VERSION);  // version without patch number
+}
+
+
+DEFINE_PRIMITIVE("%stable-version?", stable_versionp, subr0, (void))
+{
+  return MAKE_BOOLEAN(strcmp(VERSION_STATUS, "stable") == 0);
 }
 
 DEFINE_PRIMITIVE("%stklos-configure", stklos_configure, subr0, (void))
 {
-  return STk_read_from_C_string(CONF_SUMMARY);
+  char buffer[2000];
+  SCM z = STk_nil;
+
+  /* The SRFI-176 c.version property */
+  z = STk_append2(z, LIST2(STk_makekey("c-version"),
+                           STk_Cstring2string(C_VERSION)));
+
+  /* The SRFI-176 c.compile property */
+  z = STk_append2(z, LIST2(STk_makekey("c-compile"),
+                           STk_read_from_C_string(C_COMPILE)));
+
+  /* The SRFI-176 c.link property */
+  z = STk_append2(z, LIST2(STk_makekey("c-link"),
+                           STk_read_from_C_string(C_LINK)));
+
+  /* The SRFI-176 c.type-bits property */
+  snprintf(buffer,
+           sizeof(buffer),
+           "(:c-type-bits ((char %d) (short %d) (int %d) (long %d) "
+           "(float %d) (double %d) (pointer %d)))",
+           BSIZEOF(char), BSIZEOF(short), BSIZEOF(int), BSIZEOF(long),
+           BSIZEOF(float), BSIZEOF(double), BSIZEOF(SCM));
+  z = STk_append2(z, STk_read_from_C_string(buffer));
+
+  /* The SRFI-176 stklos.shlib.compile property */
+  z = STk_append2(z, LIST2(STk_makekey("shlib-compile"),
+                           STk_read_from_C_string(SHARED_LIB_COMPILE)));
+
+  /* The SRFI-176 stklos.shlib.link property */
+  z = STk_append2(z, LIST2(STk_makekey("shlib-link"),
+                           STk_read_from_C_string(SHARED_LIB_LINK)));
+
+  /* The SRFI-176 stklos.shlib.suffix property */
+  z = STk_append2(z, LIST2(STk_makekey("shlib-suffix"),
+                           STk_Cstring2string(SHARED_EXTENSION)));
+
+
+  /* Add information gathered during configuration */
+  z = STk_append2(z, STk_read_from_C_string(CONF_SUMMARY));
+
+  return z;
 }
 
 DEFINE_PRIMITIVE("%stklos-git", stklos_git, subr0, (void))
@@ -129,7 +218,7 @@ DEFINE_PRIMITIVE("%stklos-git", stklos_git, subr0, (void))
  * (void)
  * (void arg1 ...)
  *
- * Returns the special ,(emph "void") object. If arguments are passed to |void|,
+ * Returns the special *_void_* object. If arguments are passed to |void|,
  * they are evalued and simply ignored.
 doc>
  */
@@ -139,23 +228,6 @@ DEFINE_PRIMITIVE("void", scheme_void, vsubr, (int _UNUSED(argc), SCM _UNUSED(*ar
 }
 
 
-/*
-<doc EXT address-of
- * (address-of obj)
- *
- * Returns the address of the object |obj| as an integer.
-doc>
-*/
-DEFINE_PRIMITIVE("address-of", address_of, subr1, (SCM object))
-{
-  char buffer[50];     /* should be sufficient for a while */
-
-  snprintf(buffer, sizeof(buffer),
-           "%lx", (unsigned long) object); /* not very efficient ... */
-  return STk_Cstr2number(buffer, 16L);
-}
-
-
 /*===========================================================================*\
  *
  * GC stuff
@@ -171,6 +243,21 @@ static void GC_warning_handler(char *msg, GC_word arg)
   if (strstr(msg, "Returning NULL")) STk_error("OUT of memory");
 }
 
+static inline int is_GC_allocated_adressp(void *addr)
+{
+  void *base = STk_gc_base(addr);
+  return base && (base == addr);
+}
+
+
+DEFINE_PRIMITIVE("%gc-stats", gc_stats, subr0, (void))
+{
+  // NOTE: For now we just return gc_no
+  struct GC_prof_stats_s stat;
+
+  GC_get_prof_stats(&stat, sizeof(stat));
+  return LIST2(STk_makekey("gc-no"), STk_long2integer(stat.gc_no));
+}
 
 
 void STk_gc_init(void)
@@ -195,6 +282,74 @@ DEFINE_PRIMITIVE("gc", scheme_gc, subr0, (void))
 }
 
 
+/*
+<doc EXT address-of address-ref
+ * (address-of obj)
+ * (address-ref n)
+ *
+ * |Address-of| returns the address of the object |obj| as an integer.
+ * |Address-ref| returns the object of which |n| is the address.
+ *
+ * @lisp
+ * (address-of "abc")               =>  140053283366272
+ * (address-of "abc")               =>  140053289472288 ;strings are not eq?
+ *
+ * (address-of 10)                  => 41
+ * (address-of 10)                  => 41
+ *
+ * (address-ref (address-of "xyz")) => "xyz"
+ *
+ * (address-ref 0)                  => error (points to nothing)
+ * @end lisp
+doc>
+*/
+
+void STk_verify_address(unsigned long addr, SCM object)
+{
+  unsigned long tag = addr & 3;
+
+  switch(tag) {
+    case 0:        /* 00 It's a pointer! */
+      if (!is_GC_allocated_adressp((void *) addr))
+        STk_error("bad object address ~s", object);
+      break;
+
+    case 1:
+      break; /* 01 Integers are always OK */
+
+    case 2:        /* 10 Small object (characters) */
+      /* We only allow characters as small objects */
+      if ((addr & 0x7) != 0x6) /* ...110 */
+        STk_error("bad small object address ~s", object);
+      break;
+
+    case 3:        /* 11 small constant */
+      if (addr > (unsigned long) STk_void)  /* #void is the last small constant */
+        STk_error("bad small constant address ~s", object);
+      break;
+  }
+}
+
+DEFINE_PRIMITIVE("address-of", address_of, subr1, (SCM object))
+{
+  char buffer[50];     /* should be sufficient for a while */
+
+  snprintf(buffer, sizeof(buffer),
+           "%lx", (unsigned long) object); /* not very efficient ... */
+  return STk_Cstr2number(buffer, 16L);
+}
+
+DEFINE_PRIMITIVE("address-ref", address_ref, subr1, (SCM object))
+{
+  unsigned long addr;
+
+  if (!INTP(object)) STk_error("bad integer ~s", object);
+  addr =  INT_VAL(object);
+
+  STk_verify_address(addr, object);
+  return (SCM) addr;
+}
+
 /*===========================================================================*\
  *
  *                      Argument parsing
@@ -211,8 +366,8 @@ DEFINE_PRIMITIVE("%initialize-getopt", init_getopt, subr3, (SCM argv, SCM s, SCM
   int i, len;
 
   STk_start_getopt_from_scheme();
-  optind = 1;    /* Initialize optind, since it has already be used
-                  * by ouserlves  before initializing the VM.
+  optind = 1;    /* Initialize optind, since it has already been used
+                  * by ourselves before initializing the VM.
                   */
 
   /*
@@ -313,33 +468,36 @@ static char URI_regexp[] =
 <doc EXT uri-parse
  * (uri-parse str)
  *
- * Parses the string |str| as a RFC-2396 URI and return a keyed list with the
+ * Parses the string |str| as an RFC-2396 URI and return a keyed list with the
  * following components
- * ,(itemize
- * (item [|scheme| : the scheme used as a string (defaults to |"file"|)])
- * (item [|user|: the user information (generally expressed as
- *      |login:password|)])
- * (item [|host| : the host as a string (defaults to "")])
- * (item [|port| : the port as an integer (0 if no port specified)])
- * (item [|path| : the path ])
- * (item [|query| : the qury part of the URI as a string (defaults to the
- * empty string)])
- * (item [|fragment| : the fragment of the URI as a string (defaults to the
- * empty string)])
- * )
+ *
+ * - |scheme| : the scheme used as a string (defaults to |"file"|)
+ * - |user|: the user information (generally expressed as |login:password|)
+ * - |host| : the host as a string (defaults to "")
+ * - |port| : the port as an integer (0 if no port specified)
+ * - |path| : the path
+ * - |query| : the qury part of the URI as a string (defaults to the
+ *    empty string)
+ * - |fragment| : the fragment of the URI as a string (defaults to the
+ *   empty string)
+ *
  * @lisp
- * (uri-parse "http://google.com")
- *     => (:scheme "http" :user "" :host "google.com" :port 80
+ * (uri-parse "https://stklos.net")
+{*    => (:scheme "https" :user "" :host "stklos.net" :port 443
  *         :path "/" :query "" :fragment "")
- * (uri-parse "http://stklos.net:8080/a/file?x=1;y=2#end")
+ *
+ * (uri-parse "https://stklos.net:8080/a/file?x=1;y=2#end")
  *     => (:scheme "http" :user "" :host "stklos.net" :port 8080
  *         :path "/a/file" :query "x=1;y=2" :fragment "end")
+ *
  * (uri-parse "http://foo:secret@stklos.net:2000/a/file")
  *     => (:scheme "http" :user "foo:secret" :host "stklos.net"
  *         :port 2000  :path "/a/file" :query "" :fragment "")
+ *
  * (uri-parse "/a/file")
  *    => (:scheme "file" :user "" :host "" :port 0 :path "/a/file"
  *        :query "" :fragment "")
+ *
  * (uri-parse "")
  *    => (:scheme "file"  :user "" :host "" :port 0 :path ""
  *        :query "" :fragment "")
@@ -381,7 +539,7 @@ DEFINE_PRIMITIVE("uri-parse", uri_parse, subr1, (SCM url_str))
     for (start = url; *url && *url != '/' && *url != '@'; url++) {
     }
     if (*url == '@') {
-      /* We have an user */
+      /* We have a user */
       user = STk_makestring(url-start, start);
       /* read the host now */
       for (start = ++url; *url && *url != '/' && *url != ':'; url++) {
@@ -407,8 +565,9 @@ DEFINE_PRIMITIVE("uri-parse", uri_parse, subr1, (SCM url_str))
     } else {
       char *scm = STRING_CHARS(scheme);
 
-      if (strcmp(scm, "http") == 0) port = MAKE_INT(80); else
-      if (strcmp(scm, "ftp") == 0) port = MAKE_INT(21);
+      if (strcmp(scm, "http") == 0)       port = MAKE_INT(80);
+      else if (strcmp(scm, "https") == 0) port = MAKE_INT(443);
+      else if (strcmp(scm, "ftp") == 0)   port = MAKE_INT(21);
       else port = MAKE_INT(0);
     }
     if (*url) url += 1;
@@ -458,7 +617,7 @@ Error:
  * (string->html str)
  *
  * This primitive is a convenience function; it returns a string where
- * the HTML special chars are properly translated. It can easily written
+ * the HTML special chars are properly translated. It can easily be written
  * in Scheme, but this version is fast.
  * @lisp
  * (string->html "Just a <test>")
@@ -554,7 +713,7 @@ DEFINE_PRIMITIVE("get-password", getpass, subr0, (void))
 
 #define BACKTRACE_SIZE 1024
 
-DEFINE_PRIMITIVE("%debug", set_debug, subr0, (void))
+DEFINE_PRIMITIVE("%%debug", set_debug, subr0, (void))
 {
   STk_interactive_debug = !STk_interactive_debug;
   STk_debug("Debug mode %d", STk_interactive_debug);
@@ -569,6 +728,7 @@ DEFINE_PRIMITIVE("%test", test, subr1, (SCM s))
 
 DEFINE_PRIMITIVE("%c-backtrace", c_backtrace, subr0, (void))
 {
+# ifdef HAVE_BACKTRACE
   void *buffer[BACKTRACE_SIZE];
   int n;
 
@@ -578,6 +738,9 @@ DEFINE_PRIMITIVE("%c-backtrace", c_backtrace, subr0, (void))
   }
 
   backtrace_symbols_fd(buffer, n, 2);
+#else
+  STk_debug("backtrace is not available on this system. Sorry.");
+#endif
   return STk_void;
 }
 #endif
@@ -590,12 +753,15 @@ DEFINE_PRIMITIVE("%c-backtrace", c_backtrace, subr0, (void))
 int STk_init_misc(void)
 {
   ADD_PRIMITIVE(version);
-  ADD_PRIMITIVE(push_id);
+  ADD_PRIMITIVE(short_version);
+  ADD_PRIMITIVE(stable_versionp);
   ADD_PRIMITIVE(stklos_configure);
   ADD_PRIMITIVE(stklos_git);
   ADD_PRIMITIVE(scheme_void);
   ADD_PRIMITIVE(address_of);
+  ADD_PRIMITIVE(address_ref);
   ADD_PRIMITIVE(scheme_gc);
+  ADD_PRIMITIVE(gc_stats);
 
   ADD_PRIMITIVE(init_getopt);
   ADD_PRIMITIVE(getopt);
@@ -604,6 +770,9 @@ int STk_init_misc(void)
   ADD_PRIMITIVE(uri_parse);
   ADD_PRIMITIVE(str2html);
 
+  STk_make_C_parameter2("%count-allocations", get_count_allocs, set_count_allocs,
+                        STk_STklos_module);
+
 #ifdef STK_DEBUG
   ADD_PRIMITIVE(set_debug);
   ADD_PRIMITIVE(test);
