@@ -2,7 +2,7 @@
  *
  *  o b j e c t . c                     -- Objects support
  *
- * Copyright © 1994-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1994-2023 Erick Gallesio <eg@stklos.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
  *
  *            Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  9-Feb-1994 15:56
- * Last file update: 29-Apr-2021 19:12 (eg)
  */
 
 #include "stklos.h"
@@ -60,8 +59,10 @@ EXTERN_PRIMITIVE("class-of", class_of, subr1, (SCM obj));
 static SCM Top, Object, Class, Generic, Method, Simple_method, Accessor,
            Procedure_class, Entity_class;
 static SCM Boolean, Char, Pair, Procedure, String, Symbol, Vector, Number,
-           Liste, Null, Real, Complex, Rational, Integer, Keyword, Eof,
-  Struct_type, Struct, Cond, Cond_type, Box, UnknownClass;
+           Liste, Null, Real, Complex, Rational, Integer, Fixnum, Bignum,
+           Eof, Keyword, Struct_type, Struct, Cond, Cond_type, Box, Syntax,
+           Uvector, Bytevector, Port, Input_port, Output_port, Hash_table,
+           Parameter, UnknownClass;
 
 
 int STk_oo_initialized = FALSE;
@@ -183,7 +184,7 @@ static int applicablep(SCM actual, SCM formal)
 
   /* We test that (memq formal (slot-ref actual 'cpl))
    * However, we don't call memq here since we already know that
-   * the list is well formed
+   * the list is well-formed
    */
   for (ptr=INST_SLOT(actual, S_cpl); !NULLP(ptr); ptr = CDR(ptr)) {
     if (CAR(ptr) == formal) return TRUE;
@@ -411,7 +412,7 @@ DEFINE_PRIMITIVE("%method-more-specific?", method_more_specificp, subr3,
  *
 \*===========================================================================*/
 
-static Inline SCM test_change_class(SCM obj)
+static inline SCM test_change_class(SCM obj)
 {
   SCM classe = INST_CLASS_OF(obj);
 
@@ -422,7 +423,7 @@ static Inline SCM test_change_class(SCM obj)
 
 
 
-static Inline SCM get_slot_value(SCM classe, SCM obj, SCM slot_name)
+static inline SCM get_slot_value(SCM classe, SCM obj, SCM slot_name)
 {
   SCM l;
 
@@ -443,7 +444,7 @@ static Inline SCM get_slot_value(SCM classe, SCM obj, SCM slot_name)
   return CALL_GF3("slot-missing", classe, obj, slot_name);
 }
 
-static Inline SCM set_slot_value(SCM classe, SCM obj, SCM slot_name, SCM value)
+static inline SCM set_slot_value(SCM classe, SCM obj, SCM slot_name, SCM value)
 {
   register SCM l;
 
@@ -550,7 +551,7 @@ DEFINE_PRIMITIVE("%slot-ref", undoc_slot_ref, subr2, (SCM obj, SCM slot_name))
  * initialize-object
  *
  ******************************************************************************/
-// static void Inline set_slot_value_if_unbound
+// static void inline set_slot_value_if_unbound
 //                      (SCM classe, SCM obj, SCM slot_name, SCM val)
 // {
 // #ifdef FIXME
@@ -793,7 +794,7 @@ static SCM compute_getters_n_setters(SCM slots)
   /* Build a kind of A-list which is something like
    *     ( .... (slot-name #f . 3) ... )
    * where #f is the slot initialization function and 3 is the offset of a slot
-   * in a the vector of slots
+   * in a vector of slots
    */
   for (  ; !NULLP(slots); slots = CDR(slots))
     res = STk_cons(STk_cons(CAR(slots),
@@ -901,20 +902,30 @@ static void make_standard_classes(void)
   mk_cls(&String,       "<string>",     Class,           Top,       STk_nil);
   mk_cls(&Symbol,       "<symbol>",     Class,           Top,       STk_nil);
   mk_cls(&Vector,       "<vector>",     Class,           Top,       STk_nil);
+  mk_cls(&Uvector,      "<uvector>",    Class,           Top,       STk_nil);
+  mk_cls(&Bytevector,   "<bytevector>", Class,           Uvector,   STk_nil);
   mk_cls(&Number,       "<number>",     Class,           Top,       STk_nil);
   mk_cls(&Complex,      "<complex>",    Class,           Number,    STk_nil);
   mk_cls(&Real,         "<real>",       Class,           Complex,   STk_nil);
   mk_cls(&Rational,     "<rational>",   Class,           Real,      STk_nil);
   mk_cls(&Integer,      "<integer>",    Class,           Rational,  STk_nil);
+  mk_cls(&Bignum,       "<bignum>",     Class,           Integer,   STk_nil);
+  mk_cls(&Fixnum,       "<fixnum>",     Class,           Integer,   STk_nil);
   mk_cls(&Keyword,      "<keyword>",    Class,           Top,       STk_nil);
   mk_cls(&Eof,          "<eof>",        Class,           Top,       STk_nil);
   mk_cls(&Struct,       "<struct>",     Class,           Top,       STk_nil);
   mk_cls(&Struct_type,  "<struct-type>",Class,           Top,       STk_nil);
   mk_cls(&Cond,         "<condition>",  Class,           Top,       STk_nil);
   mk_cls(&Cond_type,    "<condition-type>",Class,        Top,       STk_nil);
-  mk_cls(&Box,          "<ref>",        Class,           Top,       STk_nil);
+  mk_cls(&Box,          "<box>",        Class,           Top,       STk_nil);
   mk_cls(&UnknownClass, "<unknown>",    Class,           Top,       STk_nil);
   mk_cls(&Procedure,    "<procedure>",  Procedure_class, Top,       STk_nil);
+  mk_cls(&Parameter,    "<parameter>",  Procedure_class, Procedure, STk_nil);
+  mk_cls(&Syntax,       "<syntax>",     Class,           Top,       STk_nil);
+  mk_cls(&Port,         "<port>",       Class,           Top,       STk_nil);
+  mk_cls(&Input_port,   "<input-port>", Class,           Port,      STk_nil);
+  mk_cls(&Output_port,  "<output-port>",Class,           Port,      STk_nil);
+  mk_cls(&Hash_table,   "<hash-table>", Class,           Top,       STk_nil);
 }
 
 
@@ -985,6 +996,18 @@ DEFINE_PRIMITIVE("method?", methodp, subr1, (SCM obj))
   return MAKE_BOOLEAN(METHODP(obj));
 }
 
+/*
+<doc EXT class-of
+ * (class-of obj)
+ *
+ * Return the class of object |obj|.
+ * @lisp
+ * (define-class <A> () ())
+ * (class-of (make <A>))     => #[<class> <A> 7f5dc5002a20]
+ * (class-of #t)             => #[<class> <boolean> 7f5dc551ade0]
+ * @end lisp
+doc>
+*/
 DEFINE_PRIMITIVE("class-of", class_of, subr1, (SCM obj))
 {
   if (INSTANCEP(obj)) {
@@ -1005,7 +1028,7 @@ DEFINE_PRIMITIVE("class-of", class_of, subr1, (SCM obj))
   }
 
   if (INTP(obj))
-    return Integer;
+    return Fixnum;
 
   if (CHARACTERP(obj))
     return Char;
@@ -1018,12 +1041,18 @@ DEFINE_PRIMITIVE("class-of", class_of, subr1, (SCM obj))
     case tc_real:       return Real;
     case tc_complex:    return Complex;
     case tc_rational:   return Rational;
-    case tc_bignum:     return Integer;
+    case tc_bignum:     return Bignum;
     case tc_keyword:    return Keyword;
     case tc_struct_type:return (COND_TYPEP(obj)) ? Cond_type: Struct_type;
     case tc_struct:     return (CONDP(obj)) ? Cond : Struct;
     case tc_box:        return Box;
-  default: ;
+    case tc_syntax:     return Syntax;
+    case tc_uvector:    return (UVECTOR_TYPE(obj) == UVECT_U8) ?
+                               Bytevector: Uvector;
+    case tc_port:       return IPORTP(obj)? Input_port: Output_port;
+    case tc_hash_table: return Hash_table;
+    case tc_parameter:  return Parameter;
+    default: ;
   }
 
   if (STk_procedurep(obj) == STk_true)
