@@ -1,7 +1,7 @@
 /*
  * parameter.c  -- Parameter Objects (SRFI-39)
  *
- * Copyright © 2003-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2003-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@essi.fr]
  *    Creation date:  1-Jul-2003 11:38 (eg)
- * Last file update: 10-Apr-2021 18:44 (eg)
  */
 
 
@@ -34,6 +33,7 @@ struct parameter_obj {
   int C_type;           /* 0: parameter is expressed in Scheme                    */
                         /* 1: Converter is expressed in C rather than in Scheme   */
                         /* 2: idem and getter is a procedure to call to get value */
+  SCM name;             /* #f, a string or a symbol */
   SCM value;
   SCM converter;
   SCM (*getter)(void);  /* Used only for type 2 parameter objects */
@@ -42,6 +42,7 @@ struct parameter_obj {
 
 #define PARAMETERP(o)       (BOXED_TYPE_EQ((o), tc_parameter))
 #define PARAMETER_C_TYPE(p) (((struct parameter_obj *) (p))->C_type)
+#define PARAMETER_NAME(p)   (((struct parameter_obj *) (p))->name)
 #define PARAMETER_VALUE(p)  (((struct parameter_obj *) (p))->value)
 #define PARAMETER_CONV(p)   (((struct parameter_obj *) (p))->converter)
 #define PARAMETER_GETTER(p) (((struct parameter_obj *) (p))->getter)
@@ -57,16 +58,16 @@ struct parameter_obj {
  *
 \*===========================================================================*/
 
-static void error_bad_parameter(SCM obj)
+
+static inline void verify_parameter(SCM obj)
 {
-  STk_error("bad parameter ~S", obj);
+  if (!PARAMETERP(obj)) STk_error("bad parameter ~S", obj);
 }
 
 
 SCM STk_get_parameter(SCM param)
 {
-  if (!PARAMETERP(param)) error_bad_parameter(param);
-
+  verify_parameter(param);
   return (PARAMETER_C_TYPE(param) == 2) ?
     PARAMETER_GETTER(param)():
     PARAMETER_VALUE(param);
@@ -76,7 +77,7 @@ SCM STk_set_parameter(SCM param, SCM value)
 {
   SCM conv, new;
 
-  if (!PARAMETERP(param)) error_bad_parameter(param);
+  verify_parameter(param);
 
   conv = PARAMETER_CONV(param);
 
@@ -95,43 +96,54 @@ SCM STk_set_parameter(SCM param, SCM value)
   return STk_void;
 }
 
-SCM STk_make_C_parameter(SCM symbol, SCM value, SCM (*conv)(SCM new_value),
-             SCM module)
+SCM STk_make_C_parameter(char *name, SCM value, SCM (*conv)(SCM new_value),
+                         SCM module)
 {
   SCM z;
 
   NEWCELL(z, parameter);
   PARAMETER_C_TYPE(z) = 1;
+  PARAMETER_NAME(z)   = STk_Cstring2string(name);
   PARAMETER_VALUE(z)  = conv(value);
   PARAMETER_CONV(z)   = (SCM) conv;
   PARAMETER_GETTER(z) = STk_void;
   MUT_INIT(PARAMETER_MUTEX(z));
 
   /* Bind it to the given symbol */
-  STk_define_variable(STk_intern(symbol), z, module);
+  STk_define_variable(STk_intern((char *)name), z, module);
 
   return z;
 }
 
-SCM STk_make_C_parameter2(SCM symbol, SCM (*getter)(void), SCM (*conv)(SCM new_value),
-              SCM module)
+SCM STk_make_C_parameter2(char *name, SCM (*getter)(void),
+                          SCM (*conv)(SCM new_value), SCM module)
 {
   SCM z;
 
   NEWCELL(z, parameter);
   PARAMETER_C_TYPE(z) = 2;
+  PARAMETER_NAME(z)   = STk_Cstring2string(name);
   PARAMETER_VALUE(z)  = getter();
   PARAMETER_CONV(z)   = (SCM) conv;
   PARAMETER_GETTER(z) = getter;
   MUT_INIT(PARAMETER_MUTEX(z));
 
   /* Bind it to the given symbol */
-  STk_define_variable(STk_intern(symbol), z, module);
+  STk_define_variable(STk_intern(name), z, module);
 
   return z;
 }
 
+static void print_parameter(SCM param, SCM port, int _UNUSED(mode))
+{
+  SCM name = PARAMETER_NAME(param);
 
+  STk_fprintf(port, "#[parameter ");
+  if (name != STk_false)
+    STk_fprintf(port, "%s]", STRINGP(name)? STRING_CHARS(name): SYMBOL_PNAME(name));
+  else
+    STk_fprintf(port, "%lx]", param);
+}
 
 /*===========================================================================*\
  *
@@ -139,6 +151,25 @@ SCM STk_make_C_parameter2(SCM symbol, SCM (*getter)(void), SCM (*conv)(SCM new_v
  *
 \*===========================================================================*/
 
+DEFINE_PRIMITIVE("%parameter-name", parameter_name, subr1, (SCM obj))
+{
+  verify_parameter(obj);
+  {
+    SCM name = PARAMETER_NAME(obj);
+    return SYMBOLP(name) ? STk_Cstring2string(SYMBOL_PNAME(name)) : name;
+  }
+}
+
+DEFINE_PRIMITIVE("%set-parameter-name!", set_parameter_name, subr2, (SCM obj, SCM n))
+{
+  verify_parameter(obj);
+  if (!SYMBOLP(n) && !STRINGP(n)) STk_error("bad parameter name ~S", n);
+  PARAMETER_NAME(obj) = n;
+  return STk_void;
+}
+
+
+
 /*
 <doc EXT make-parameter
  * (make-parameter init)
@@ -148,7 +179,7 @@ SCM STk_make_C_parameter2(SCM symbol, SCM (*getter)(void), SCM (*conv)(SCM new_v
  * environment to a cell containing the value returned by the call
  * |(converter init)|. If the conversion procedure |converter| is not
  * specified the identity function is used instead.
- * @l
+ *
  * The parameter object is a procedure which accepts zero or one
  * argument. When it is called with no argument, the content of the
  * cell bound to this parameter object in the current dynamic
@@ -156,8 +187,7 @@ SCM STk_make_C_parameter2(SCM symbol, SCM (*getter)(void), SCM (*conv)(SCM new_v
  * content of the cell bound to this parameter object in the current
  * dynamic environment is set to the result of the call
  * |(converter arg)|, where |arg| is the argument passed to the
- * parameter object, and
- * an unspecified value is returned.
+ * parameter object, and an unspecified value is returned.
  *
  * @lisp
  * (define radix
@@ -202,6 +232,7 @@ DEFINE_PRIMITIVE("make-parameter", make_parameter, subr12, (SCM value, SCM conv)
 
   NEWCELL(z, parameter);
   PARAMETER_C_TYPE(z) = 0;
+  PARAMETER_NAME(z)   = STk_false;
   PARAMETER_VALUE(z)  = v;
   PARAMETER_CONV(z)   = (conv) ? conv : STk_false;
   PARAMETER_GETTER(z) = STk_void;
@@ -215,7 +246,7 @@ DEFINE_PRIMITIVE("make-parameter", make_parameter, subr12, (SCM value, SCM conv)
 <doc EXT parameter?
  * (parameter? obj)
  *
- *  Returns |#t| if |obj| is a parameter object, otherwise returns |#f|.
+ * Returns |#t| if |obj| is a parameter object, otherwise returns |#f|.
 doc>
  */
 DEFINE_PRIMITIVE("parameter?", parameterp, subr1, (SCM obj))
@@ -230,7 +261,10 @@ DEFINE_PRIMITIVE("parameter?", parameterp, subr1, (SCM obj))
  *
 \*===========================================================================*/
 
-static struct extended_type_descr xtype_parameter = { .name = "parameter" };
+static struct extended_type_descr xtype_parameter = {
+  .name = "parameter",
+  .print = print_parameter
+};
 
 int STk_init_parameter(void)
 {
@@ -240,6 +274,8 @@ int STk_init_parameter(void)
   /* Define primitives */
   ADD_PRIMITIVE(make_parameter);
   ADD_PRIMITIVE(parameterp);
+  ADD_PRIMITIVE(parameter_name);
+  ADD_PRIMITIVE(set_parameter_name);
 
   return TRUE;
 }
