@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  9-Jan-2000 14:25 (eg)
- * Last file update: 24-Sep-2021 09:08 (eg)
  */
 
 #include "stklos.h"
@@ -36,9 +35,9 @@
  *
 \*===========================================================================*/
 
-static void tilde_expand(char *name, char *result, size_t result_len)
+static void tilde_expand(const char *name, char *result, size_t result_len)
 {
-  char *p;
+  const char *p;
   struct passwd *pwPtr;
 
   if (name[0] != '~') {
@@ -144,7 +143,7 @@ static void absolute(char *s, char *pathname)
 \*===========================================================================*/
 #define MAXLINK 50  /* Number max of link before declaring we have a loop */
 
-SCM STk_resolve_link(char *path, int count)
+SCM STk_resolve_link(const char *path, int count)
 {
 #ifdef WIN32
   return STk_Cstring2string(STk_expand_file_name(path));
@@ -203,7 +202,7 @@ SCM STk_resolve_link(char *path, int count)
  * STk_expand_file_name
  *
 \*===========================================================================*/
-char *STk_expand_file_name(char *s)
+char *STk_expand_file_name(const char *s)
 {
   char expanded[2 * MAX_PATH_LENGTH], abs[2 * MAX_PATH_LENGTH];
   /* Warning: absolute makes no control about path overflow. Hence the "2 *" */
