@@ -1,7 +1,7 @@
 /*
  * p r i n t . c                                -- writing stuff
  *
- * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,19 +21,21 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: ??-Oct-1993 ??:??
- * Last file update: 28-Sep-2021 11:53 (eg)
  *
  */
 #include <ctype.h>
 #include "stklos.h"
 
+#define MAX_BUF_SIZE 1024    /* Buffer size for printing strings */
+
+
 static int pretty_quotes = 1;
 
 
 static void printlist(SCM exp, SCM port, int mode)
 {
   register SCM tmp;
-  char *s;
+  const char *s;
 
   if (pretty_quotes) {
     /* Special case for pretty printing of quoted expressions */
@@ -60,9 +62,9 @@ static void printlist(SCM exp, SCM port, int mode)
 }
 
 
-static Inline void printsymbol(SCM symb, SCM port, int mode)
+static inline void printsymbol(SCM symb, SCM port, int mode)
 {
-  char *s = SYMBOL_PNAME(symb);
+  const char *s = SYMBOL_PNAME(symb);
 
   if ((mode==WRT_MODE) &&
       ((BOXED_INFO(symb) & SYMBOL_NEEDS_BARS) ||
@@ -85,7 +87,7 @@ static Inline void printsymbol(SCM symb, SCM port, int mode)
         STk_putc('\\', port);
         STk_putc(c, port);
       }
-      else 
+      else
         STk_putc(*s, port);
    }
     STk_putc('|', port);
@@ -93,9 +95,9 @@ static Inline void printsymbol(SCM symb, SCM port, int mode)
     STk_puts(s, port);
 }
 
-static Inline void printkeyword(SCM key, SCM port, int mode)
+static inline void printkeyword(SCM key, SCM port, int mode)
 {
-  char *s = KEYWORD_PNAME(key);
+  const char *s = KEYWORD_PNAME(key);
 
   if (mode==WRT_MODE) {
     if ((BOXED_INFO(key) & SYMBOL_NEEDS_BARS) ||
@@ -109,7 +111,7 @@ static Inline void printkeyword(SCM key, SCM port, int mode)
 }
 
 
-static Inline char printhexa(int x)
+static inline char printhexa(int x)
 {
   return (x >= 10) ? (x - 10 + 'a') : (x + '0');
 }
@@ -122,11 +124,11 @@ static void printstring(SCM s, SCM port, int mode)
   } else {
     register char *p    = STRING_CHARS(s);
     register size_t len = STRING_SIZE(s);
-    char buffer[MAX_TOKEN_SIZE], *buff = buffer;
+    char buffer[MAX_BUF_SIZE], *buff = buffer;
 
     *buff++ = '"';
     for (   ; len; len--, p++) {
-      if (buff >= buffer + MAX_TOKEN_SIZE - 7) { /* 7 because we can add \X" and */
+      if (buff >= buffer + MAX_BUF_SIZE - 7) {   /* 7 because we can add \X" and */
         /* buffer is full. Flush it */           /* a null char at this positon  */
         *buff = '\0';
         STk_puts(buffer, port);
@@ -157,7 +159,7 @@ static void printstring(SCM s, SCM port, int mode)
                       if (printable)
                         *buff++ = *p;
                       else {
-                        /* Non printable char. (It works only for char < 0xFF !!) */
+                        /* Non-printable char. (It works only for char < 0xFF !!) */
                         *buff++ = '\\';
                         *buff++ = 'x';
                         *buff++ = printhexa((unsigned char) *p / 16);
@@ -272,6 +274,7 @@ void STk_print(SCM exp, SCM port, int mode)
     case tc_subr01:
     case tc_subr12:
     case tc_subr23:
+    case tc_subr34:
     case tc_vsubr:
     case tc_apply:
       STk_puts("#[primitive ", port);
@@ -294,7 +297,7 @@ void STk_print(SCM exp, SCM port, int mode)
         struct extended_type_descr *xdescr = BOXED_XTYPE(exp);
 
         if (xdescr) {
-          void (*p)() = XTYPE_PRINT(xdescr);
+          void (*p)(SCM, SCM, int) = XTYPE_PRINT(xdescr);
 
           if (p)
             /* Use the defined function */
