@@ -2,7 +2,7 @@
  *
  * p r o c . c                          -- Things about procedures
  *
- * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2023 Erick Gallesio <eg@stklos.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,12 +21,16 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 15-Nov-1993 22:02
- * Last file update: 18-Aug-2021 16:22 (eg)
  */
 
 #include "stklos.h"
 #include "object.h"
 
+
+/* Keywords used in procedure plists */
+SCM STk_key_source, STk_key_formals, STk_key_doc;
+
+
 /*===========================================================================*\
  *
  *                                  Utilities
@@ -50,14 +54,12 @@ SCM STk_make_closure(STk_instr *code, int size, int arity, SCM *cst, SCM env)
 
   NEWCELL(z, closure);
   CLOSURE_ENV(z)   = env;
-  CLOSURE_FORMALS(z) = STk_false;
   CLOSURE_PLIST(z) = STk_nil;
   CLOSURE_NAME(z)  = STk_false;
   CLOSURE_ARITY(z) = arity;
   CLOSURE_CONST(z) = cst;
   CLOSURE_BCODE(z) = code;
   CLOSURE_SIZE(z)  = size;
-  CLOSURE_DOC(z)   = STk_false;
   return z;
 }
 
@@ -69,7 +71,7 @@ static void print_lambda(SCM closure, SCM port, int mode)
   else
     STk_fprintf(port, "#[closure %lx", (unsigned long) closure);
 
-  SCM formals = CLOSURE_FORMALS(closure);
+  SCM formals = STk_key_get(CLOSURE_PLIST(closure), STk_key_formals, STk_false);
   if (formals != STk_false) {
     STk_nputs(port, " ", 1);
     STk_print(formals, port, mode);
@@ -116,6 +118,7 @@ DEFINE_PRIMITIVE("procedure?", procedurep, subr1, (SCM obj))
     case tc_subr01:
     case tc_subr12:
     case tc_subr23:
+    case tc_subr34:
     case tc_vsubr:
     case tc_apply:
     case tc_next_method:
@@ -150,6 +153,7 @@ DEFINE_PRIMITIVE("%procedure-name", procedure_name, subr1, (SCM obj))
     case tc_subr01:
     case tc_subr12:
     case tc_subr23:
+    case tc_subr34:
     case tc_vsubr:
     case tc_apply:    return STk_Cstring2string(PRIMITIVE_NAME(obj));
 #ifdef HAVE_FFI
@@ -176,8 +180,25 @@ DEFINE_PRIMITIVE("%set-procedure-name!", set_procedure_name, subr2, (SCM obj, SC
 <doc EXT closure?
  * (closure? obj)
  *
- * Returns |#t| if |obj| is a procedure created with the |lambda| syntax and
- * |#f| otherwise.
+ * Returns |#t| if |obj| is a procedure created with the |lambda|
+ * syntax and |#f| otherwise.
+ *
+ * Note that primitive procedures (those which are written in C) are
+ * *not* closures:
+ *
+ * @lisp
+ * (define (cube x) (* x x x))
+ * (closure? cube)               => #t
+ *
+ * (define square-root sqrt)
+ * (eq? square-root sqrt)        => #t
+ * (closure? square-root)        => #f
+ *
+ * (closure? 10)                 => #f
+ * (closure? display)            => #f
+ * (closure? (lambda (x) (- x))) => #t
+ * (closure? any)                => #t
+ * @end lisp
 doc>
  */
 DEFINE_PRIMITIVE("closure?", closurep, subr1, (SCM obj))
@@ -198,6 +219,7 @@ DEFINE_PRIMITIVE("%procedure-plist", proc_plist, subr1, (SCM obj))
     case tc_subr01:
     case tc_subr12:
     case tc_subr23:
+    case tc_subr34:
     case tc_vsubr:   return PRIMITIVE_PLIST(obj);
     case tc_closure: return CLOSURE_PLIST(obj);
     default:         error_bad_procedure(obj);
@@ -217,6 +239,7 @@ DEFINE_PRIMITIVE("%set-procedure-plist!", set_proc_plist, subr2, (SCM obj, SCM v
     case tc_subr01:
     case tc_subr12:
     case tc_subr23:
+    case tc_subr34:
     case tc_vsubr:   PRIMITIVE_PLIST(obj) = v; break;
     case tc_closure: CLOSURE_PLIST(obj)   = v; break;
     default:         error_bad_procedure(obj);
@@ -239,6 +262,7 @@ DEFINE_PRIMITIVE("%procedure-arity", proc_arity, subr1, (SCM proc))
     case tc_subr01:       res = -1; break;
     case tc_subr12:       res = -2; break;
     case tc_subr23:       res = -3; break;
+    case tc_subr34:       res = -4; break;
     case tc_vsubr:        res = -1; break;
     case tc_apply:        res = -1; break;
       /*  case tc_next_method: */
@@ -273,13 +297,69 @@ DEFINE_PRIMITIVE("%procedure-code", proc_code, subr1, (SCM proc))
 DEFINE_PRIMITIVE("%procedure-doc", proc_doc, subr1, (SCM proc))
 {
   if (!CLOSUREP(proc)) return STk_false;
-  return CLOSURE_DOC(proc);
+  return STk_key_get(CLOSURE_PLIST(proc), STk_key_doc, STk_false);
 }
 
-DEFINE_PRIMITIVE("%procedure-signature", proc_signature, subr1, (SCM proc))
+DEFINE_PRIMITIVE("%procedure-environment", proc_env, subr1, (SCM proc))
 {
   if (!CLOSUREP(proc)) return STk_false;
-  return CLOSURE_FORMALS(proc);
+  return CLOSURE_ENV(proc);
+}
+
+/*
+<doc EXT procedure-formals
+ * (procedure-formals proc)
+ *
+ * Returns the formal parameters of procedure |proc|.
+ * Note that procedure formal parameters are kept in memory only if
+ * the compiler flag <<"compiler:keep-formals">> is set at its creation.
+ * If |proc| formal parameters are not available, |procedure-formals|
+ * returns |#f|.
+ *
+ * @lisp
+ * (compiler:keep-formals #t)
+ *
+ * (define (f x y) (+ (* 3 x) y))
+ * (procedure-formals f)           => (x y)
+ * @end lisp
+doc>
+ */
+DEFINE_PRIMITIVE("procedure-formals", proc_formals, subr1, (SCM proc))
+{
+  if (!CLOSUREP(proc)) error_bad_procedure(proc);
+  return STk_key_get(CLOSURE_PLIST(proc), STk_key_formals, STk_false);
+}
+
+/*
+<doc EXT procedure-source
+ * (procedure-source proc)
+ *
+ * Returns the source form used to define procedure |proc|.
+ * Note that procedure source is kept in memory only if the compiler flag
+ * <<"compiler:keep-source">> is set at its creation. If |proc| source is
+ * not available, |procedure-source| returns |#f|.
+ *
+ * @lisp
+ * (compiler:keep-source #t)
+ *
+ * (define (f x y) (+ (* 3 x) y))
+ * (procedure-source f)           => (lambda (x y) (+ (* 3 x) y))
+ * @end lisp
+doc>
+ */
+DEFINE_PRIMITIVE("procedure-source", proc_source, subr1, (SCM proc))
+{
+  if (!CLOSUREP(proc)) error_bad_procedure(proc);
+  else {
+    SCM args = STk_key_get(CLOSURE_PLIST(proc), STk_key_formals, STk_false);
+    SCM body = STk_key_get(CLOSURE_PLIST(proc), STk_key_source,  STk_false);
+
+    if (args != STk_false && body != STk_false) {
+      return STk_cons(STk_intern("lambda"),
+                      STk_cons(args, body));
+    }
+  }
+  return STk_false;
 }
 
 /*===========================================================================*\
@@ -371,7 +451,7 @@ DEFINE_PRIMITIVE("map", map, vsubr, (int argc, SCM* argv))
  * calls proc for its side effects rather than for its values.
  * Unlike |map|, |for-each| is guaranteed to call proc on the elements of
  * the lists in order from the first element(s) to the last, and the value
- * returned by |for-each| is ,(emph "void").
+ * returned by |for-each| is *_void_*.
  * @lisp
  * (let ((v (make-vector 5)))
  *   (for-each (lambda (i)
@@ -388,20 +468,13 @@ DEFINE_PRIMITIVE("for-each", for_each, vsubr, (int argc, SCM* argv))
 }
 
 
-#ifdef FIXME
-// DEFINE_PRIMITIVE("make-expander", make_expander, subr1, (SCM form))
-// {
-//   SCM eval, compiled_form, ref;
-//
-//   eval               = STk_lookup(STk_intern("eval"), STk_current_module, &ref);
-//   compiled_form = STk_C_apply(eval, 1 , form);
-//
-//   return compiled_form;
-// }
-#endif
-
 int STk_init_proc(void)
 {
+  // Define some keywords to avoid calls to STk_makekey (which uses a mutex!)
+  STk_key_source  = STk_makekey("source");
+  STk_key_formals = STk_makekey("formals");
+  STk_key_doc     = STk_makekey("documentation");
+
   DEFINE_XTYPE(closure, &xtype_closure);
   ADD_PRIMITIVE(procedurep);
   ADD_PRIMITIVE(closurep);
@@ -409,13 +482,14 @@ int STk_init_proc(void)
   ADD_PRIMITIVE(set_proc_plist);
   ADD_PRIMITIVE(proc_code);
   ADD_PRIMITIVE(proc_doc);
+  ADD_PRIMITIVE(proc_source);
   ADD_PRIMITIVE(proc_arity);
   ADD_PRIMITIVE(procedure_name);
   ADD_PRIMITIVE(set_procedure_name);
-  ADD_PRIMITIVE(proc_signature);
+  ADD_PRIMITIVE(proc_formals);
+  ADD_PRIMITIVE(proc_env);
 
   ADD_PRIMITIVE(map);
   ADD_PRIMITIVE(for_each);
-  /* //FIXME  ADD_PRIMITIVE(make_expander); */
   return TRUE;
 }
