@@ -1,7 +1,7 @@
 /*
  * r e a d  . c                         -- reading stuff
  *
- * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2024 Erick Gallesio <eg@stklos.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,14 +20,24 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: ??-Oct-1993 ??:??
- * Last file update: 28-Sep-2021 07:51 (eg)
  *
  */
 
 #include <ctype.h>
 #include "stklos.h"
+#include "hash.h"
 
 
+/* Define here special small constants for the reader. These constants are used
+ * by the reader logic. They cannot be used by a program.
+ *
+ * NOTE: We use here the fact that STk_void must be the last small constant
+ * defined in stklos.h
+*/
+
+#define dot_cst         (STk_void+(1<<2))
+#define close_par_cst   (STk_void+(2<<2))
+
 struct read_context {
   SCM cycles;
   SCM inner_refs;
@@ -43,8 +53,6 @@ static SCM read_rec(SCM port, struct read_context *ctx, int inlist);
 static SCM sym_quote, sym_quasiquote, sym_unquote, sym_unquote_splicing, sym_dot;
 static SCM sym_read_brace, sym_read_bracket, read_error;
 
-int STk_read_case_sensitive = DEFAULT_CASE_SENSITIVE;
-
 
 #define PLACEHOLDERP(x)         (CONSP(x) && (BOXED_INFO(x) & CONS_PLACEHOLDER))
 #define PLACEHOLDER_VAL(x)      (CDR(x))
@@ -52,6 +60,14 @@ int STk_read_case_sensitive = DEFAULT_CASE_SENSITIVE;
 #define SYMBOL_VALUE(x,ref)     STk_lookup((x), STk_current_module(), &(ref), FALSE)
 
 #define MAX_HEX_SEQ_LEN 20      /* Normally max value is 10FFFFF => 9 with '\0' */
+#define TOKEN_SIZE      200     /* Initial allocation size when reading a token */
+
+
+typedef struct {                  /* structure used to read tokens */
+  char buffer[TOKEN_SIZE];        /* a statically allocated buffer */
+  char *word;                     /* the word itself (&buffer or allocated) */
+} s_word;
+
 
 /* errors when reading strings */
 #define BAD_HEX_SEQUENCE     1
@@ -65,7 +81,22 @@ int STk_read_case_sensitive = DEFAULT_CASE_SENSITIVE;
 #define COLON_AFTER  0x2
 #define COLON_BOTH   0x3
 
-static char colon_pos;
+static char colon_pos = COLON_BOTH;
+
+
+/* sharp_table is a table for reading #xxx constants (for instance #u8
+ * #!optional)
+ *
+ * sharp_char_table is an alist of character wich can follow a
+ * sharp character and which permit to enter a constant with its own syntax
+ * (for instance #*1101) to read a bitvector constant. The alist is of the form
+ * ( (ch1 . proc1) (ch2 .proc2) ... ) wher ch is a character and where proc is
+ * a procedure which take a parameter which is the port to read (the character
+ * ch being pushed back on this port)
+ */
+static SCM sharp_table, sharp_char_table = STk_nil;
+
+typedef SCM (*sharp_func) (SCM, struct read_context*, const char*);
 
 /*===========================================================================*\
  *
@@ -88,7 +119,7 @@ static void signal_error(SCM port, char *format, SCM param)
 
 static void error_token_too_large(SCM port, char *tok)
 {
-  tok[MAX_TOKEN_SIZE-1] = '\0'; /* truncate the name (should be sufficient !!) */
+  tok[TOKEN_SIZE-1] = '\0'; /* truncate the name (should be sufficient !!) */
   signal_error(port, "token too large ~S", STk_Cstring2string(tok));
 }
 
@@ -144,11 +175,12 @@ static void warning_parenthesis(SCM port)
 
 static void warning_bad_escaped_sequence(SCM port, int c)
 {
-  STk_warning("character %c must not be escaped on line %d of ~S", c, PORT_LINE(port), port);
+  STk_warning("character %c must not be escaped on line %d of ~S", c,
+              PORT_LINE(port), port);
 }
 
 
-static int colon_position_value(char *str)
+static int colon_position_value(const char *str)
 {
   if      (strcmp(str, "none")   == 0) return COLON_NONE;
   else if (strcmp(str, "before") == 0) return COLON_BEFORE;
@@ -178,7 +210,7 @@ static int flush_spaces(SCM port, char *message, SCM file)
 }
 
 
-static int read_hex_sequence(SCM port, char* utf8_seq, int use_utf8) // ⇒ -1 if incorrect
+static int read_hex_sequence(SCM port, char *utf8_seq, int use_utf8) // ⇒ -1 if incorrect
 {
   char buffer[MAX_HEX_SEQ_LEN];
   char *end;   /* normally max value is 10FFFFF */
@@ -236,18 +268,18 @@ static SCM read_list(SCM port, char delim, struct read_context *ctx)
     line = PORT_LINE(port);
     cur  = read_rec(port, ctx, TRUE);
 
-    if (cur == STk_close_par) {
+    if (cur == close_par_cst) {
       c = STk_getc(port);
       if (c != delim) warning_parenthesis(port);
       return start;
     }
 
-    if (cur == STk_dot) {
+    if (cur == dot_cst) {
       if (last == STk_nil)
         error_bad_dotted_list(port, line);  // dot before an element
 
       cur = read_rec(port, ctx, TRUE);
-      if (cur == STk_close_par)
+      if (cur == close_par_cst)
         error_bad_dotted_list(port, line); // dot not followed by an element
 
       c = flush_spaces(port, eof_seen, port);
@@ -257,11 +289,11 @@ static SCM read_list(SCM port, char delim, struct read_context *ctx)
         STk_ungetc(c, port);
         tmp = read_rec(port, ctx, TRUE);
 
-        if (tmp != STk_close_par) STk_error("closing parenthesis expected");
+        if (tmp != close_par_cst) STk_error("closing parenthesis expected");
         c = flush_spaces(port, eof_seen, port);
       }
       if (c != delim)
-        error_bad_dotted_list(port, line); // dot not befor last element
+        error_bad_dotted_list(port, line); // dot not before last element
       CDR(last) = cur;
       return start;
     }
@@ -276,8 +308,7 @@ static SCM read_list(SCM port, char delim, struct read_context *ctx)
          /* Constant read uses extended cons instead of cons */
          tmp = STk_econs(cur, STk_nil, PORT_FNAME(port), line, PORT_POS(port));
          BOXED_INFO(tmp) |= CONS_CONST;
-       }
-       else {
+       } else {
          tmp = STk_cons(cur, STk_nil);
        }
 
@@ -291,14 +322,36 @@ static SCM read_list(SCM port, char delim, struct read_context *ctx)
   }
 }
 
-static int read_word(SCM port, int c, char *tok, int case_significant, int *last,
+
+static char *enlarge_word(s_word *pword, int *sz)
+/* Enlarge word buffer which contains n characters in a buffer of size "sz" */
+{
+  int new_size = *sz * 3 / 2;
+
+  if  (pword->word == pword->buffer) {
+    /* never dynamically allocated */
+    pword->word = STk_must_malloc_atomic(new_size);
+    memcpy(pword->word, pword->buffer, *sz);
+  } else {
+    /* word is already dynamically allocated, realloc it */
+    pword->word =STk_must_realloc(pword->word, new_size);
+  }
+  *sz =new_size;
+  return pword->word;
+}
+
+static int read_word(SCM port, int c, s_word *pword, int case_significant, int *last,
                      int *seen_pipe)
 // read an item whose 1st char is in c. Return its length
 // At exit "last", if not NULL,  contains the last character read for this item
 {
   register int j = 0;
   int allchars   = 0;
-  int next;
+  int next, sz;
+  char *tok;
+
+  tok = pword->word= pword->buffer;
+  sz = TOKEN_SIZE;             /* size statically allocated for reading a token */
 
   for( ; ; ) {
     allchars  ^= (c == '|');
@@ -331,13 +384,9 @@ static int read_word(SCM port, int c, char *tok, int case_significant, int *last
           if (len < 0)
             error_bad_inline_hexa_sequence(port, buffer, 1); /* 1 = symbol */
           else {
-            if (j + len >= MAX_TOKEN_SIZE-1) {
-              tok[j] = '\0';
-              error_token_too_large(port, tok);
-            } else {
-              memcpy(tok + j-1, buffer, len);
-              j += len-1;
-            }
+            if (j + len >= sz-1) tok = enlarge_word(pword, &sz);
+            memcpy(tok + j-1, buffer, len);
+            j += len-1;
           }
           break;
         }
@@ -356,7 +405,7 @@ static int read_word(SCM port, int c, char *tok, int case_significant, int *last
       }
     }
     c = next;
-    if (j >= MAX_TOKEN_SIZE-1) error_token_too_large(port, tok);
+    if (j >= sz-1) tok = enlarge_word(pword, &sz);
   }
 
   if (last) *last = c;
@@ -365,15 +414,19 @@ static int read_word(SCM port, int c, char *tok, int case_significant, int *last
   return j;
 }
 
-static SCM read_token(SCM port, int c, int case_significant)
+static SCM read_token(SCM port, int c, struct read_context *ctx)
 {
-  char tok[MAX_TOKEN_SIZE];
+  s_word w;
   int len, last, seen_pipe=0;
+  char *tok;
+  int ci= ctx->case_significant && (c != '#'); // #xxx constants are case insensitive
+
+  len = read_word(port, c, &w, ci, &last, &seen_pipe);
+  tok = w.word;
+
 
-  len = read_word(port, c, tok, case_significant, &last, &seen_pipe);
   if (!seen_pipe) {
     SCM z = STk_Cstr2number(tok, 10L);
-
     if (z != STk_false)
       return z;
   }
@@ -381,21 +434,25 @@ static SCM read_token(SCM port, int c, int case_significant)
   /* It is not a number */
   if (*tok == '#') {
     if (len > 1) {
-      if (len == 2) {
-        if (tok[1] == 't' || tok[1] == 'T') return STk_true;
-        if (tok[1] == 'f' || tok[1] == 'F') return STk_false;
-      }
       if (tok[1] == ':')
         return STk_makekey(tok+2);
-      else if (strcasecmp(tok+1, "true") == 0)
-        return STk_true;
-      else if (strcasecmp(tok+1, "false") == 0)
-        return STk_false;
-      else if (strcasecmp(tok+1, "eof") == 0)
-        return STk_eof;
-      else if (strcasecmp(tok+1, "void") == 0)
-        return STk_void;
+      else {
+        sharp_func fct = STk_C_hash_get(sharp_table, tok+1);
+
+        if (fct) // tok is a known keyword
+          return fct(port, ctx, tok+1);
+
+        if (tok[1] == '!') {
+          // We had #!... where ... is not recognized => comment
+          do {
+            if (c == EOF) return STk_eof;
+          }
+          while ((c=STk_getc(port)) != '\n');
+          STk_ungetc(c, port);
+          return NULL;
+        }
       }
+    }
     error_bad_sharp_syntax(port, tok);
   } else {
     /* We have a symbol or a keyword */
@@ -408,6 +465,8 @@ static SCM read_token(SCM port, int c, int case_significant)
       return STk_makekey(tok);
     }
     else {
+      // FIXME: we could have a `STk_intern_no_dup` to avoid the duplication of tok
+      // when tok != w.buffer. Is it really worthwhile?
       return STk_intern(tok);
     }
   }
@@ -417,7 +476,7 @@ static SCM read_token(SCM port, int c, int case_significant)
 static SCM read_char(SCM port, int c)
 /* read a char (or a char name) item whose 1st char is in c */
 {
-  char tok[MAX_TOKEN_SIZE];
+  char tok[TOKEN_SIZE];
   register int j = 0;
 
   for( ; ; ) {
@@ -430,7 +489,7 @@ static SCM read_char(SCM port, int c)
       STk_ungetc(c, port);
       break;
     }
-    if (j >= MAX_TOKEN_SIZE-1) error_token_too_large(port, tok);
+    if (j >= TOKEN_SIZE-1) error_token_too_large(port, tok);
   }
   tok[j] = '\0';
 
@@ -441,21 +500,39 @@ static SCM read_char(SCM port, int c)
 
 static SCM read_address(SCM port)
 {
-  char *end, tok[MAX_TOKEN_SIZE] = "0";
+  char *end, tok[TOKEN_SIZE] = "0x";
   unsigned long address;
+  register int j;
 
-  read_word(port, 'x', tok+1, FALSE, NULL, NULL);
+  for(j = 2; ; j++) {
+    int c;
+
+    tok[j] = c = STk_getc(port);
+    if (c == EOF || ((c <=0x80) && isspace((unsigned char)c)))
+      /* (c < 0x80) is for MacOs */
+      break;
+    if (j >= TOKEN_SIZE-1) error_token_too_large(port, tok);
+  }
+  tok[j] = '\0';
+
+  /* convert the hexa number contained in token to an address */
+  errno = 0;                 // Weird: strtoul doesn't set it anymore
   address = strtoul(tok, &end, 16);
-  if (*end)
+  if (*end || errno)
     signal_error(port, "bad address specifier #p~a", STk_Cstring2string(tok+2));
 
+  /* Verify that the address isvalid */
+  STk_verify_address(address, STk_Cstring2string(tok+2));
+
   return (SCM) address;
 }
 
+
 static SCM read_here_string(SCM port)
 {
-  SCM eof_token = read_token(port, STk_getc(port), TRUE);
   SCM res, line;
+  struct read_context ctx = {.case_significant =TRUE};
+  SCM eof_token           = read_token(port, STk_getc(port), &ctx);
   int first_line = TRUE;
 
   if (!SYMBOLP(eof_token)) STk_error("bad symbol for here string ~S", eof_token);
@@ -527,7 +604,7 @@ static void patch_references(SCM port, SCM l, SCM cycles)
 static SCM read_cycle(SCM port, int c, struct read_context *ctx)
 /* read a #xx# or #xx= cycle item whose 1st char is in c. */
 {
-  char buffer[MAX_TOKEN_SIZE];
+  char buffer[TOKEN_SIZE];
   int  j = 0;
   SCM k, tmp, val;
 
@@ -535,7 +612,7 @@ static SCM read_cycle(SCM port, int c, struct read_context *ctx)
     buffer[j++] = c;
     c = STk_getc(port);
     if (c == EOF || !isdigit(c)) break;
-    if (j >= MAX_TOKEN_SIZE-1) error_token_too_large(port, buffer);
+    if (j >= TOKEN_SIZE-1) error_token_too_large(port, buffer);
   }
   buffer[j] = '\0';
   k = MAKE_INT(atoi(buffer));
@@ -681,7 +758,7 @@ static SCM read_string(SCM port, int constant)
   }
   *p = '\0';
 
-  switch(error) {    /* No BAD_ASCII_CHAR here: '\!' is equivalent to '!' */
+  switch(error) {    /* No BAD_ASCII_CHAR here: '\Z' is equivalent to 'Z' */
     case BAD_HEX_SEQUENCE:
       error_bad_inline_hexa_sequence(port, hex_buffer, 0); break;
     case BAD_ESCAPED_SPACE:
@@ -790,43 +867,154 @@ static SCM read_vector(SCM port, struct read_context *ctx)
   return v;
 }
 
-static SCM maybe_read_uniform_vector(SCM port, int c, struct read_context *ctx)
+static SCM read_uniform_vector(SCM port, struct read_context *ctx, const char *kind)
 {
-  char tok[MAX_TOKEN_SIZE];
-  int tag, len;
-  SCM v;
+  int tag = STk_uniform_vector_tag(kind);
+  int c = STk_getc(port);
 
-  len = read_word(port, c, tok, ctx->case_significant, NULL, NULL);
+  if (c == '"' && strcmp(kind, "u8") == 0)
+    return read_srfi207_bytevector(port, ctx->constant);
 
-  if (strcasecmp(tok, "f") ==0 || strcasecmp(tok, "false") ==0) {
-    /* This is the #f constant */
-    return STk_false;
-  } else {
-    if ((!STk_uvectors_allowed &&  (strcmp(tok, "u8") == 0)) ||
-        (STk_uvectors_allowed && (len == 2 || len == 3))) {
-      c = STk_getc(port);
-      if (c == '"')
-        return read_srfi207_bytevector(port, ctx->constant);
-      if (c != '(') goto bad_spec;
-        tag = STk_uniform_vector_tag(tok);
-        if (tag >= 0) {
-          int konst = ctx->constant;
+  if (c == '(' && tag >= 0) {
+    SCM v;
+    int konst = ctx->constant;
 
-          /* Ok that's seems correct read the list of values (this IS a constant) */
-          ctx->constant = TRUE;
-          v =  STk_list2uvector(tag, read_list(port, ')', ctx));
-          ctx->constant = konst;
-          BOXED_INFO(v) |= VECTOR_CONST;
-          return v;
-        }
-    }
+    /* Read the list of values (this IS a constant) */
+    ctx->constant = TRUE;
+    v =  STk_list2uvector(tag, read_list(port, ')', ctx));
+    ctx->constant = konst;
+    BOXED_INFO(v) |= VECTOR_CONST;
+    return v;
   }
- bad_spec:
-  signal_error(port, "bad uniform vector specification ~A",STk_Cstring2string(tok));
+  signal_error(port, "bad uniform vector specification ~A",STk_Cstring2string(kind));
   return STk_void;
 }
 
 
+/* read #.... syntax. If we read an object (a keyword such as #!rest or a
+ * constant such as #true), it is returned. Otherwise this is a comment and we
+ * return NULL.  */
+static SCM read_sharp(SCM port, struct read_context *ctx, int inlist)
+{
+  int c = STk_getc(port);
+
+  switch(c) {
+    case '\\': return read_char(port, STk_getc(port));
+
+    case '(' : return read_vector(port, ctx);
+
+   case '!' : {
+      SCM word;
+
+      // Force case insensitive for reading #!xxx
+      ctx->case_significant = FALSE;
+
+      STk_ungetc(c, port);
+      word = read_token(port, '#', ctx);
+
+      // set back value from the port (in case of #!{no,}fold-case)
+      ctx->case_significant = (PORT_FLAGS(port)&PORT_CASE_SENSITIVE) != 0;
+
+      if (word)
+        /* DSSSL keyword , #!fold-case or #!keyword-colon-position-... */
+        return word;
+      /* Comment*/
+      return NULL;
+    }
+
+    case '|':  {
+      char prev = ' ';
+
+      ctx->comment_level += 1;
+      for ( ; ; ) {
+        switch (c = STk_getc(port)) {
+        case EOF:
+          goto end_comment;
+        case '\n':
+          break;
+        case '#':
+          if (prev == '|') {
+            ctx->comment_level -= 1;
+            if (!ctx->comment_level) goto end_comment;
+          }
+          break;
+        case '|':
+          if (prev == '#')
+            ctx->comment_level += 1;
+          break;
+        default: ;
+        }
+        prev = c;
+      }
+      end_comment:
+      c = flush_spaces(port, (char *) NULL, (SCM) NULL);
+      if (c == EOF) {
+        if (ctx->comment_level)
+          signal_error(port, "eof encountered when reading a comment", STk_nil);
+        else
+          return STk_eof;
+      } else {
+        STk_ungetc(c,port);
+        return NULL;
+      }
+      break;   /* for the compiler */
+    }
+
+    case '<': {
+      char c2 = STk_getc(port);
+      if (c2 == '<' )
+        return read_here_string(port);
+      else  {
+        STk_ungetc(c2, port);
+        goto default_sharp;
+      }
+    }
+
+    case '&': return STk_make_box(read_rec(port, ctx, inlist));
+
+    case 'p':
+    case 'P': return read_address(port);
+
+    case ';': /* R6RS comments */
+      read_rec(port, ctx, FALSE);
+      c = flush_spaces(port, NULL, NULL);
+      STk_ungetc(c, port);
+      if (inlist && (c == ')' || c == ']' || c == '}'))
+        return close_par_cst;
+      return NULL;
+
+    case ',': /* SRFI-10 */
+      return read_srfi10(port, read_rec(port, ctx, inlist));
+
+    case '0':
+    case '1':
+    case '2':
+    case '3':
+    case '4':
+    case '5':
+    case '6':
+    case '7':
+    case '8':
+    case '9': return read_cycle(port, c, ctx);
+
+    default:
+    default_sharp:
+      {
+        SCM reader = STk_int_assq(MAKE_CHARACTER(c), sharp_char_table);
+
+        STk_ungetc(c, port);
+        if (reader != STk_false) {
+          return STk_C_apply(CDR(reader), 1, port);
+        } else {
+          return read_token(port, '#', ctx);
+        }
+      }
+  }
+  return NULL; // for the compiler
+}
+
+
+
 static SCM read_rec(SCM port, struct read_context *ctx, int inlist)
 {
   int c;
@@ -851,15 +1039,15 @@ static SCM read_rec(SCM port, struct read_context *ctx, int inlist)
         return(read_list(port, ']', ctx));
       }
 
-    case '{': {
-      SCM ref, read_brace_func = SYMBOL_VALUE(sym_read_brace, ref);
+      case '{': {
+        SCM ref, read_brace_func = SYMBOL_VALUE(sym_read_brace, ref);
 
         if (read_brace_func != STk_void) {
           STk_ungetc(c, port);
           return STk_C_apply(read_brace_func, 1, port);
         }
-        goto default_case;
-      return read_list(port, '}', ctx);
+        goto default_case;                   //FIXME
+        //return read_list(port, '}', ctx);
       }
 
       case ')':
@@ -867,161 +1055,31 @@ static SCM read_rec(SCM port, struct read_context *ctx, int inlist)
       case '}':
         if (inlist) {
           STk_ungetc(c, port);
-          return STk_close_par;
+          return close_par_cst;
         }
         warning_parenthesis(port);
         break;
       case '\'':
         quote_type = sym_quote;
         goto read_quoted;
+
       case '`':
         quote_type = sym_quasiquote;
-    read_quoted:
+      read_quoted:
         {
           SCM tmp = read_rec(port, ctx, inlist);
-          if (tmp == STk_dot || tmp == STk_close_par)
+          if (tmp == dot_cst || tmp == close_par_cst)
             signal_error(port, "bad quote/quasiquote syntax", STk_nil);
           return LIST2(quote_type, tmp);
         }
+
       case '#':
-        switch(c=STk_getc(port)) {
-          // FIXME:
-          case 'F' :
-          case 'f' : if (STk_uvectors_allowed)
-                       return maybe_read_uniform_vector(port, c, ctx);
-                     goto default_sharp;
-          case '\\': return read_char(port, STk_getc(port));
-          case '(' : return read_vector(port, ctx);
-          case '!' : { /* This can be a comment, a DSSSL keyword, or fold-case */
-                       c = STk_getc(port);
-                       if (c == 'o' || c == 'k' || c == 'r' || c == 'n' || c == 'f') {
-                         SCM word = read_token(port, c, FALSE);
-
-                         if (SYMBOLP(word)) {
-                           char *s = SYMBOL_PNAME(word);
-
-                           /* Try to see if it is a DSSL keyword */
-                           if ((strcmp(s, "optional") == 0) ||
-                               (strcmp(s, "key")      == 0) ||
-                               (strcmp(s, "rest")     == 0))
-                             return STk_makekey(s);
-
-                           /* Treat fold-case and no-fold-case */
-                           if ((strcmp(s, "fold-case") == 0) ||
-                               (strcmp(s, "no-fold-case") == 0)) {
-                             if (c == 'n') {
-                               PORT_FLAGS(port) |= PORT_CASE_SENSITIVE;
-                               ctx->case_significant = TRUE;
-                             }
-                             else {
-                               PORT_FLAGS(port) &= ~PORT_CASE_SENSITIVE;
-                               ctx->case_significant = FALSE;
-                             }
-                             continue;
-                           }
-                           /* Treat keyword-colon-{none,before,after,both} */
-                           if ((strcmp(s, "keyword-colon-position-none")   == 0) ||
-                               (strcmp(s, "keyword-colon-position-before") == 0) ||
-                               (strcmp(s, "keyword-colon-position-after")  == 0) ||
-                               (strcmp(s, "keyword-colon-position-both")   == 0)) {
-                             PORT_KW_COL_POS(port) = colon_position_value(s+23); // none, before, ...
-                             continue;
-                           }
-                         }
-                       }
-                       /* if we are here, consider the rest of the line
-                        * as a comment*/
-                       do {
-                         if (c == EOF) return STk_eof;
-                       } while ((c=STk_getc(port)) != '\n');
-                       STk_ungetc(c, port);
-                       continue;
-                     }
-          case '|':  {
-                       char prev = ' ';
-
-                       ctx->comment_level += 1;
-                       for ( ; ; ) {
-                         switch (c = STk_getc(port)) {
-                           case EOF:
-                             goto end_comment;
-                           case '\n':
-                             break;
-                           case '#':
-                             if (prev == '|') {
-                               ctx->comment_level -= 1;
-                               if (!ctx->comment_level) goto end_comment;
-                             }
-                             break;
-                           case '|':
-                             if (prev == '#')
-                               ctx->comment_level += 1;
-                             break;
-                           default: ;
-                         }
-                         prev = c;
-                       }
-                       end_comment:
-                       c = flush_spaces(port, (char *) NULL, (SCM) NULL);
-                       if (c == EOF) {
-                         if (ctx->comment_level)
-                           signal_error(port,
-                                        "eof encountered when reading a comment",
-                                        STk_nil);
-                         else
-                           return STk_eof;
-                       } else {
-                         STk_ungetc(c,port);
-                         continue;
-                       }
-                       break;   /* for the compiler */
-                    }
-          case '<': {
-                       char c2 = STk_getc(port);
-                       if (c2 == '<' )
-                         return read_here_string(port);
-                       else  {
-                         STk_ungetc(c2, port);
-                         goto default_sharp;
-                       }
-                    }
-          case '&': return STk_make_box(read_rec(port, ctx, inlist));
-          case 'p':
-          case 'P': return read_address(port);
-          case 'S':
-          case 's':
-          case 'U':
-          case 'u': if (STk_uvectors_allowed || c == 'u')
-                      /* For R7RS #u8 is always valid (bytevectors) */
-                      return maybe_read_uniform_vector(port, c, ctx);
-                    else
-                      goto default_sharp;
-         case ';': /* R6RS comments */
-                   read_rec(port, ctx, FALSE);
-                   c = flush_spaces(port, NULL, NULL);
-                   STk_ungetc(c, port);
-                   if (inlist && (c == ')' || c == ']' || c == '}'))
-                     return STk_close_par;
-                   continue;
-          case ',': /* SRFI-10 */
-                    return read_srfi10(port,
-                                       read_rec(port, ctx, inlist));
-          case '0':
-          case '1':
-          case '2':
-          case '3':
-          case '4':
-          case '5':
-          case '6':
-          case '7':
-          case '8':
-          case '9': return read_cycle(port, c, ctx);
-          default:
-            default_sharp:
-                    STk_ungetc(c, port);
-                    return read_token(port, '#', ctx->case_significant);
+        {
+          SCM tmp = read_sharp(port, ctx, inlist);;
+          if (tmp) return tmp;
+          continue;
         }
-        break; /* for the compiler */
+
       case ',': {
         SCM symb, tmp;
 
@@ -1033,15 +1091,18 @@ static SCM read_rec(SCM port, struct read_context *ctx, int inlist)
           STk_ungetc(c, port);
         }
         tmp = read_rec(port, ctx, inlist);
-        if (tmp == STk_dot || tmp == STk_close_par)
+        if (tmp == dot_cst || tmp == close_par_cst)
           signal_error(port, "bad unquote/unquote-splice syntax", STk_nil);
         return LIST2(symb, tmp);
       }
+
       case '"':
         return read_string(port, ctx->constant);
+
       default:
-    default_case: {
-          SCM tmp = read_token(port, c, ctx->case_significant);
+        default_case:
+        {
+          SCM tmp = read_token(port, c, ctx);
           if (tmp != sym_dot)
             return tmp;
 
@@ -1049,11 +1110,12 @@ static SCM read_rec(SCM port, struct read_context *ctx, int inlist)
             return STk_intern(".");
 
           if (inlist)
-            return STk_dot;
+            return dot_cst;
           signal_error(port, "dot outside of list", STk_nil);
         }
     }
   }
+  return NULL; // for the compiler
 }
 
 /*===========================================================================*\
@@ -1164,51 +1226,19 @@ static SCM read_srfi10(SCM port, SCM l)
 }
 
 
-/*
-<doc EXT read-case-sensitive
- * (read-case-sensitive)
- * (read-case-sensitive value)
- *
- * This parameter object permits to change the default behaviour of
- * the |read| primitive when reading a symbol. If this parameter has a
- * a true value a symbol is not converted to a default case when interned.
- * Since ,(rseven) requires that symbol are case insignificant, the default
- * value  of this parameter is |#t|.
- * @lisp
- * (read-case-sensitive)        => |#t|
- * (read-from-string "ABC")     => ABC
- * (read-case-sensitive #f)
- * (read-from-string "ABC")     => abc
- * @end lisp
- * ,(bold "Note:")  Default behaviour can be changed for a whole execution
- * with the |--case-sensitive| or |case-insensitive| options.
- * @l
- * ,(bold "Note:") See also syntax for ,(ref :mark "bar-in-symbol" :text
- * [special characters]) in symbols.
- *
-doc>
-*/
-static SCM read_case_sensitive_conv(SCM value)
-{
-  STk_read_case_sensitive = (value != STk_false);
-  return MAKE_BOOLEAN(STk_read_case_sensitive);
-}
-
-
 /*
 <doc EXT keyword-colon-position
  * (keyword-colon-position)
  * (keyword-colon-position value)
  *
  * This parameter object indicates the convention used by the reader to
- * denote keywords. The allowed values are
- * ,(itemize
- *     (item [none, to forbid a symbol with colon to be interpreted as a
- *           keyword])
- *     (item [before, to read symbols starting with a colon as keywords])
- *     (item [after, to read symbols ending with a colon as keywords])
- *     (item [both, to read symbols starting or ending with a colon as
- *            keywords]))
+ * denote keywords. The allowed values are:
+ *
+ * - *none*, to forbid a symbol with colon to be interpreted as a keyword,
+ * - *before*, to read symbols starting with a colon as keywords,
+ * - *after*, to read symbols ending with a colon as keywords,
+ * - *both*,  to read symbols starting or ending with a colon as keywords.
+ *
  * Note that the notation |#:key| is always read as a keyword independently
  * of the value of |keyword-colon-position|. Hence, we have
  * @lisp
@@ -1251,7 +1281,90 @@ int STk_keyword_colon_convention(void)
   return colon_pos;
 }
 
-/*===========================================================================*\
+
+DEFINE_PRIMITIVE("%read-list", user_read_list, subr2, (SCM port, SCM end_delim))
+{
+  struct read_context ctx;
+
+  if (!PORTP(port)) STk_error_bad_port(port);
+  if (!CHARACTERP(end_delim)) STk_error("bad character ~s", end_delim);
+
+  ctx.cycles           = STk_nil;
+  ctx.inner_refs       = STk_nil;
+  ctx.comment_level    = 0;
+  ctx.case_significant = PORT_CASE_SENSITIVEP(port);
+  ctx.constant         = TRUE;
+
+  return read_list(port, CHARACTER_VAL(end_delim), &ctx);
+}
+
+/*=======================================================================*\
+ *
+ * Sharp reader helper functions
+ *
+\* ======================================================================*/
+
+static SCM sharp_simple_keyword(SCM _UNUSED(port), struct read_context _UNUSED(*ctx),
+                                const char *word)
+{
+  switch (*word) {
+    case 't': return STk_true;    // #t or #true
+    case 'f': return STk_false;   // #f or #false
+    case 'e': return STk_eof;     // #eof
+    case 'v': return STk_void;    // #void
+    default: // assert: word[0] == '!'
+      switch (word[1]) {
+        case 'o': return STk_makekey("optional"); // #!optional (DSSSL)
+        case 'k': return STk_makekey("key");      // #!key (DSSSL)
+        case 'r': return STk_makekey("rest");     // #!rest (DSSSL)
+      }
+  }
+  return NULL; // for the compiler
+}
+
+static SCM sharp_fold_keyword(SCM port, struct read_context _UNUSED(*ctx),
+                              const char *word)
+{
+  STk_port_cs_set(port, MAKE_BOOLEAN((word[1] == 'n'))); // word = "!no-fold-case"
+  return NULL;  // NULL since the keyword is not returned
+}
+
+static SCM sharp_keypos(SCM _UNUSED(port), struct read_context _UNUSED(*ctx),
+                        const char *word)
+{
+  const char *val=sizeof("keyword-colon-position-") + word; // none, before, ...
+  PORT_KW_COL_POS(port) = colon_position_value(val);
+  return NULL;  // NULL since the keword is not returned
+}
+
+static SCM sharp_uvector(SCM port, struct read_context *ctx, const char *word)
+{
+  return read_uniform_vector(port, ctx, word);
+}
+
+void STk_add_uvector_reader_tag(const char *tag)
+{
+  STk_C_hash_set(sharp_table, tag, sharp_uvector);
+}
+
+
+DEFINE_PRIMITIVE("%add-sharp-reader", add_sharp_reader, subr2, (SCM ch, SCM proc))
+{
+  SCM old;
+  if (!CHARACTERP(ch))                   STk_error("bad character ~S", ch);
+  if (STk_procedurep(proc) == STk_false) STk_error("bad procedure ~S", proc);
+
+  old = STk_int_assq(ch, sharp_char_table);
+  if (old != STk_false)
+    CDR(old) = proc;
+  else
+    sharp_char_table = STk_cons(STk_cons(ch, proc), sharp_char_table);
+
+  return STk_void;
+}
+
+
+/*===========================================================================* \
  *
  *                      I n i t i a l i z a t i o n
  *
@@ -1278,17 +1391,43 @@ int STk_init_reader(void)
   /* Declare SRFI-10 support function */
   ADD_PRIMITIVE(reader_ctor);
 
-  /* Declare parameter read-case-sensitve */
-  STk_make_C_parameter("read-case-sensitive",
-                       MAKE_BOOLEAN(STk_read_case_sensitive),
-                       read_case_sensitive_conv,
-                       STk_STklos_module);
-
   /* Declare parameter keyword-colon-position */
   colon_pos = COLON_BOTH;
   STk_make_C_parameter2("keyword-colon-position",
                         keyword_colon_position_get,
                         keyword_colon_position_set,
                         STk_STklos_module);
+
+  /* Initialize the table for objects wich start with a '#' */
+  sharp_table = STk_make_C_hash_table();
+
+  STk_C_hash_set(sharp_table, "t",             sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "true",          sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "false",         sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "f",             sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "eof",           sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "void",          sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "!optional",     sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "!key",          sharp_simple_keyword);
+  STk_C_hash_set(sharp_table, "!rest",         sharp_simple_keyword);
+
+  STk_C_hash_set(sharp_table, "!fold-case",    sharp_fold_keyword);
+  STk_C_hash_set(sharp_table, "!no-fold-case", sharp_fold_keyword);
+
+  STk_C_hash_set(sharp_table, "!keyword-colon-position-none",   sharp_keypos);
+  STk_C_hash_set(sharp_table, "!keyword-colon-position-before", sharp_keypos);
+  STk_C_hash_set(sharp_table, "!keyword-colon-position-after",  sharp_keypos);
+  STk_C_hash_set(sharp_table, "!keyword-colon-position-both",   sharp_keypos);
+
+  /* Add reader for #u8 constants */
+  STk_add_uvector_reader_tag("u8");
+
+  /* Add a primitive to permit the definition new forms of sharp constants */
+  ADD_PRIMITIVE(add_sharp_reader);
+
+  /* Add primitive for reading a list whose first character is already read */
+  /* This is useful to add specialized reader on [] and {} */
+  ADD_PRIMITIVE(user_read_list);
+
   return TRUE;
 }
