@@ -2,7 +2,7 @@
  *
  * s t r . c                            -- Strings management
  *
- * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,16 +22,12 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: ??????
- * Last file update:  6-Sep-2021 18:44 (eg)
  */
 
 #include <ctype.h>
 #include "stklos.h"
 
 
-extern SCM STk_make_bytevector_from_string(char *str, long len);
-
-
 /* min size added to a string when reallocated in a string-set! */
 #define UTF8_STRING_INCR        8
 
@@ -86,7 +82,7 @@ static int stringcomp(SCM s1, SCM s2)
 
   if (STk_use_utf8 && (!STRING_MONOBYTE(s1) || !STRING_MONOBYTE(s2))) {
     /* At least one string is multi-bytes */
-    uint32_t ch1, ch2;
+    utf8_char ch1, ch2;
     char *end1, *end2;
 
     str1 = STRING_CHARS(s1); end1 = str1 + STRING_SIZE(s1);
@@ -128,7 +124,7 @@ static int stringcompi(SCM s1, SCM s2)
 
   if (STk_use_utf8 && (!STRING_MONOBYTE(s1) || !STRING_MONOBYTE(s2))) {
     /* At least one string is multi-bytes */
-    uint32_t ch1, ch2;
+    utf8_char ch1, ch2;
     char *end1, *end2;
 
     str1 = STRING_CHARS(s1); end1 = str1 + STRING_SIZE(s1);
@@ -206,9 +202,9 @@ static SCM control_index(int argc, SCM *argv, long *pstart, long *pend)
   return s;
 }
 
-static uint32_t *string2int(char *s, int len, int *utf8_len, uint32_t(*func)(uint32_t))
+static utf8_char *string2int(char *s, int len, int *utf8_len, utf8_char(*func)(utf8_char))
 {
-  uint32_t ch, *tmp, *buff = STk_must_malloc_atomic(len * sizeof(uint32_t));
+  utf8_char ch, *tmp, *buff = STk_must_malloc_atomic(len * sizeof(utf8_char));
   int space = 0;
 
   for (tmp = buff; len--; tmp++) {
@@ -222,7 +218,7 @@ static uint32_t *string2int(char *s, int len, int *utf8_len, uint32_t(*func)(uin
   return buff;
 }
 
-static SCM make_string_from_int_array(uint32_t *buff, int len, int utf8_len)
+static SCM make_string_from_int_array(utf8_char *buff, int len, int utf8_len)
 {
   SCM z;
   char *s, *end;
@@ -242,7 +238,7 @@ static SCM make_string_from_int_array(uint32_t *buff, int len, int utf8_len)
 }
 
 
-static void copy_array(uint32_t *buff, int len, char* from)
+static void copy_array(utf8_char *buff, int len, char *from)
 {
   while (len--)
     from += STk_char2utf8(*buff++, from);
@@ -255,7 +251,7 @@ static SCM make_substring(SCM string, long from, long to)
     return STk_makestring(to - from, STRING_CHARS(string)+from);
   else {
     /* multi-bytes string */
-    uint32_t c;
+    utf8_char c;
     char *pfrom, *pto;
     SCM z;
 
@@ -271,7 +267,7 @@ static SCM make_substring(SCM string, long from, long to)
 }
 
 
-SCM STk_makestring(int len, char *init)
+SCM STk_makestring(int len, const char *init)
 {
   register SCM z;
 
@@ -297,7 +293,7 @@ SCM STk_makestring(int len, char *init)
 }
 
 
-SCM STk_Cstring2string(char *str) /* Embed a C string in Scheme world  */
+SCM STk_Cstring2string(const char *str) /* Embed a C string in Scheme world  */
 {
   SCM  z;
   size_t len = strlen(str);
@@ -459,7 +455,7 @@ DEFINE_PRIMITIVE("string-ref", string_ref, subr2, (SCM str, SCM index))
     return MAKE_CHARACTER((unsigned char)STRING_CHARS(str)[k]);
   else {
     /* We have multibytes chars */
-    uint32_t c;
+    utf8_char c;
     char *s = STRING_CHARS(str);
 
     do
@@ -475,7 +471,7 @@ DEFINE_PRIMITIVE("string-ref", string_ref, subr2, (SCM str, SCM index))
  * (string-set! string k char)
  *
  * |String-set!| stores |char| in element |k| of |string| and returns
- * ,(emph "void") (|k| must be a valid index of |string|).
+ * *_void_* (|k| must be a valid index of |string|).
  *
  * @lisp
  * (define (f) (make-string 3 #\*))
@@ -559,8 +555,8 @@ DEFINE_PRIMITIVE("string-set!", string_set, subr3, (SCM str, SCM index, SCM valu
  * characters in the same positions, otherwise returns |#f|. |String-ci=?|
  * treats upper and lower case letters as though they were the same character,
  * but |string=?| treats upper and lower case as distinct characters.
- * @l
- * ,(bold "Note"): R5RS version of these functions accept only two arguments.
+ *
+ * NOTE: R5RS version of these functions accept only two arguments.
 doc>
  */
 
@@ -581,8 +577,8 @@ doc>
  * characters. If two strings differ in length but are the same up to the
  * length of the shorter string, the shorter string is considered to be
  * lexicographically less than the longer string.
- * @l
- * ,(bold "Note"): R5RS version of these functions accept only two arguments.
+ *
+ * NOTE: R5RS version of these functions accept only two arguments.
 doc>
  */
 
@@ -700,14 +696,14 @@ DEFINE_PRIMITIVE("string-append", string_append, vsubr, (int argc, SCM* argv))
  * It is guaranteed that string-append! will return the same object that
  * was passed to it as first argument, whose size may be larger.
  *
- * ,(linebreak)
- * ,(bold "Note:") This function is defined in SRFI-118.
+ *
+ * NOTE: This function is defined in SRFI-118.
 doc>
  */
 DEFINE_PRIMITIVE("string-append!", string_dappend, vsubr, (int argc, SCM* argv))
 {
   int i;
-  unsigned total_size, total_length;
+  unsigned total_size=0, total_length; // initialize for compiler
   char *q; /* first string */
   char *p; /* others (chars or strings) */
 
@@ -768,7 +764,7 @@ int get_substring_size(SCM string, long from, long to) {
     return (to - from);
   else {
     /* multi-bytes string */
-    uint32_t c;
+    utf8_char c;
     char *pfrom, *pto;
 
     pto = pfrom = STk_utf8_index(STRING_CHARS(string), from, STRING_SIZE(string));
@@ -782,12 +778,14 @@ int get_substring_size(SCM string, long from, long to) {
 
 /*
 <doc EXT string-replace!
- * (string-replace! dst dst-start dst-end src [src-start [src-end]])
+ * (string-replace! dst dst-start dst-end src)
+ * (string-replace! dst dst-start dst-end src src-start)
+ * (string-replace! dst dst-start dst-end src src-start src-end)
  *
  * Replaces the characters of the variable-size string dst (between
  * dst-start and dst-end) with the characters of the string src
  * (between src-start and src-end). The number of characters from src
- * may be different than the number replaced in dst, so the string may
+ * may be different from the number replaced in dst, so the string may
  * grow or contract. The special case where dst-start is equal to
  * dst-end corresponds to insertion; the case where src-start is equal
  * to src-end corresponds to deletion. The order in which characters
@@ -795,16 +793,22 @@ int get_substring_size(SCM string, long from, long to) {
  * destination overlap, copying takes place as if the source is first
  * copied into a temporary string and then into the destination.
  * Returns string, appended with the characters form the concatenation
- * of the given arguments, which can be wither strings or characters.
+ * of the given arguments, which can be either strings or characters.
  *
  * It is guaranteed that string-replace! will return the same object that
  * was passed to it as first argument, whose size may be larger.
- * ,(linebreak)
- * ,(bold "Note:") This function is defined in SRFI-118.
+ *
+ * NOTE: This function is defined in SRFI-118.
 doc>
  */
 DEFINE_PRIMITIVE("string-replace!", string_dreplace, vsubr, (int argc, SCM* argv))
 {
+  if (argc<4 || argc >6) {
+    STk_error("incorrect number of arguments (%d)", argc);
+    return STk_void;
+  }
+
+  /* Number of parameters is OK */
   SCM dst = argv[0];
   SCM src = argv[-3];
   long dst_start, dst_end, src_start, src_end;
@@ -831,8 +835,9 @@ DEFINE_PRIMITIVE("string-replace!", string_dreplace, vsubr, (int argc, SCM* argv
     if (  src_end < 0 ||   src_end > STRING_LENGTH(src))
       error_index_out_of_bound(src, argv[-5]);
     break;
-  default: STk_error("incorrect number of arguments (%d)", argc);
-    return STk_void;      /* for the compiler */
+  default:
+    // already tested before. Needed to avoid warnings
+    return STk_void;
   }
 
   if (!INTP(argv[-1])) error_bad_index(argv[-1]);
@@ -902,7 +907,7 @@ DEFINE_PRIMITIVE("string-replace!", string_dreplace, vsubr, (int argc, SCM* argv
        variable. do it here. */
     start_char_dst = STk_utf8_index(STRING_CHARS(dst),dst_start,STRING_SIZE(dst));
 
-  char* start_char_src = STk_utf8_index(STRING_CHARS(src),src_start,STRING_SIZE(src));
+  char *start_char_src = STk_utf8_index(STRING_CHARS(src),src_start,STRING_SIZE(src));
   memcpy(start_char_dst, start_char_src, (unsigned long) src_substring_size);
 
   STRING_LENGTH(dst) = STRING_LENGTH(dst) + (src_end - src_start) - (dst_end - dst_start);
@@ -923,8 +928,7 @@ DEFINE_PRIMITIVE("string-replace!", string_dreplace, vsubr, (int argc, SCM* argv
  * which must be a list of characters. |String->list| and
  * |list->string| are inverses so far as |equal?| is concerned.
  *
- * @l
- * ,(bold "Note"): The R5RS version of |string->list| accepts only one
+ * NOTE: The R5RS version of |string->list| accepts only one
  * parameter.
 doc>
  */
@@ -934,7 +938,7 @@ DEFINE_PRIMITIVE("string->list", string2list, subr1, (SCM str))
 {
   register char *s;
   int len;
-  uint32_t c;
+  utf8_char c;
   SCM tmp, tmp1, z;
 
   if (!STRINGP(str)) error_bad_string(str);
@@ -993,8 +997,8 @@ DEFINE_PRIMITIVE("list->string", list2string, subr1, (SCM l))
  *
  * Returns a newly allocated copy of the part of the given |string|
  * between |start| and |stop|.
- * @l
- * ,(bold "Note"): The R5RS version of |string-copy| accepts only one argument.
+ *
+ * NOTE: The R5RS version of |string-copy| accepts only one argument.
 doc>
 */
 DEFINE_PRIMITIVE("string-copy", string_copy, vsubr, (int argc, SCM *argv))
@@ -1017,7 +1021,7 @@ DEFINE_PRIMITIVE("string-copy", string_copy, vsubr, (int argc, SCM *argv))
  *
  * Stores |char| in every element of the given |string| between |start| and |end|.
  * @l
- * ,(bold "Note"): The R5RS version of |string-fill!| accepts only one argument.
+ * NOTE: The R5RS version of |string-fill!| accepts only one argument.
 doc>
 */
 DEFINE_PRIMITIVE("string-fill!", string_fill, subr2, (SCM str, SCM c))
@@ -1125,7 +1129,7 @@ DEFINE_PRIMITIVE("string-find?", string_find, subr2, (SCM s1, SCM s2))
  * (string-position "ba" "abracadabra") =>  #f
  * @end lisp
  *
- * ,(bold "Note") This function was also called |string-index|. This name is deprecated
+ * NOTE: This function was also called |string-index|. This name is deprecated
  * since it conficts with the |string-index| defined in SRFI-13.
 doc>
 */
@@ -1149,7 +1153,7 @@ DEFINE_PRIMITIVE("string-position", string_position, subr2, (SCM s1, SCM s2))
  * (string-split str)
  * (string-split str delimiters)
  *
- * parses |string| and returns a list of tokens ended by a character of the
+ * Parses |string| and returns a list of tokens ended by a character of the
  * |delimiters| string. If |delimiters| is omitted, it defaults to a string
  * containing a space, a tabulation and a newline characters.
  * @lisp
@@ -1229,12 +1233,12 @@ DEFINE_PRIMITIVE("string-mutable?", string_mutable, subr1, (SCM obj))
  * (string-downcase "Foo BAR" 4 6)    => "ba"
  * @end lisp
  *
- * ,(bold "Note"): In R7RS, |string-downcase| accepts only one argument.
+ * NOTE: In R7RS, |string-downcase| accepts only one argument.
 doc>
  */
 static SCM string_xxcase(int argc, SCM *argv,
                          int (*toxx)(int),
-                         uint32_t (*towxx)(uint32_t))
+                         utf8_char (*towxx)(utf8_char))
 {
   SCM s;
   long start, end;
@@ -1242,7 +1246,7 @@ static SCM string_xxcase(int argc, SCM *argv,
   s = control_index(argc, argv, &start, &end);
 
   if (STk_use_utf8 && !STRING_MONOBYTE(s)) {
-    uint32_t *wchars;
+    utf8_char *wchars;
     int len;
     char *startp = STk_utf8_index(STRING_CHARS(s), start, STRING_SIZE(s));
 
@@ -1283,7 +1287,7 @@ doc>
 */
 static SCM string_dxxcase(int argc, SCM *argv,
                           int (*toxx)(int),
-                          uint32_t (*towxx)(uint32_t))
+                          utf8_char (*towxx)(utf8_char))
 {
   SCM s;
   long i, start, end;
@@ -1292,7 +1296,7 @@ static SCM string_dxxcase(int argc, SCM *argv,
   if (BOXED_INFO(s) & STRING_CONST) error_change_const_string(s);
 
   if (STk_use_utf8 && !STRING_MONOBYTE(s)) {        /* multibyte string */
-    uint32_t *wchars;
+    utf8_char *wchars;
     int len;
     char *startp = STk_utf8_index(STRING_CHARS(s), start, STRING_SIZE(s));
     char *endp   = STk_utf8_index(STRING_CHARS(s), end, STRING_SIZE(s));
@@ -1303,7 +1307,7 @@ static SCM string_dxxcase(int argc, SCM *argv,
       copy_array(wchars, end-start, startp);
     }
     else {
-      /* This code is inefficient, but it seems that that the converted case
+      /* This code is inefficient, but it seems that the converted case
          character always use the same length encoding. It is likely that this
          code is never used in practice
       */
@@ -1335,7 +1339,7 @@ DEFINE_PRIMITIVE("string-downcase!", string_ddowncase, vsubr, (int argc, SCM *ar
  * If |start| is omited, it defaults to 0. If |end| is omited, it defaults to
  * the length of |str|.
  * @l
- * ,(bold "Note"): In R7RS, |string-upcase| accepts only one argument.
+ * NOTE: In R7RS, |string-upcase| accepts only one argument.
 doc>
  */
 DEFINE_PRIMITIVE("string-upcase", string_upcase, vsubr, (int argc, SCM *argv))
@@ -1369,7 +1373,7 @@ DEFINE_PRIMITIVE("string-upcase!", string_dupcase, vsubr, (int argc, SCM *argv))
  * If |start| is omited, it defaults to 0. If |end| is omited, it defaults to
  * the length of |str|.
  * @l
- * ,(bold "Note"): In R7RS, |string-foldcase| accepts only one argument.
+ * NOTE: In R7RS, |string-foldcase| accepts only one argument.
 doc>
  */
 DEFINE_PRIMITIVE("string-foldcase", string_foldcase, vsubr, (int argc, SCM *argv))
