@@ -2,7 +2,7 @@
  *
  * s y m b o l . c                      -- Symbols management
  *
- * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,6 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 20-Nov-1993 12:12
- * Last file update: 28-Sep-2021 11:41 (eg)
  */
 
 #include <ctype.h>
@@ -42,7 +41,7 @@ static void error_bad_string(SCM str)
   STk_error("bad string ~S", str);
 }
 
-int STk_symbol_flags(register char *s)
+int STk_symbol_flags(register const char *s)
 {
   if (!*s || (*s == '.' && !s[1])) {
     /* Special symbols || and |.| which always need bars */
@@ -68,7 +67,7 @@ int STk_symbol_flags(register char *s)
   }
 }
 
-SCM STk_make_uninterned_symbol(char *name)
+SCM STk_make_uninterned_symbol(const char *name)
 {
   SCM z;
 
@@ -135,7 +134,7 @@ doc>
 DEFINE_PRIMITIVE("symbol->string", symbol2string, subr1, (SCM symbol))
 {
   SCM str;
-  char *s;
+  const char *s;
 
   if (!SYMBOLP(symbol)) STk_error("bad symbol ~S", symbol);
 
@@ -155,18 +154,19 @@ DEFINE_PRIMITIVE("symbol->string", symbol2string, subr1, (SCM symbol))
  * because in some implementations of Scheme they cannot be read as themselves.
  *
  * @lisp
- *    (eq? 'mISSISSIppi 'mississippi)     =>  #t
- *    (string->symbol "mISSISSIppi")      =>  @pipemISSISSIppi@pipe
- *    (eq? 'bitBlt (string->symbol "bitBlt"))
- *                                        =>  #f
+ *    (string->symbol "mISSISSIppi")                    =>  @pipemISSISSIppi@pipe
+ *    (eq? 'bitBlt (string->symbol "bitBlt"))           =>  #f
  *    (eq? 'JollyWog
  *         (string->symbol
- *           (symbol->string 'JollyWog))) =>  #t
+ *           (symbol->string 'JollyWog)))               =>  #t
  *    (string=? "K. Harper, M.D."
  *              (symbol->string
- *                (string->symbol "K. Harper, M.D.")))
- *                                        =>  #t
+ *                (string->symbol "K. Harper, M.D.")))  =>  #t
  * @end lisp
+ *
+ * NOTE: The expression `(eq? 'mISSISSIppi 'mississippi)` returns `#f` if
+ * {{stklos}} is running in case-sensitive mode (default), whereas it returns
+ * `#t` otherwise.
 doc>
  */
 DEFINE_PRIMITIVE("string->symbol", string2symbol, subr1, (SCM string))
@@ -181,7 +181,7 @@ DEFINE_PRIMITIVE("string->symbol", string2symbol, subr1, (SCM string))
  * (string->unterned-symbol string)
  *
  * Returns the symbol whose print name is made from the characters of
- * |string|. This symbol is guaranteed to be ,(emph "unique") (i.e. not
+ * |string|. This symbol is guaranteed to be _unique_ (i.e. not
  * |eq?| to any other symbol):
  * @lisp
  * (let ((ua (string->uninterned-symbol "a")))
