@@ -1,7 +1,7 @@
 /*
  * thread-common.c                      -- Threads support in STklos
  *
- * Copyright © 2006-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@essi.fr>
+ * Copyright © 2006-2024 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@essi.fr]
  *    Creation date: 23-Jan-2006 12:14 (eg)
- * Last file update: 10-Apr-2021 18:48 (eg)
  */
 #include <unistd.h>
 #include "stklos.h"
@@ -76,7 +75,7 @@ struct timeval STk_thread_abstime_to_reltime(double abs_secs)
 <doc EXT current-thread
  * (current-thread)
  *
- *  Returns the current thread.
+ * Returns the current thread.
  * @lisp
  * (eq? (current-thread) (current-thread))  =>  #t
  * @end lisp
@@ -131,7 +130,7 @@ static SCM do_make_thread(SCM thunk, SCM name, int stack_size)
 DEFINE_PRIMITIVE("%make-thread", make_thread, subr3,(SCM thunk, SCM name, SCM ssize))
 {
   SCM z;
-  int stack_size;
+  long stack_size;
 
   if (STk_procedurep(thunk) == STk_false)
     STk_error("bad thunk ~S", thunk);
@@ -186,7 +185,11 @@ DEFINE_PRIMITIVE("thread-name", thread_name, subr1, (SCM thr))
  * (thread-stack-size thread)
  *
  * Returns the allocated stack size for |thread|.
- * Note that this procedure is not present in ,(quick-link-srfi 18).
+ * @lisp
+ * (thread-stack-size (make-thread (lambda () #f) 'foo 2000)) => 2000
+ * @end lisp
+ *
+ * Note that this procedure is not present in {{quick-link-srfi 18}}.
 doc>
 */
 DEFINE_PRIMITIVE("thread-stack-size", thread_ssize, subr1, (SCM thr))
@@ -291,14 +294,58 @@ DEFINE_PRIMITIVE("thread-start!", thread_start, subr1, (SCM thr))
   new->eport          = vm->eport;
   new->scheme_thread  = thr;
 
-  THREAD_VM(thr)      = new;
-  THREAD_STATE(thr)   = th_runnable;
+  THREAD_VM(thr)              = new;
+  THREAD_STATE(thr)           = th_runnable;
+  THREAD_ALLOCATIONS(thr)     = 0;
+  THREAD_BYTES_ALLOCATED(thr) = 0;
 
   STk_sys_thread_start(thr);
 
   return thr;
 }
 
+
+/*
+ *
+ * Thread memory allocation counter management
+ *
+ */
+void STk_thread_inc_allocs(SCM thr, size_t size)
+{
+  THREAD_ALLOCATIONS(thr)++;
+  THREAD_BYTES_ALLOCATED(thr) += size;
+}
+
+DEFINE_PRIMITIVE("%thread-allocation-reset!",
+                 thread_allocs_reset, subr01, (SCM thr))
+{
+  if (!thr) thr = STk_current_thread();
+  else
+    if (!THREADP(thr)) STk_error_bad_thread(thr);
+
+  THREAD_BYTES_ALLOCATED(thr) = 0;
+  THREAD_ALLOCATIONS(thr)     = 0;
+  return STk_void;
+}
+
+DEFINE_PRIMITIVE("%thread-allocation-counter", thread_alloc_count, subr01, (SCM thr))
+{
+  if (!thr) thr = STk_current_thread();
+  else
+    if (!THREADP(thr)) STk_error_bad_thread(thr);
+
+  return STk_ulong2integer(THREAD_ALLOCATIONS(thr));
+}
+
+DEFINE_PRIMITIVE("%thread-allocation-bytes", thread_alloc_bytes, subr01, (SCM thr))
+{
+  if (!thr) thr = STk_current_thread();
+  else
+    if (!THREADP(thr)) STk_error_bad_thread(thr);
+
+  return STk_ulong2integer(THREAD_BYTES_ALLOCATED(thr));
+}
+
 /* ======================================================================
  *      Initialization ...
  * ======================================================================
@@ -362,11 +409,13 @@ int STk_init_threads(int stack_size, void *start_stack)
   primordial = do_make_thread(STk_false,
                               STk_Cstring2string("primordial"),
                               stack_size);
-  THREAD_STATE(primordial) = th_runnable;
-  THREAD_VM(primordial)    = vm;
-  vm->scheme_thread        = primordial;
-  vm->start_stack          = start_stack;
-  STk_primordial_thread    = primordial;
+  THREAD_STATE(primordial)           = th_runnable;
+  THREAD_VM(primordial)              = vm;
+  THREAD_ALLOCATIONS(primordial)     = 0;
+  THREAD_BYTES_ALLOCATED(primordial) = 0;
+  vm->scheme_thread                  = primordial;
+  vm->start_stack                    = start_stack;
+  STk_primordial_thread              = primordial;
 
   /* Thread primitives */
   ADD_PRIMITIVE(current_thread);
@@ -388,6 +437,10 @@ int STk_init_threads(int stack_size, void *start_stack)
   ADD_PRIMITIVE(thread_join);
   ADD_PRIMITIVE(thread_sleep);
   ADD_PRIMITIVE(thread_system);
+  ADD_PRIMITIVE(thread_allocs_reset);
+  ADD_PRIMITIVE(thread_alloc_count);
+  ADD_PRIMITIVE(thread_alloc_bytes);
+  
 
   return TRUE;
 }
