@@ -1,7 +1,7 @@
 /*
  * utf8.c               -- UTF-8 support functions
  *
- * Copyright © 2011-2021 Erick Gallesio - Polytech'Nice-Sophia <eg@unice.fr>
+ * Copyright © 2011-2023 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 30-Apr-2011 19:46 (eg)
- * Last file update: 10-Sep-2021 11:40 (eg)
  */
 
 #include "stklos.h"
@@ -29,7 +28,7 @@
 int STk_use_utf8 = -1;
 
 
-static void error_bad_sequence(char *str)
+static void error_bad_sequence(const char *str)
 {
   int i;
   char *buffer = STk_must_malloc_atomic(strlen(str) + 1);
@@ -43,7 +42,7 @@ static void error_bad_sequence(char *str)
 }
 
 
-char *STk_utf8_grab_char(char *str, uint32_t *c) /* result = pos. after current one */
+char *STk_utf8_grab_char(char *str, utf8_char *c) /* result = pos. after current one */
 {
   uint8_t *buff = (uint8_t *) str;
 
@@ -68,8 +67,8 @@ char *STk_utf8_grab_char(char *str, uint32_t *c) /* result = pos. after current
     return str + 3;
   }
 
-  *c = ((buff[0] & 0x0f) << 16) +
-       ((buff[1] & 0x3f) <<  6) +
+  *c = ((buff[0] & 0x07) << 18) +
+       ((buff[1] & 0x3f) << 12) +
        ((buff[2] & 0x3f) <<  6) +
         (buff[3] & 0x3f);
   return str + 4;
@@ -80,7 +79,7 @@ int STk_utf8_read_char(SCM port)
   int c = STk_getc(port);
 
   if (STk_use_utf8 && (c >= 0x80)) {
-    /* Read an UTF-8 character */
+    /* Read a UTF-8 character */
     if ((c < 0xc0) || (c > 0xf7))
       return UTF8_INCORRECT_SEQUENCE;
     else if (c < 0xe0) {
@@ -141,9 +140,9 @@ int STk_utf8_char_bytes_needed(unsigned int ch)
   return 1; /* to avoid infinite loop, but obiously incorrect */
 }
 
-int STk_utf8_sequence_length(char *str)
+int STk_utf8_sequence_length(const char *str)
 {
-  /* return length of a the UTF-8 sequence starting at given address */
+  /* return length of the UTF-8 sequence starting at given address */
   uint8_t c = *((uint8_t *) str);
 
   if (c < 0x80)                         return 1;
@@ -154,10 +153,10 @@ int STk_utf8_sequence_length(char *str)
 }
 
 
-int STk_utf8_strlen(char *s, int max)
+int STk_utf8_strlen(const char *s, int max)
 {
   int len;
-  char *start = s, *end = s + max;
+  const char *start = s, *end = s + max;
 
   for (len = 0;  s < end; len++) {
     int sz =  STk_utf8_sequence_length(s);
@@ -217,10 +216,10 @@ int STk_utf8_char_from_byte(char *s, int i, int max) /*  byte index => char inde
   return (s == idx) ? pos : -1;
 }
 
-/* ======================================================================
- *      STklos Primitives
- * ====================================================================== */
 #ifdef STK_DEBUG
+/* ======================================================================
+ *      STklos Primitives (defined only if we are in debug mode)
+ * ====================================================================== */
 DEFINE_PRIMITIVE("%char-utf8-encoding", char_utf8_encoding, subr1, (SCM c))
 {
   SCM lst = STk_nil;
@@ -237,8 +236,8 @@ DEFINE_PRIMITIVE("%char-utf8-encoding", char_utf8_encoding, subr1, (SCM c))
 
 DEFINE_PRIMITIVE("%dump-string", dump_string, subr12, (SCM str, SCM index))
 {
-  int i;
-  uint32_t c = 0;
+  long i;
+  utf8_char c = 0;
 
   STk_debug("String ~S. space=%d, size=%d, len =%d", str,
             STRING_SPACE(str), STRING_SIZE(str), STRING_LENGTH(str));
@@ -250,7 +249,7 @@ DEFINE_PRIMITIVE("%dump-string", dump_string, subr12, (SCM str, SCM index))
 
   if (index) {
     i = STk_integer_value(index);
-    printf("------\nChar starting at index %d\n", i);
+    printf("------\nChar starting at index %ld\n", i);
     STk_debug("  length of char = %d",
               STk_utf8_sequence_length(&(STRING_CHARS(str)[i])));
     STk_utf8_grab_char(STRING_CHARS(str)+i, &c);
@@ -260,17 +259,14 @@ DEFINE_PRIMITIVE("%dump-string", dump_string, subr12, (SCM str, SCM index))
   return STk_void;
 }
 
-#endif
-
-
 /* ======================================================================
  *      Initialization
  * ====================================================================== */
 int STk_init_utf8(void)
 {
-#ifdef STK_DEBUG
   ADD_PRIMITIVE(char_utf8_encoding);
   ADD_PRIMITIVE(dump_string);
-#endif
   return TRUE;
 }
+
+#endif
