@@ -1,7 +1,7 @@
 /*
  * u v e c t o r . c                    -- Uniform Vectors Implementation
  *
- * Copyright © 2001-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2001-2024 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,12 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 15-Apr-2001 10:13 (eg)
- * Last file update: 10-Apr-2021 18:47 (eg)
  */
 
+#include <float.h>
+#include <math.h>
 #include "stklos.h"
 
-int STk_uvectors_allowed = 0;
 static SCM u64_max, s64_min, s64_max;
 
 
@@ -35,19 +35,21 @@ static SCM u64_max, s64_min, s64_max;
  *
  */
 
-static char* type_vector(int tip)
+static char *type_vector(int tip)
 {
   switch (tip) {
-    case UVECT_S8:  return "s8";
-    case UVECT_U8:  return "u8";
-    case UVECT_S16: return "s16";
-    case UVECT_U16: return "u16";
-    case UVECT_S32: return "s32";
-    case UVECT_U32: return "u32";
-    case UVECT_S64: return "s64";
-    case UVECT_U64: return "u64";
-    case UVECT_F32: return "f32";
-    case UVECT_F64: return "f64";
+    case UVECT_S8:   return "s8";
+    case UVECT_U8:   return "u8";
+    case UVECT_S16:  return "s16";
+    case UVECT_U16:  return "u16";
+    case UVECT_S32:  return "s32";
+    case UVECT_U32:  return "u32";
+    case UVECT_S64:  return "s64";
+    case UVECT_U64:  return "u64";
+    case UVECT_F32:  return "f32";
+    case UVECT_F64:  return "f64";
+    case UVECT_C64:  return "c64";
+    case UVECT_C128: return "c128";
     default:        return ""; /* never reached */
   }
 }
@@ -58,40 +60,43 @@ static void error_change_const_vector(SCM v)
   STk_error("changing the constant vector ~s is not allowed", v);
 }
 
-static void error_bad_vector(SCM v)
-{
-  STk_error("bad vector ~s", v);
-}
-
-static void error_bad_uvector(SCM v, int tip)
+static void error_bad_typed_uvector(SCM v, int tip)
 {
   STk_error("bad #%s vector ~s", type_vector(tip), v);
 }
 
-static void error_bad_index(SCM index)
-{
-  STk_error("index ~S is invalid or out of bounds", index);
-}
-
 static void error_bad_length(SCM length)
 {
   STk_error("invalid vector length ~s", length);
 }
 
-static void error_bad_uniform_type(SCM type)
-{
-  STk_error("bad type for an uniform vector ~s", type);
-}
 
 static void error_bad_list(SCM l)
 {
   STk_error("bad list ~s", l);
 }
 
-
-static int vector_element_size(int type)
+static inline void check_uvector(SCM o)
 {
-  /* compute len of one element depending of type.  We assume here
+  if (!UVECTORP(o)) STk_error("bad uniform vector ~s", o);
+}
+
+static inline void check_index(SCM uvect, SCM index, long i)
+{
+  if (i < 0 || i >= UVECTOR_SIZE(uvect))
+    STk_error("index ~S is invalid or out of bounds", index);
+}
+
+static inline void check_uniform_type(SCM type, long i)
+{
+  if (i < UVECT_S8 || i > UVECT_C128)
+    STk_error("bad type for an uniform vector ~s", type);
+}
+
+
+int STk_vector_element_size(int type)
+{
+  /* compute len of one element depending on type.  We assume here
    * that characters use 8 bits and that we are at least on a 32 bits
    * architecture. Consquenetly, S8, S16 and S32 are represented
    * without boxing whereas S64 are represeneted by a bignum
@@ -106,6 +111,8 @@ static int vector_element_size(int type)
     case UVECT_S64: case UVECT_U64: return sizeof(SCM);
     case UVECT_F32:                 return 4;
     case UVECT_F64:                 return 8;
+    case UVECT_C64:                 return 8;
+    case UVECT_C128:                return 16;
   }
   return 0; /* never reached */
 }
@@ -140,7 +147,7 @@ static SCM control_index(int argc, SCM *argv, long *pstart, long *pend, SCM *pfi
   }
 
   /* Controlling s */
-  if (!UVECTORP(v)) error_bad_uvector(v, UVECT_U8);
+  if (!UVECTORP(v)) error_bad_typed_uvector(v, UVECT_U8);
   len = UVECTOR_SIZE(v);
 
   /* Controlling start index */
@@ -166,11 +173,14 @@ static SCM control_index(int argc, SCM *argv, long *pstart, long *pend, SCM *pfi
 }
 
 
-/* Return the type of an uniform vector given its tag */
-int STk_uniform_vector_tag(char *s)
+/* Return the type of a uniform vector given its tag */
+int STk_uniform_vector_tag(const char *s)
 {
   static char *table[] =
-    {"s8", "u8", "s16", "u16", "s32", "u32", "s64", "u64", "f32", "f64", "" };
+    {"s8", "u8", "s16", "u16", "s32", "u32", "s64", "u64",
+     "f32", "f64",
+     "c64", "c128",
+     "" };
   char **p;
 
   for (p = table; **p; p++) {
@@ -186,17 +196,37 @@ int STk_uvector_equal(SCM u1, SCM u2)
       (UVECTOR_SIZE(u1) != UVECTOR_SIZE(u2)))
     return 0;
 
-  /* same length and same tyep, compare the bytes */
-  int len = vector_element_size(UVECTOR_TYPE(u1)) * UVECTOR_SIZE(u1);
+  /* same length and same type, compare the bytes */
+  int len = STk_vector_element_size(UVECTOR_TYPE(u1)) * UVECTOR_SIZE(u1);
 
   return (memcmp(UVECTOR_DATA(u1), UVECTOR_DATA(u2), len) == 0);
 }
 
+
+/* Duplicated from number.c: */
+static inline SCM Cmake_complex(SCM r, SCM i)
+{
+  SCM z;
+
+  NEWCELL(z, complex);
+  COMPLEX_REAL(z) = r;
+  COMPLEX_IMAG(z) = i;
+  return z;
+}
+
+
 /*
- * Basic accessors to an uniform vector
+ * We use exact->inexact to cast complex vectors in c64 and c128 types
+ * (possibly resultingin +inf.0 or -inf.0)
+ */
+EXTERN_PRIMITIVE("exact->inexact", ex2inex, subr1, (SCM z));
+
+
+/*
+ * Basic accessors to a uniform vector
  *
  */
-static void uvector_set(int _UNUSED(type), SCM v, long i, SCM value)
+static void uvector_set(SCM v, long i, SCM value)
 {
   long vali;
   int overflow;
@@ -261,15 +291,71 @@ static void uvector_set(int _UNUSED(type), SCM v, long i, SCM value)
         }
       break;
     case UVECT_F32:
-      if (REALP(value)) {
-        ((float *) UVECTOR_DATA(v))[i] = (float) REAL_VAL(value);
-        return;
+    case UVECT_F64:
+      {
+        double d = STk_number2double(value);
+
+        // if value is not a number, STk_number2double returns a NaN. However,
+        // a NaN is also a correct value for a #f32 or #f64 number. So, value
+        // is correct is it is a NaN or it's conversion is not a NaN.
+        if (STk_isnan(value) || !isnan(d)) {
+          if (UVECTOR_TYPE(v) == UVECT_F32)
+            ((float *) UVECTOR_DATA(v))[i] = (float) d;
+          else
+            ((double *) UVECTOR_DATA(v))[i] = d;
+          return;
+        }
       }
       break;
-    case UVECT_F64:
-      if (REALP(value)) {
-        ((double *) UVECTOR_DATA(v))[i] = (double) REAL_VAL(value);
+    /*
+     Complexes are stored with the real part in the even-indexed cells, and
+     imaginary parts in odd-indexed cells:
+
+      -----+-----+-----+-----+-------
+     | r_1 | i_1 | r_2 | i_2 |  ...  |
+      -----+-----+-----+-----+-------
+     Every complex real-part and imag-part is transformed into inexact, so it would into
+     a double. Forthermore, for c64 vectors, they're downcasted to floats.
+    */
+    case UVECT_C64:
+      if (COMPLEXP(value)) {
+        /* Following what exact->inexact does, we don't signal error on
+           overflow when converting. This is similar to what other Schemes
+           do. */
+        SCM rea = STk_ex2inex(COMPLEX_REAL(value));
+        SCM img = STk_ex2inex(COMPLEX_IMAG(value));
+        /* We'll actually DOWNCAST the number to float + float.  */
+        ((float *) UVECTOR_DATA(v))[2*i]     = (float) REAL_VAL(rea);
+        ((float *) UVECTOR_DATA(v))[2*i + 1] = (float) REAL_VAL(img);
         return;
+      } else {
+        float f = STk_number2double(value);
+
+        if (STk_isnan(value) || !isnan(f)) { // See comment for F64
+          ((float *) UVECTOR_DATA(v))[2*i]     = f;
+          ((float *) UVECTOR_DATA(v))[2*i + 1] = (float) 0.0;
+          return;
+        }
+      }
+      break;
+    case UVECT_C128:
+      if (COMPLEXP(value)) {
+        /* Following what exact->inexact does, we don't signal error on
+           overflow when converting. This is similar to what other Schemes
+           do. */
+        SCM rea = STk_ex2inex(COMPLEX_REAL(value));
+        SCM img = STk_ex2inex(COMPLEX_IMAG(value));
+        ((double *) UVECTOR_DATA(v))[2*i]     = (double) REAL_VAL(rea);
+        ((double *) UVECTOR_DATA(v))[2*i + 1] = (double) REAL_VAL(img);
+        return;
+      } else {
+        double d = STk_number2double(value);
+
+        if (STk_isnan(value) || !isnan(d)) {  // See comment for F64
+          ((double *) UVECTOR_DATA(v))[2*i]     = d;
+          ((double *) UVECTOR_DATA(v))[2*i + 1] = (double) 0.0;
+          return;
+        }
       }
       break;
   }
@@ -279,7 +365,13 @@ static void uvector_set(int _UNUSED(type), SCM v, long i, SCM value)
             value, type_vector(UVECTOR_TYPE(v)));
 }
 
-static SCM uvector_ref(int _UNUSED(type), SCM v, long i)
+
+void STk_uvector_put(SCM v, long i, SCM value) /* public version of uvector_set */
+{
+  uvector_set(v, i, value);
+}
+
+static SCM uvector_ref(SCM v, long i)
 {
   switch (UVECTOR_TYPE(v)) {
     case UVECT_S8: return MAKE_INT(((char *) UVECTOR_DATA(v))[i]);
@@ -295,10 +387,40 @@ static SCM uvector_ref(int _UNUSED(type), SCM v, long i)
 
     case UVECT_F32: return STk_double2real(((float *) UVECTOR_DATA(v))[i]);
     case UVECT_F64: return STk_double2real(((double *) UVECTOR_DATA(v))[i]);
+
+   /*
+     Complexes are stored with the real part in the even-indexed cells, and
+     imaginary parts in odd-indexed cells:
+
+      -----+-----+-----+-----+-------
+     | r_1 | i_1 | r_2 | i_2 |  ...  |
+      -----+-----+-----+-----+-------
+     Every complex real-part and imag-part is transformed into inexact, so it would into
+     a double. Forthermore, for c64 vectors, they're downcasted to floats.
+    */
+    case UVECT_C64:
+        /* Don't return complexes with zero imaginary part! Those are reals... */
+        if ( ( (float) (((float*)UVECTOR_DATA(v))[2*i + 1]) ) == 0.0 )
+            return STk_double2real( (float) (((float*)UVECTOR_DATA(v))[2*i]) );
+        else
+            return Cmake_complex(STk_double2real((float) ((float *) UVECTOR_DATA(v))[2*i]),
+                                 STk_double2real((float) ((float *) UVECTOR_DATA(v))[2*i + 1]));
+
+    case UVECT_C128:
+        /* Don't return complexes with zero imaginary part! Those are reals... */
+        if ( ( (double) (((double*) UVECTOR_DATA(v))[2*i + 1]) ) == 0.0 )
+            return STk_double2real( (double) (((double*) UVECTOR_DATA(v))[2*i]) );
+        else
+            return Cmake_complex(STk_double2real(((double *) UVECTOR_DATA(v))[2*i]),
+                                 STk_double2real(((double *) UVECTOR_DATA(v))[2*i + 1]));
   }
   return STk_void; /* never reached */
 }
 
+SCM STk_uvector_get(SCM v, long i)      /* public version of uvector_ref */
+{
+  return uvector_ref(v, i);
+}
 
 /*
  *
@@ -310,13 +432,13 @@ static SCM makeuvect(int type, int len, SCM init)
   long i, size = 1;
   SCM  z;
 
-  /* compute len of one element depending of type.  We assume here
+  /* compute len of one element depending on type.  We assume here
    * that characters use 8 bits and that we are at least on a 32 bits
    * architecture. Consquenetly, S8, S16 and S32 are represented
    * without boxing whereas S64 are represeneted by a bignum
    * (even on 64 machines where we can do better). Furthermore, we
    * suppose that C floats and doubles correspond to single and
-   * double IEEE-754 reals
+   * double IEEE-754 reals.
    */
   switch (type) {
     case UVECT_S8:  case UVECT_U8:  size = 1;           break;
@@ -325,18 +447,28 @@ static SCM makeuvect(int type, int len, SCM init)
     case UVECT_S64: case UVECT_U64: size = sizeof(SCM); break;
     case UVECT_F32:                 size = 4;           break;
     case UVECT_F64:                 size = 8;           break;
+    case UVECT_C64:                 size = 8;           break;
+    case UVECT_C128:                size = 16;          break;
   }
-  NEWCELL_WITH_LEN(z, uvector, sizeof(struct vector_obj) + size*len - 1);
+  NEWCELL_WITH_LEN(z, uvector, sizeof(struct uvector_obj) + size*len - 1);
   UVECTOR_TYPE(z) = type;
   UVECTOR_SIZE(z) = len;
 
   if (init) {
     for(i=0; i < len; i++)
-      uvector_set(type, z, i, init);
+      uvector_set(z, i, init);
   }
   return z;
 }
 
+
+// public version of makeuvect (used by srfi-160)
+SCM STk_makeuvect(int type, int len, SCM init)
+{
+  return makeuvect(type, len, init);
+}
+
+
 SCM STk_list2uvector(int type, SCM l)
 {
   long i, len = STk_int_length(l);
@@ -346,12 +478,14 @@ SCM STk_list2uvector(int type, SCM l)
 
   z = makeuvect(type, len, (SCM) NULL);
   for (i = 0; i < len; i++) {
-    uvector_set(type, z, i, CAR(l));
+    uvector_set(z, i, CAR(l));
     l = CDR(l);
   }
   return z;
 }
 
+
+
 /*===========================================================================*\
  *
  * User primitives on uniform vectors.
@@ -359,96 +493,122 @@ SCM STk_list2uvector(int type, SCM l)
  *
 \*===========================================================================*/
 
-DEFINE_PRIMITIVE("%make-uvector", make_uvector, subr3,(SCM type, SCM len, SCM init))
+DEFINE_PRIMITIVE("%make-uvector", make_uvector, subr3, (SCM len, SCM init, SCM type))
 {
   long l   = STk_integer_value(len);
   long tip = STk_integer_value(type);
 
-  if (l < 0)                             error_bad_length(len);
-  if (tip < UVECT_S8 || tip > UVECT_F64) error_bad_uniform_type(type);
+  if (l < 0) error_bad_length(len);
+  check_uniform_type(type, tip);
 
   return makeuvect(tip, l, init);
 }
 
-DEFINE_PRIMITIVE("%uvector?", uvectorp, subr2, (SCM type, SCM vect))
+DEFINE_PRIMITIVE("%uvector?", uvectorp, subr12, (SCM vect, SCM type))
 {
-  long tip = STk_integer_value(type);
-  return MAKE_BOOLEAN(UVECTORP(vect) && UVECTOR_TYPE(vect) == tip);
+  if (type) {
+    long tip = STk_integer_value(type);
+    return MAKE_BOOLEAN(UVECTORP(vect) && UVECTOR_TYPE(vect) == tip);
+  }
+  else
+    return MAKE_BOOLEAN(UVECTORP(vect));
 }
 
-DEFINE_PRIMITIVE("%uvector", uvector, subr2, (SCM type, SCM values))
+DEFINE_PRIMITIVE("%uvector", uvector, subr2, (SCM values, SCM type))
 {
   long tip = STk_integer_value(type);
 
-  if (tip < UVECT_S8 || tip > UVECT_F64) error_bad_uniform_type(type);
+  check_uniform_type(type, tip);
   return STk_list2uvector(tip, values);
 }
 
-DEFINE_PRIMITIVE("%uvector-length", uvector_length, subr2, (SCM type, SCM v))
+DEFINE_PRIMITIVE("%uvector-length", uvector_length, subr12, (SCM v, SCM type))
 {
-  long tip = STk_integer_value(type);
-
-  if (tip < UVECT_S8 || tip > UVECT_F64)        error_bad_uniform_type(type);
-  if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip)) error_bad_uvector(v, tip);
-
+  check_uvector(v);
+  if (type) {
+    long tip = STk_integer_value(type);
+    check_uniform_type(type, tip);
+    if (UVECTOR_TYPE(v) != tip) error_bad_typed_uvector(v, tip);
+  }
   return MAKE_INT(UVECTOR_SIZE(v));
 }
 
-DEFINE_PRIMITIVE("%uvector-ref", uvector_ref, subr3, (SCM type, SCM v, SCM index))
+DEFINE_PRIMITIVE("%uvector-ref", uvector_ref, subr23, (SCM v, SCM index, SCM type))
 {
-  long i   = STk_integer_value(index);
-  long tip = STk_integer_value(type);
+  long i = STk_integer_value(index);
 
-  if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip))  error_bad_vector(v);
-  if (tip < UVECT_S8 || tip > UVECT_F64)         error_bad_uniform_type(type);
-  if (i < 0 || i >= UVECTOR_SIZE(v))             error_bad_index(index);
+  check_uvector(v);
+  check_index(v, index, i);
 
-  return uvector_ref(tip, v, i);
+  if (type) {
+    long tip = STk_integer_value(type);
+
+    check_uniform_type(type, tip);
+    if (UVECTOR_TYPE(v) != tip) error_bad_typed_uvector(v, tip);
+  }
+
+  return uvector_ref(v, i);
 }
 
-DEFINE_PRIMITIVE("%uvector-set!", uvector_set, subr4,
-                 (SCM type, SCM v, SCM index, SCM value))
+DEFINE_PRIMITIVE("%uvector-set!", uvector_set, subr34,
+                 (SCM v, SCM index, SCM value, SCM type))
 {
-  long i   = STk_integer_value(index);
-  long tip = STk_integer_value(type);
+  long i = STk_integer_value(index);
 
-  if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip)) error_bad_vector(v);
-  if (tip < UVECT_S8 || tip > UVECT_F64)        error_bad_uniform_type(type);
-  if (BOXED_INFO(v) & VECTOR_CONST)             error_change_const_vector(v);
-  if (i < 0 || i >= UVECTOR_SIZE(v))            error_bad_index(index);
+  check_uvector(v);
+  check_index(v, index, i);
+  if (BOXED_INFO(v) & VECTOR_CONST)  error_change_const_vector(v);
 
-  uvector_set(tip, v, i, value);
+  if (type) {
+    long tip = STk_integer_value(type);
+
+    check_uniform_type(type, tip);
+    if (UVECTOR_TYPE(v) != tip) error_bad_typed_uvector(v, tip);
+  }
+
+  uvector_set(v, i, value);
   return STk_void;
 }
 
 
-DEFINE_PRIMITIVE("%uvector->list", uvector_list, subr2, (SCM type, SCM v))
+DEFINE_PRIMITIVE("%uvector->list", uvector_list, subr12, (SCM v, SCM type))
 {
-  long i, len, tip = STk_integer_value(type);
+  long i, len;
   SCM z, tmp;
 
-  if (tip < UVECT_S8 || tip > UVECT_F64)        error_bad_uniform_type(type);
-  if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip)) error_bad_vector(v);
+  check_uvector(v);
+  if (type) {
+    long tip = STk_integer_value(type);
+    check_uniform_type(type, tip);
+    if (UVECTOR_TYPE(v) != tip)              error_bad_typed_uvector(v, tip);
+  }
 
   len = UVECTOR_SIZE(v);
   if (!len) return STk_nil;
 
   /* len > 0. Build the fist cell and iterate */
-  tmp = z = STk_cons(uvector_ref(tip, v, 0), STk_nil);
+  tmp = z = STk_cons(uvector_ref(v, 0), STk_nil);
   for (i=1; i<len; i++) {
-    tmp = CDR(tmp) = STk_cons(uvector_ref(tip, v, i), STk_nil);
+    tmp = CDR(tmp) = STk_cons(uvector_ref(v, i), STk_nil);
   }
   return z;
 }
 
-DEFINE_PRIMITIVE("%list->uvector", list_uvector, subr2, (SCM type, SCM l))
+DEFINE_PRIMITIVE("%list->uvector", list_uvector, subr2, (SCM l, SCM type))
 {
   long tip = STk_integer_value(type);
 
-  if (tip < UVECT_S8 || tip > UVECT_F64) error_bad_uniform_type(type);
+  check_uniform_type(type, tip);
   return STk_list2uvector(tip, l);
 }
 
+DEFINE_PRIMITIVE("uvector-tag", uvector_tag, subr1, (SCM v))
+{
+  check_uvector(v);
+  return STk_intern(type_vector(UVECTOR_TYPE(v)));
+}
+
+
 DEFINE_PRIMITIVE("%allow-uvectors", allow_uvectors, subr0, (void))
 {
   ADD_PRIMITIVE(uvector_list);
@@ -459,9 +619,14 @@ DEFINE_PRIMITIVE("%allow-uvectors", allow_uvectors, subr0, (void))
   s64_min = STk_Cstr2number(S64_MIN, 10);
   s64_max = STk_Cstr2number(S64_MAX, 10);
 
-  /* Retain that we can use uniform vectors from now on */
-  STk_uvectors_allowed = 1;
+  /* Add new readers #xxx syntax */
+  STk_add_uvector_reader_tag("s8");  /* "u8" is defined by default */
+  STk_add_uvector_reader_tag("s16"); STk_add_uvector_reader_tag("u16");
 
+  STk_add_uvector_reader_tag("s32"); STk_add_uvector_reader_tag("u32");
+  STk_add_uvector_reader_tag("s64"); STk_add_uvector_reader_tag("u64");
+  STk_add_uvector_reader_tag("f32"); STk_add_uvector_reader_tag("f64");
+  STk_add_uvector_reader_tag("c64"); STk_add_uvector_reader_tag("c128");
 
   return STk_void;
 }
@@ -476,11 +641,10 @@ static void print_uvector(SCM vect, SCM port, int mode)
 {
   int i;
   int n = UVECTOR_SIZE(vect);
-  int t = UVECTOR_TYPE(vect);
 
   STk_fprintf(port, "#%s(", type_vector(UVECTOR_TYPE(vect)));
   for (i = 0; i < n; i++) {
-    STk_print(uvector_ref(t, vect, i), port, mode);
+    STk_print(uvector_ref(vect, i), port, mode);
     if (i < n - 1) STk_putc(' ', port);
   }
   STk_putc(')', port);
@@ -559,7 +723,7 @@ DEFINE_PRIMITIVE("bytevector-append", bytevector_append, vsubr,(int argc, SCM *a
   /* compute length of final result */
   for (i = 0; i < argc; i++) {
     if (!UVECTORP(argv[-i]) || UVECTOR_TYPE(argv[-i]) != UVECT_U8)
-      error_bad_uvector(argv[-i], UVECT_U8);
+      error_bad_typed_uvector(argv[-i], UVECT_U8);
     len += UVECTOR_SIZE(argv[-i]);
   }
 
@@ -613,11 +777,8 @@ DEFINE_PRIMITIVE("utf8->string", utf82string, vsubr, (int argc, SCM *argv))
   len        = end_addr - start_addr;
 
   /* Verify that the sub-vector denotes a correct string */
-  if (STk_utf8_verify_sequence((char *) start_addr, len)) {
-    SCM z = STk_makestring(len, NULL);
-    memcpy(STRING_CHARS(z), start_addr, end_addr - start_addr);
-    return z;
-  }
+  if (STk_utf8_verify_sequence((char *) start_addr, len))
+    return STk_makestring(len, (char *) start_addr);
   else
     STk_error("bad UTF8 sequence between %d and %d in ~S", start, end, v);
 
@@ -640,6 +801,7 @@ int STk_init_uniform_vector(void)
   ADD_PRIMITIVE(uvector_length);
   ADD_PRIMITIVE(uvector_ref);
   ADD_PRIMITIVE(uvector_set);
+  ADD_PRIMITIVE(uvector_tag);
 
   /* R7RS specific bytevectors primitives */
   ADD_PRIMITIVE(bytevector_copy);
