@@ -1,7 +1,7 @@
 /*
  * vport.c                                      -- Virtual Ports
  *
- * Copyright © 2005-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@essi.fr>
+ * Copyright © 2005-2024 Erick Gallesio <eg@stklos.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  *
  *           Author: Erick Gallesio [eg@essi.fr]
  *    Creation date: 17-Aug-2005 08:31 (eg)
- * Last file update: 28-Aug-2021 11:19 (eg)
  */
 
 #include "stklos.h"
@@ -123,7 +122,7 @@ static int call_user_close(void *stream)
  */
 
 static int call_user_putstring(SCM s, void *stream);
-static int vport_nputs(void *stream, char *s, int len);
+static int vport_nputs(void *stream, const char *s, int len);
 
 static int call_user_putc(int c, void *stream)
 {
@@ -184,10 +183,10 @@ static int vport_read(void *stream, void *buf, int count)
 }
 
 
-static int vport_write(void *stream, void *buf, int count)
+static int vport_write(void *stream, const void *buf, int count)
 {
   int i;
-  char *s = buf;
+  const char *s = buf;
 
   for (i = 0; i < count; i++) {
     int c = call_user_putc(*s++, stream);
@@ -204,7 +203,7 @@ static off_t vport_seek(void  _UNUSED(*stream),
   return 0;
 }
 
-static int vport_nputs(void *stream, char *s, int len)
+static int vport_nputs(void *stream, const char *s, int len)
 {
   int i;
 
@@ -213,7 +212,7 @@ static int vport_nputs(void *stream, char *s, int len)
   return len;
 }
 
-static int vport_puts(char *s, void *stream)
+static int vport_puts(const char *s, void *stream)
 {
   return vport_nputs(stream, s, strlen(s));
 }
@@ -231,18 +230,18 @@ static int vport_puts(char *s, void *stream)
  * Returns a virtual port using the |read-char| procedure to read a
  * character from the port, |ready?| to know if there is any data to
  * read from the port, |eof?| to know if the end of file is reached
- * on the port and finally |close| to close the port. All theses
+ * on the port and finally |close| to close the port. All these
  * procedure takes one parameter which is the port from which the input
  * takes place.  |Open-input-virtual| accepts also the special value
  * |#f| for the I/O procedures with the following conventions:
- * ,(itemize
- *    (item [if |read-char| or |eof?| is |#f|, any attempt to read
- * the virtual port will return an eof object;])
- *    (item [if |ready?| is |#f|, the file is always  ready
- * for reading;])
- *    (item [if |close| is |#f|, no action is done when the port is
- * closed.]))
- * @l
+ *
+ * - if |read-char| or |eof?| is |#f|, any attempt to read
+ *   the virtual port will return an eof object;
+ *
+ * - if |ready?| is |#f|, the file is always  ready for reading;
+ *
+ * - if |close| is |#f|, no action is done when the port is closed.
+ *
  * Hereafter is a possible implementation of |open-input-string|
  * using virtual ports:
  * @lisp
@@ -250,10 +249,10 @@ static int vport_puts(char *s, void *stream)
  *   (let ((index 0))
  *     (open-input-virtual
  *        :read-char (lambda (p)
- *                  ;; test on eof is already done by the system
- *                  (let ((res (string-ref str index)))
- *                    (set! index (+ index 1))
- *                    res))
+ *                     ;; test on eof is already done by the system
+ *                     (let ((res (string-ref str index)))
+ *                       (set! index (+ index 1))
+ *                       res))
  *        :eof? (lambda (p) (>= index (string-length str))))))
  * @end lisp
 doc>
@@ -262,7 +261,7 @@ DEFINE_PRIMITIVE("%open-input-virtual", open_input_vport, subr1, (SCM v))
 {
   SCM z;
   struct vstream  *vs;
-  int flag = (STk_read_case_sensitive) ? PORT_CASE_SENSITIVE : 0;
+  int flag = PORT_CASE_SENSITIVE;
 
   if (!VECTORP(v) || VECTOR_SIZE(v) != 4) error_bad_vector(v, 4);
 
@@ -318,8 +317,8 @@ DEFINE_PRIMITIVE("%open-input-virtual", open_input_vport, subr1, (SCM v))
  * |Open-output-virtual| accepts also the special value |#f|
  * for the I/O procedures. If a procedure is |#f| nothing is done
  * on the corresponding action.
- * @l
- * Hereafter is an (very inefficient) implementation of a variant of
+ *
+ * Hereafter is a (very inefficient) implementation of a variant of
  * |open-output-string| using virtual ports. The value of the output
  * string is printed when the port is closed:
  * @lisp
@@ -327,17 +326,18 @@ DEFINE_PRIMITIVE("%open-input-virtual", open_input_vport, subr1, (SCM v))
  *   (let ((str ""))
  *     (open-output-virtual
  *        :write-char (lambda (c p)
- *                   (set! str (string-append str (string c))))
+ *                      (set! str (string-append str (string c))))
  *        :write-string (lambda (s p)
- *                     (set! str (string-append str s)))
+ *                        (set! str (string-append str s)))
  *        :close (lambda (p) (write str) (newline)))))
  * @end lisp
- * ,(bold "Note:") |write-string| is mainly used for writing strings and is
+ *
+ * NOTE: |write-string| is mainly used for writing strings and is
  * generally more efficient than writing the string character by character.
  * However, if |write-string| is not provided, strings are printed with
  * |write-char|.  On the other hand, if |write-char| is absent,
  * characters are written by successive allocation of one character strings.
- * @l
+ *
  * Hereafter is another example: a virtual file port where all characters
  * are converted to upper case:
  * @lisp
@@ -357,7 +357,7 @@ DEFINE_PRIMITIVE("%open-output-virtual", open_output_vport, subr1, (SCM v))
 {
   SCM z;
   struct vstream  *vs;
-  int flag = (STk_read_case_sensitive) ? PORT_CASE_SENSITIVE : 0;
+  int flag = PORT_CASE_SENSITIVE;
 
   if (!VECTORP(v) || VECTOR_SIZE(v) != 4) error_bad_vector(v, 4);
 
@@ -402,7 +402,7 @@ DEFINE_PRIMITIVE("%open-output-virtual", open_output_vport, subr1, (SCM v))
  * (output-virtual-port? obj)
  *
  * Returns |#t| if |obj| is a virtual input port or a virtual output port
- * respectively, otherwise returns #f.
+ * respectively, otherwise returns |#f|.
 doc>
  */
 DEFINE_PRIMITIVE("input-virtual-port?", input_vportp, subr1, (SCM obj))
@@ -425,4 +425,3 @@ int STk_init_vport(void)
   ADD_PRIMITIVE(output_vportp);
   return TRUE;
 }
-
